@@ -51,8 +51,7 @@ namespace ArrayCore {
     _rotateStep=2.0*M_PI/static_cast<double>(_rotateStepCount);
     _invRotateStep=1.0/_rotateStep;
 
-    _rotatedRayleigh=nullptr;
-    _rotatedLove=nullptr;
+    _rotatedMatrices=nullptr;
 
     resize();
   }
@@ -63,8 +62,7 @@ namespace ArrayCore {
   FKCrossSpectrum::~FKCrossSpectrum()
   {
     TRACE;
-    delete [] _rotatedRayleigh;
-    delete [] _rotatedLove;
+    delete [] _rotatedMatrices;
   }
 
   void FKCrossSpectrum::resize()
@@ -72,80 +70,50 @@ namespace ArrayCore {
     int stationCount=_stations.count();
     FKParameters::ProcessType processType;
     processType=static_cast<const FKParameters *>(_parameters)->processType();
-    switch(_mode) {
-    case ArrayStations::Vertical:
-      _rayleigh.resize(stationCount);
+    switch(processType) {
+    case FKParameters::Conventional:
+    case FKParameters::Capon:
+    case FKParameters::Omni:
+    case FKParameters::ActiveConventional:
+    case FKParameters::ActiveCapon:
+      _matrix.resize(stationCount);
       break;
-    case ArrayStations::Horizontal:
-      switch(processType) {
-      case FKParameters::DirectSteering:
-      case FKParameters::ActiveDirectSteering:
-      case FKParameters::DirectSteeringRadial:
-      case FKParameters::DirectSteeringRefined:
-      case FKParameters::DirectSteeringVertical:
-      case FKParameters::Omni:
-        _rayleigh.resize(2*stationCount);
-        _love.resize(2*stationCount);
-        break;
-      case FKParameters::Conventional:
-      case FKParameters::RTBF:
-      case FKParameters::RTBFRadial:
-      case FKParameters::ActiveRTBF:
-      case FKParameters::ActiveConventional:
-      case FKParameters::PoggiVertical:
-      case FKParameters::PoggiRadial:
-        delete [] _rotatedRayleigh;
-        delete [] _rotatedLove;
-        _rotatedRayleigh=new ComplexMatrix[_rotateStepCount];
-        _rotatedLove=new ComplexMatrix[_rotateStepCount];
-        for(int i=0; i<_rotateStepCount; i++) {
-          _rotatedRayleigh[i].resize(stationCount);
-          _rotatedLove[i].resize(stationCount);
-        }
+    case FKParameters::PoggiVertical:
+    case FKParameters::PoggiRadial:
+    case FKParameters::CaponRadial:
+    case FKParameters::CaponTransverse:
+    case FKParameters::ConventionalRadial:
+    case FKParameters::ConventionalTransverse:
+      delete [] _rotatedMatrices;
+      _rotatedMatrices=new ComplexMatrix[_rotateStepCount];
+      for(int i=0; i<_rotateStepCount; i++) {
+        _rotatedMatrices[i].resize(stationCount);
       }
       break;
-    case ArrayStations::ThreeComponents:
-      switch(processType) {
-      case FKParameters::Conventional:
-      case FKParameters::RTBF:
-      case FKParameters::RTBFRadial:
-        delete [] _rotatedRayleigh;
-        delete [] _rotatedLove;
-        _rotatedRayleigh=new ComplexMatrix[_rotateStepCount];
-        _rotatedLove=new ComplexMatrix[_rotateStepCount];
-        for(int i=0; i<_rotateStepCount; i++) {
-          _rotatedRayleigh[i].resize(2*stationCount);
-          _rotatedLove[i].resize(stationCount);
-        }
-        break;
-      case FKParameters::PoggiVertical:
-      case FKParameters::PoggiRadial:
-        delete [] _rotatedRayleigh;
-        delete [] _rotatedLove;
-        _rayleigh.resize(stationCount);
-        _rotatedRayleigh=new ComplexMatrix[_rotateStepCount];
-        _rotatedLove=new ComplexMatrix[_rotateStepCount];
-        for(int i=0; i<_rotateStepCount; i++) {
-          _rotatedRayleigh[i].resize(stationCount);
-          _rotatedLove[i].resize(stationCount);
-        }
-        break;
-      case FKParameters::DirectSteering:
-      case FKParameters::DirectSteeringRadial:
-      case FKParameters::DirectSteeringRefined:
-      case FKParameters::DirectSteeringVertical:
-      case FKParameters::ActiveDirectSteering:
-      case FKParameters::Omni:
-        _rayleigh.resize(3*stationCount);
-        _love.resize(2*stationCount);
-        break;
-      case FKParameters::ActiveConventional:
-      case FKParameters::ActiveRTBF:
-        _rayleigh.resize(2*stationCount);
-        _love.resize(stationCount);
-        break;
+    case FKParameters::ConventionalRayleigh:
+    case FKParameters::RTBF:
+    case FKParameters::RTBFRadial:
+    case FKParameters::RTBFFixedEll:
+    case FKParameters::ActiveConventionalRayleigh:
+    case FKParameters::ActiveRTBF:
+      delete [] _rotatedMatrices;
+      _rotatedMatrices=new ComplexMatrix[_rotateStepCount];
+      for(int i=0; i<_rotateStepCount; i++) {
+        _rotatedMatrices[i].resize(2*stationCount);
       }
       break;
+    case FKParameters::LDS:
+      _matrix.resize(2*stationCount);
+      break;
+    case FKParameters::RDS:
+    case FKParameters::RDSRadial:
+    case FKParameters::RDSVertical:
+    case FKParameters::RDSRefined:
+    case FKParameters::RDSSingle:
+    case FKParameters::RDSFixedEll:
+    case FKParameters::ActiveDS:
+      _matrix.resize(3*stationCount);
+      break;
     }
   }
 
@@ -225,9 +193,7 @@ namespace ArrayCore {
     return src;
   }
 
-  bool FKCrossSpectrum::calculate(const QVector<int>& blocks,
-                                  AbstractFKFunction * rayleigh,
-                                  AbstractFKFunction * love)
+  bool FKCrossSpectrum::calculate(const QVector<int>& blocks, AbstractFKFunction * function)
   {
     TRACE;
 #ifdef CROSS_SPECTRUM_TIMING
@@ -237,25 +203,14 @@ namespace ArrayCore {
     chrono.start();
 #endif
     // For active process, estimate the average source location
-    if(rayleigh) {
-      if(!rayleigh->setSource(blocks)) {
-        return false;
-      }
-    } else if(love) {
-      if(!love->setSource(blocks)) {
-        return false;
-      }
+    if(!function->setSource(blocks)) {
+      return false;
     }
 
 #ifdef CROSS_SPECTRUM_TIMING
     t1=chrono.elapsed();
 #endif
-    if(rayleigh) {
-      rayleigh->resetCrossSpectrum();
-    }
-    if(love) {
-      love->resetCrossSpectrum();
-    }
+    function->resetCrossSpectrum();
 
 #ifdef CROSS_SPECTRUM_TIMING
     t2=chrono.elapsed();
@@ -264,45 +219,36 @@ namespace ArrayCore {
 #ifdef CROSS_SPECTRUM_TIMING
     tBeforeLock=chrono.elapsed();
 #endif
-      if(lockTimeWindow(_timeWindows->list().at(*it), _timeWindows->plan())) {
+      if(lockTimeWindow(_timeWindows->list().at(*it))) {
 #ifdef CROSS_SPECTRUM_TIMING
     tLock+=chrono.elapsed()-tBeforeLock;
 #endif
-        if(rayleigh) {
-          rayleigh->addCrossSpectrum();
-        }
-        if(love) {
-          love->addCrossSpectrum();
-        }
-        unlockTimeWindow();
+        function->addCrossSpectrum();
+         unlockTimeWindow();
       }
     }
 #ifdef CROSS_SPECTRUM_TIMING
     t3=chrono.elapsed();
 #endif
 
-    if(rayleigh) {
-      rayleigh->meanCrossSpectrum(blocks.count());
-      if(!rayleigh->setFixedEllipticity(_timeWindows->frequency().center())) {
-        return false;
-      }
-    }
-    if(love) {
-      love->meanCrossSpectrum(blocks.count());
+    function->meanCrossSpectrum(blocks.count());
+    if(!function->setFixedEllipticity(_timeWindows->frequency().center())) {
+      return false;
     }
 
     // Experimental, mask built once with station name lists (1 or 0)
-    if(rayleigh && !_rayleighArrayMask.isEmpty()) {
-      _rayleigh=_rayleigh.dotMultiply(_rayleighArrayMask);
-      APP_LOG(4, _rayleigh.toOctaveString()+"\n")
-    }
-    if(love && !_loveArrayMask.isEmpty()) {
-      _love=_love.dotMultiply(_loveArrayMask);
+    if(!_arrayMask.isEmpty()) {
+      _matrix=_matrix.dotMultiply(_arrayMask);
+      APP_LOG(4, _matrix.toOctaveString()+"\n")
     }
 
     switch(parameters()->processType()) {
+    case FKParameters::RDSSingle:
     case FKParameters::Conventional:
+    case FKParameters::ConventionalRadial:
+    case FKParameters::ConventionalTransverse:
     case FKParameters::ActiveConventional:
+    case FKParameters::ActiveConventionalRayleigh:
       return true;
     default:
       break;
@@ -314,15 +260,8 @@ namespace ArrayCore {
     // Invert average cross spectra
     // Using Cholesky does not change speed for Vertical
     // Using Cholesky does change speed for 3-Component: from ~11' to 3'20" for a typical case.
-    if(rayleigh) {
-      if(!rayleigh->invertCrossSpectrum()) {
-        return false;
-      }
-    }
-    if(love) {
-      if(!love->invertCrossSpectrum()) {
-        return false;
-      }
+    if(!function->invertCrossSpectrum()) {
+      return false;
     }
 #ifdef CROSS_SPECTRUM_TIMING
     t5=chrono.elapsed();
@@ -332,17 +271,10 @@ namespace ArrayCore {
     return true;
   }
 
-  void FKCrossSpectrum::resetRotatedRayleigh()
+  void FKCrossSpectrum::resetRotatedMatrices()
   {
     for(int i=0; i<_rotateStepCount; i++) {
-      _rotatedRayleigh[i].zero(0.0);
-    }
-  }
-
-  void FKCrossSpectrum::resetRotatedLove()
-  {
-    for(int i=0; i<_rotateStepCount; i++) {
-      _rotatedLove[i].zero(0.0);
+      _rotatedMatrices[i].zero(0.0);
     }
   }
 
@@ -367,60 +299,29 @@ namespace ArrayCore {
     }
   }
 
-  void FKCrossSpectrum::meanRotatedRayleigh(int nBlocks)
+  void FKCrossSpectrum::meanRotatedMatrices(int nBlocks)
   {
     double invBlockCount=1.0/static_cast<double>(nBlocks);
     for(int i=0; i<_rotateStepCount; i++) {
-      _rotatedRayleigh[i]*=invBlockCount;
+      _rotatedMatrices[i]*=invBlockCount;
     }
   }
 
-  void FKCrossSpectrum::meanRotatedLove(int nBlocks)
+  bool FKCrossSpectrum::invert()
   {
-    double invBlockCount=1.0/static_cast<double>(nBlocks);
-    for(int i=0; i<_rotateStepCount; i++) {
-      _rotatedLove[i]*=invBlockCount;
-    }
-  }
-
-  bool FKCrossSpectrum::errorInvert(QString component)
-  {
-    TRACE;
-    App::log(tr("Cannot invert cross spectrum for %1\n").arg(component));
-    return false;
-  }
-
-  bool FKCrossSpectrum::invertRayleigh()
-  {
-    if(!invert(_rayleigh)) {
-      return errorInvert(Mode::userPolarization(Mode::Rayleigh));
+    if(!invert(_matrix)) {
+      App::log(tr("Cannot invert cross spectrum\n"));
+      return false;
     }
     return true;
   }
 
-  bool FKCrossSpectrum::invertLove()
-  {
-    if(!invert(_love)) {
-      return errorInvert(Mode::userPolarization(Mode::Love));
-    }
-    return true;
-  }
-
-  bool FKCrossSpectrum::invertRotatedRayleigh()
+  bool FKCrossSpectrum::invertRotatedMatrices()
   {
     for(int i=0; i<_rotateStepCount; i++) {
-      if(!invert(_rotatedRayleigh[i])) {
-        return errorInvert(Mode::userPolarization(Mode::Radial));
-      }
-    }
-    return true;
-  }
-
-  bool FKCrossSpectrum::invertRotatedLove()
-  {
-    for(int i=0; i<_rotateStepCount; i++) {
-      if(!invert(_rotatedLove[i])) {
-        return errorInvert(Mode::userPolarization(Mode::Transverse));
+      if(!invert(_rotatedMatrices[i])) {
+        App::log(tr("Cannot invert cross spectrum for direction %1\n").arg(i));
+        return false;
       }
     }
     return true;
@@ -432,7 +333,7 @@ namespace ArrayCore {
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
     const ArrayStationSignals * stat;
-    ComplexMatrix& covmat=_rotatedRayleigh[rotationIndex];
+    ComplexMatrix& covmat=_rotatedMatrices[rotationIndex];
     ComplexMatrix sig(stationCount, 1);
     Angle rotation;
     rotation.setRadians(_rotateStep*rotationIndex);
@@ -499,7 +400,7 @@ namespace ArrayCore {
     }*/
 
     const ArrayStationSignals * stat;
-    ComplexMatrix& covmat=_rotatedLove[rotationIndex];
+    ComplexMatrix& covmat=_rotatedMatrices[rotationIndex];
     ComplexMatrix sig(stationCount, 1);
     Angle rotation;
     rotation.setRadians(_rotateStep*rotationIndex);
@@ -544,7 +445,7 @@ namespace ArrayCore {
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
     const ArrayStationSignals * stat;
-    ComplexMatrix& covmat=_rotatedRayleigh[rotationIndex];
+    ComplexMatrix& covmat=_rotatedMatrices[rotationIndex];
     ComplexMatrix sig(2*stationCount, 1);
     Angle rotation;
     rotation.setRadians(_rotateStep*rotationIndex);
@@ -587,7 +488,7 @@ namespace ArrayCore {
     int stationCount2=2*stationCount;
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
-    ComplexMatrix& covmat=_rayleigh;
+    ComplexMatrix& covmat=_matrix;
     ComplexMatrix sig(3*stationCount, 1);
     if(parameters()->frequencyBandwidth()==0.0) {
       for(int is=0; is<stationCount; is++) {
@@ -685,7 +586,7 @@ namespace ArrayCore {
     int stationCount=_stations.count();
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
-    ComplexMatrix& covmat=_rayleigh;
+    ComplexMatrix& covmat=_matrix;
     ComplexMatrix sig(stationCount, 1);
 #ifndef COMPATIBILITY_2_5_0
     if(parameters()->frequencyBandwidth()==0.0) {
@@ -723,7 +624,7 @@ namespace ArrayCore {
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
     const ArrayStationSignals * stat;
-    ComplexMatrix& covmat=_rayleigh;
+    ComplexMatrix& covmat=_matrix;
     ComplexMatrix sig(2*stationCount, 1);
     if(parameters()->frequencyBandwidth()==0.0) {
       for(int is=0; is<stationCount; is++) {
@@ -766,7 +667,7 @@ namespace ArrayCore {
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
     const ArrayStationSignals * stat;
-    ComplexMatrix& covmat=_rayleigh;
+    ComplexMatrix& covmat=_matrix;
     ComplexMatrix sig(stationCount, 1);
     if(parameters()->frequencyBandwidth()==0.0) {
       for(int is=0; is<stationCount; is++) {
@@ -800,7 +701,7 @@ namespace ArrayCore {
     int stationCount2=2*stationCount;
     const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
 
-    ComplexMatrix& covmat=_rayleigh;
+    ComplexMatrix& covmat=_matrix;
     ComplexMatrix sig(3*stationCount, 1);
     if(parameters()->frequencyBandwidth()==0.0) {
     } else {
@@ -860,6 +761,7 @@ namespace ArrayCore {
         covmat.at(ir, ir)+=d;
       }
     }
+    //ComplexMatrix original=covmat;
     covmat.singularValue(sigma, u, vt);
     double sRef=1e-12*sigma.first();
     for(int row=0; row<n ;row++ ) {
@@ -870,60 +772,50 @@ namespace ArrayCore {
         return false;
       }
     }
-    return covmat.invert(u, sigma, vt);
+    if(!covmat.invert(u, sigma, vt)) {
+      return false;
+    }
+
+    /*original*=covmat;
+    double sum=0.0;
+    for(int i=original.rowCount()-1; i>=0; i--) {
+      for(int j=original.columnCount()-1; j>i; j--) {
+        sum+=original.at(i, j).abs();
+      }
+      sum+=original.at(i, i).abs()-1.0;
+      for(int j=i-1; j>=0; j--) {
+        sum+=original.at(i, j).abs();
+      }
+    }
+    QTextStream(stdout) << sum;
+    exit(2);*/
+    return true;
   }
 
-  bool FKCrossSpectrum::setMask(AbstractFKFunction::Type t)
+  bool FKCrossSpectrum::setMask()
   {
     TRACE;
     FKParameters::ProcessType processType;
     processType=static_cast<const FKParameters *>(_parameters)->processType();
-    switch(t) {
-    case AbstractFKFunction::Rayleigh:
-      switch(_mode) {
-      case ArrayStations::Vertical:
-        switch(processType) {
-        case FKParameters::Conventional:
-        case FKParameters::Omni:
-        case FKParameters::ActiveRTBF:
-        case FKParameters::ActiveDirectSteering:
-        case FKParameters::ActiveConventional:
-          break;
-        case FKParameters::DirectSteering:
-        case FKParameters::DirectSteeringRadial:
-        case FKParameters::DirectSteeringRefined:
-        case FKParameters::DirectSteeringVertical:
-        case FKParameters::RTBF:
-        case FKParameters::RTBFRadial:
-        case FKParameters::PoggiVertical:
-        case FKParameters::PoggiRadial:
-          return setOneComponentArrayMask();
-        }
+    switch(_mode) {
+    case ArrayStations::Vertical:
+      switch(processType) {
+      case FKParameters::Conventional:
+      case FKParameters::Capon:
+        return setOneComponentArrayMask();
+      default:
         break;
-      case ArrayStations::Horizontal:
-        break;
-      case ArrayStations::ThreeComponents:
-        switch(processType) {
-        case FKParameters::Conventional:
-        case FKParameters::RTBF:
-        case FKParameters::RTBFRadial:
-        case FKParameters::DirectSteeringRadial:
-        case FKParameters::DirectSteeringVertical:
-        case FKParameters::DirectSteeringRefined:
-        case FKParameters::Omni:
-        case FKParameters::PoggiVertical:
-        case FKParameters::PoggiRadial:
-        case FKParameters::ActiveRTBF:
-        case FKParameters::ActiveDirectSteering:
-        case FKParameters::ActiveConventional:
-          break;
-        case FKParameters::DirectSteering:
-          return setRayleighArrayMask();
-        }
       }
       break;
-    case AbstractFKFunction::RayleighFixedEll:
-    case AbstractFKFunction::Love:
+    case ArrayStations::Horizontal:
+      break;
+    case ArrayStations::ThreeComponents:
+      switch(processType) {
+      case FKParameters::RDS:
+        return setRayleighArrayMask();
+      default:
+        break;
+      }
       break;
     }
     return false;
@@ -941,8 +833,8 @@ namespace ArrayCore {
     for(int i=n-1; i>=0; i--) {
       stationIndex.insert(_stations.at(i)->originalSignals()->name(), i);
     }
-    _rayleighArrayMask.resize(3*n);
-    _rayleighArrayMask.zero(0.0);
+    _arrayMask.resize(3*n);
+    _arrayMask.zero(0.0);
     QList<QStringList>::const_iterator ita;
     APP_LOG(1, tr("Mapping station names to index...\n"))
     for(ita=arrays.begin(); ita!=arrays.end(); ita++) {
@@ -963,7 +855,7 @@ namespace ArrayCore {
           int ic0=*itc;
           for(int ir=2*n+ir0; ir>=ir0; ir-=n) {
             for(int ic=2*n+ic0; ic>=ic0; ic-=n) {
-              double& v=_rayleighArrayMask.at(ir, ic);
+              double& v=_arrayMask.at(ir, ic);
               if(v==0.0) {
                 v=1;
               }
@@ -972,7 +864,7 @@ namespace ArrayCore {
         }
       }
     }
-    APP_LOG(4, _rayleighArrayMask.toUserString())
+    APP_LOG(4, _arrayMask.toUserString())
     return true;
   }
 
@@ -988,8 +880,8 @@ namespace ArrayCore {
     for(int i=n-1; i>=0; i--) {
       stationIndex.insert(_stations.at(i)->originalSignals()->name(), i);
     }
-    _rayleighArrayMask.resize(n);
-    _rayleighArrayMask.zero(0.0);
+    _arrayMask.resize(n);
+    _arrayMask.zero(0.0);
     QList<QStringList>::const_iterator ita;
     APP_LOG(1, tr("Mapping station names to index...\n"))
     for(ita=arrays.begin(); ita!=arrays.end(); ita++) {
@@ -1007,27 +899,81 @@ namespace ArrayCore {
       }
       for(QVector<int>::const_iterator itr=index.begin(); itr!=index.end(); itr++) {
         for(QVector<int>::const_iterator itc=index.begin(); itc!=index.end(); itc++) {
-          double& v=_rayleighArrayMask.at(*itr, *itc);
+          double& v=_arrayMask.at(*itr, *itc);
           if(v==0.0) {
             v=1;
           }
         }
       }
     }
-    APP_LOG(4, _rayleighArrayMask.toOctaveString()+"\n")
+    APP_LOG(4, _arrayMask.toOctaveString()+"\n")
     return true;
   }
 
   void FKCrossSpectrum::setEllipticity(const ComplexMatrix& ell)
   {
-    if(_rotatedRayleigh) {
+    if(_rotatedMatrices) {
       for(int i=0; i<_rotateStepCount; i++) {
-        _rotatedRayleigh[i]=_rotatedRayleigh[i].dotMultiply(ell);
+        _rotatedMatrices[i]=_rotatedMatrices[i].dotMultiply(ell);
       }
     } else {
-      _rayleigh=_rayleigh.dotMultiply(ell);
+      _matrix=_matrix.dotMultiply(ell);
     }
   }
 
+  void FKCrossSpectrum::addRayleighSingle()
+  {
+    Complex east, north, vertical;
+    ASSERT(_stations.count()==1);
+    const GaussianFrequencyBand& frequencyFilter=_timeWindows->frequencyFilter();
+    if(parameters()->frequencyBandwidth()==0.0) {
+      east=_stations.at(0)->spectrumValue(_eastIndex);
+      north=_stations.at(0)->spectrumValue(_northIndex);
+      vertical=_stations.at(0)->spectrumValue(_verticalIndex);
+    } else {
+      int nFreq=frequencyFilter.maximumIndex();
+      double filter;
+      for(int iFreq=frequencyFilter.minimumIndex(); iFreq<=nFreq; iFreq++) {
+        filter=frequencyFilter.taperValue(iFreq);
+        east=_stations.at(0)->spectrumValue(_eastIndex, iFreq)*filter;
+        north=_stations.at(0)->spectrumValue(_northIndex, iFreq)*filter;
+        vertical=_stations.at(0)->spectrumValue(_verticalIndex, iFreq)*filter;
+      }
+    }
+    Complex conjVertical=conjugate(vertical);
+    Complex crossEV=east*conjVertical;
+    Complex crossNV=north*conjVertical;
+    // Find horizontal propagation direction that provides a cross product with a null real part
+    // Its imaginary part should be -|S_z|^2 je+... write the noise and Love contributions
+    double theta=atan(-crossEV.abs()/crossNV.abs()*
+                      cos(crossEV.phase())/cos(crossNV.phase()));
+    Complex h=cos(theta)*east+sin(theta)*north;
+    double crossH=(h*conjVertical).im();
+    // Make sure that delay between h and vertical has always the same sign
+    Complex delay;
+    if(crossH>0.0) {
+      delay.setUnitExp(-vertical.phase());
+    } else {
+      delay.setUnitExp(vertical.phase());
+    }
+     // Set a null phase for vertical: real for vertical and imaginary positive for horizontal
+    h*=delay;
+    vertical*=delay;
+    _matrix.at(0, 0)+=h;
+    _matrix.at(0, 1)+=vertical;
+    // TODO:
+    //  - check that the stupid stack of abs values is as good
+    //  - write it in a dedicated SingleEllipticityCrossSpectrum avoid fake grid search
+    //  - check also average of ellipticity values (crossH/vertical.abs2())
+    //  - number of blocks driven be stability of ellipticity
+    //  - check that correlation coefficient from Hobiger is always 1, hence useless.
+    //  - show effect of bandwidth
+  }
+
+  void FKCrossSpectrum::meanRayleighSingle()
+  {
+    _matrix.at(2, 2)=sqrt(_matrix.at(0, 0).abs2()/_matrix.at(0, 1).abs2());
+  }
+
 } // namespace ArrayCore
 
