@@ -38,17 +38,17 @@ namespace ArrayCore {
   FKParameters::FKParameters()
     : ArrayParameters()
   {
-    setDefaultValues(DirectSteering, RefinedGrid, ArrayStations::Vertical);
+    setDefaultValues(RTBF, RefinedGrid);
     _cacheGridStepFactor=0.05;
     _cacheGridStep=0.0;
     _gridStep=0.0;
-    _gridSize=0;
+    _gridSize=0.0;
     _kmin=0.0;
     _kmax=0.0;
     _smin=1.0/3500.0;
     _smax=1.0/50.0;
     _minAzimuth=0.0;
-    _maxAzimuth=360.0;
+    _maxAzimuth=0.0;
     _sourceGridStep=1.0;
     _sourceGridSize=0.0;
   }
@@ -61,6 +61,7 @@ namespace ArrayCore {
     _cacheGridStep=o._cacheGridStep;
     _gridStepFactor=o._gridStepFactor;
     _gridStep=o._gridStep;
+    _gridSizeFactor=o._gridSizeFactor;
     _gridSize=o._gridSize;
     _kmin=o._kmin;
     _kmax=o._kmax;
@@ -68,18 +69,17 @@ namespace ArrayCore {
     _smax=o._smax;
     _minAzimuth=o._minAzimuth;
     _maxAzimuth=o._maxAzimuth;
-    _refineGridIndex=o._refineGridIndex;
     _refineSlowMin=o._refineSlowMin;
     _refineSlowMax=o._refineSlowMax;
     _maxPeakCount=o._maxPeakCount;
     _absoluteThreshold=o._absoluteThreshold;
     _relativeThreshold=o._relativeThreshold;
+    _saveUndefinedEllipticities=o._saveUndefinedEllipticities;
     _exportAllFKGrids=o._exportAllFKGrids;
     _damping=o._damping;
     _rotateStepCount=o._rotateStepCount;
     _processType=o._processType;
     _arrays=o._arrays;
-    _skipLove=o._skipLove;
     _fixedEllipticityFileName=o._fixedEllipticityFileName;
     _minimumDistance=o._minimumDistance;
     _maximumDistance=o._maximumDistance;
@@ -87,29 +87,19 @@ namespace ArrayCore {
     _sourceGridSize=o._sourceGridSize;
   }
 
-  void FKParameters::setDefaultValues(ProcessType t, InversionMethod invm, ArrayStations::Mode m)
+  void FKParameters::setDefaultValues(ProcessType t, InversionMethod invm)
   {
-    int n=1;
-    switch(m) {
-    case ArrayStations::Vertical:
-      break;
-    case ArrayStations::Horizontal:
-      n=2;
-      break;
-    case ArrayStations::ThreeComponents:
-      n=3;
-      break;
-    }
     _processType=t;
     _maxPeakCount=INT_MAX; // Avoid sensitivity to negligible differences between
-                           // peak amplitudes. Accept by default all peaks down to
-                           // 90% of the maximum amplitude.
+                           // peak amplitudes. Accept all peaks down to
+                           // 90% of the maximum amplitude rather than a single value
+                           // If null ellipticities are not saved, max file size is kept
+                           // reasonable even without filter (RTBF and RDS).
     _absoluteThreshold=0.0;
     _relativeThreshold=90.0;
 
+    _saveUndefinedEllipticities=false;
     _exportAllFKGrids=false;
-    _refineGridIndex=-1;
-    _skipLove=false;
     _damping=0.0;
     _rotateStepCount=72;
     setfrequencyBandwidth(0.05);
@@ -117,16 +107,7 @@ namespace ArrayCore {
     blockAveraging().setStatisticMaxOverlap(0.0);
 
     switch(_processType) {
-    case DirectSteering:
-    case DirectSteeringRadial:
-    case DirectSteeringVertical:
-    case DirectSteeringRefined:
-    case Omni:
-    case PoggiRadial:
-    case PoggiVertical:
-    case RTBF:
-    case RTBFRadial:
-    case Conventional:
+    FKPARAMETERS_PASSIVE_PROCESS_TYPES
       windowing().setPeriodCount(100.0);
       setOversamplingFactor(1.0);
       _minimumDistance=0.0;
@@ -134,9 +115,7 @@ namespace ArrayCore {
       windowing().setSeismicEventTrigger(false);
       setSelectDurationFactor(0.0);
       break;
-    case ActiveDirectSteering:
-    case ActiveRTBF:
-    case ActiveConventional:
+    FKPARAMETERS_ACTIVE_PROCESS_TYPES
       windowing().setLength(5.0);
       setOversamplingFactor(5.0);
       _minimumDistance=10.0;
@@ -148,25 +127,44 @@ namespace ArrayCore {
     }
 
     switch(_processType) {
-    case DirectSteering:
-    case DirectSteeringRadial:
-    case DirectSteeringVertical:
-    case DirectSteeringRefined:
+    case RDS:
+    case RDSRadial:
+    case RDSVertical:
+    case RDSRefined:
+    case RDSSingle:
+    case RDSFixedEll:
+      blockAveraging().setCount(0);
+      blockAveraging().setCountFactor(6.0);
+      _relativeThreshold=0.0; //
+      break;
+    case Capon:
+    case CaponRadial:
+    case CaponTransverse:
     case Omni:
     case PoggiRadial:
     case PoggiVertical:
       blockAveraging().setCount(0);
-      blockAveraging().setCountFactor(2.0*n);
+      blockAveraging().setCountFactor(2.0);
       break;
     case RTBF:
     case RTBFRadial:
+    case RTBFFixedEll:
       blockAveraging().setCount(0);
-      blockAveraging().setCountFactor(n==1 ? 2.0 : 2.0*(n-1));
+      blockAveraging().setCountFactor(4.0);
+      _relativeThreshold=0.0;
       invm=RefinedGrid; // Gradient not supported for RTBF
       break;
-    case Conventional:
+    case LDS:
       blockAveraging().setCount(0);
-      blockAveraging().setCountFactor(n==1 ? 0.5 : 0.5*(n-1));
+      blockAveraging().setCountFactor(4.0);
+      break;
+    case Conventional:
+    case ConventionalRadial:
+    case ConventionalTransverse:
+    case ConventionalRayleigh:
+    case ActiveConventionalRayleigh:
+      blockAveraging().setCount(0);
+      blockAveraging().setCountFactor(0.5);
       setMaximumPeakCount(1);
       break;
     case ActiveConventional:
@@ -174,12 +172,14 @@ namespace ArrayCore {
       setMaximumPeakCount(1);
       break;
     case ActiveRTBF:
-    case ActiveDirectSteering:
+    case ActiveDS:
+    case ActiveCapon:
       blockAveraging().setCount(3);
       break;
     }
     _inversionMethod=invm;
     _gridStepFactor=defaultGridStepFactor(t, _inversionMethod);
+    _gridSizeFactor=2.0;
     // Force default grid step and size if they are not yet set
     setKmin(_kmin);
     setKmax(_kmax);
@@ -188,17 +188,25 @@ namespace ArrayCore {
   double FKParameters::defaultGridStepFactor(ProcessType t, InversionMethod m)
   {
     switch(t) {
-    case DirectSteering:
-    case DirectSteeringRadial:
-    case DirectSteeringVertical:
-    case DirectSteeringRefined:
+    case Capon:
     case Omni:
-    case PoggiRadial:
-    case PoggiVertical:
+    case CaponRadial:
+    case CaponTransverse:
     case RTBF:
     case RTBFRadial:
+    case RTBFFixedEll:
+    case RDS:
+    case RDSRadial:
+    case RDSVertical:
+    case RDSRefined:
+    case RDSSingle:
+    case RDSFixedEll:
+    case PoggiVertical:
+    case PoggiRadial:
+    case LDS:
+    case ActiveCapon:
     case ActiveRTBF:
-    case ActiveDirectSteering:
+    case ActiveDS:
       switch(m) {
       case Gradient:
         return 0.5;
@@ -206,7 +214,11 @@ namespace ArrayCore {
         return 0.1;
       }
     case Conventional:
+    case ConventionalRadial:
+    case ConventionalTransverse:
+    case ConventionalRayleigh:
     case ActiveConventional:
+    case ActiveConventionalRayleigh:
       break;
     }
     switch(m) {
@@ -218,11 +230,6 @@ namespace ArrayCore {
     return 1.0;
   }
 
-  double FKParameters::gridSize(double kmax)
-  {
-    return kmax*2.0;
-  }
-
   double FKParameters::cacheGridStep() const
   {
     if(_cacheGridStep==0.0) {
@@ -241,6 +248,15 @@ namespace ArrayCore {
     }
   }
 
+  double FKParameters::gridSize() const
+  {
+    if(_gridSize==0.0) {
+      return _kmax*_gridSizeFactor;
+    } else {
+      return _gridSize;
+    }
+  }
+
   void FKParameters::setKmin(double k)
   {
     _kmin=k;
@@ -249,9 +265,6 @@ namespace ArrayCore {
   void FKParameters::setKmax(double k)
   {
     _kmax=k;
-    if(_gridSize==0.0) {
-      _gridSize=gridSize(_kmax);
-    }
   }
 
   /*!
@@ -260,11 +273,12 @@ namespace ArrayCore {
   double FKParameters::effectiveGridSize() const
   {
     double ksmax=2.0*M_PI*frequencySampling().maximum()*_smax;
+    double gs=gridSize();
     // For the GUI frequency sampling maximum is usually null
-    if(ksmax<_gridSize && ksmax>0.0) {
+    if(ksmax<gs && ksmax>0.0) {
       return ksmax;
     } else {
-      return _gridSize;
+      return gs;
     }
   }
 
@@ -276,10 +290,11 @@ namespace ArrayCore {
   {
     TRACE;
     double ksmax=2.0*M_PI*frequency*_smax;
-    if(ksmax<_gridSize) {
+    double gs=gridSize();
+    if(ksmax<gs) {
       return ksmax;
     } else {
-      return _gridSize;
+      return gs;
     }
   }
 
@@ -290,19 +305,6 @@ namespace ArrayCore {
     _refineSlowMax.clear();
   }
 
-  /*!
-    \a gridIndex can be -1, 0 or 1.
-      \li -1 to switch off refine computation (full frequency range is computed)
-      \li 0 Rayleigh grid
-      \li 1 Love grid
-  */
-  void FKParameters::FKParameters::setRefineGridIndex(int gridIndex)
-  {
-    TRACE;
-    ASSERT(gridIndex>=-1 || gridIndex<=1);
-    _refineGridIndex=gridIndex;
-  }
-
   void FKParameters::addCurveRefine(double f, double minSlow, double maxSlow)
   {
     TRACE;
@@ -312,7 +314,7 @@ namespace ArrayCore {
 
   int FKParameters::keywordCount(PARAMETERS_KEYWORDCOUNT_ARGS) const
   {
-    return 27+ArrayParameters::keywordCount();
+    return 29+ArrayParameters::keywordCount();
   }
 
   void FKParameters::collectKeywords(PARAMETERS_COLLECTKEYWORDS_ARGS)
@@ -326,6 +328,7 @@ namespace ArrayCore {
     keywords.add(prefix+"GRID_STEP"+suffix, this, baseIndex+16);
     keywords.add(prefix+"GRID_STEP_FACTOR"+suffix, this, baseIndex+18);
     keywords.add(prefix+"GRID_SIZE"+suffix, this, baseIndex+17);
+    keywords.add(prefix+"GRID_SIZE_FACTOR"+suffix, this, baseIndex+28);
     keywords.add(prefix+"K_MIN"+suffix, this, baseIndex);
     keywords.add(prefix+"K_MAX"+suffix, this, baseIndex+1);
     keywords.add(prefix+"MIN_V"+suffix, this, baseIndex+2);
@@ -336,11 +339,12 @@ namespace ArrayCore {
     keywords.add(prefix+"ABSOLUTE_THRESHOLD"+suffix, this, baseIndex+4);
     keywords.add(prefix+"RELATIVE_THRESHOLD"+suffix, this, baseIndex+5);
     keywords.add(prefix+"EXPORT_ALL_FK_GRIDS"+suffix, this, baseIndex+6);
+    keywords.add(prefix+"SAVE_UNDEFINED_ELLIPTICITIES"+suffix, this, baseIndex+27);
     keywords.add(prefix+"DAMPING_FACTOR"+suffix, this, baseIndex+7);
     keywords.add(prefix+"ROTATE_STEP_COUNT"+suffix, this, baseIndex+8);
     keywords.add(prefix+"PROCESS_TYPE"+suffix, this, baseIndex+9);
     keywords.add(prefix+"ARRAY"+suffix, this, baseIndex+10);
-    keywords.add(prefix+"SKIP_LOVE"+suffix, this, baseIndex+11);
+    keywords.add(prefix+"SKIP_LOVE"+suffix, this, baseIndex+11);        // For compatibility
     keywords.add(prefix+"FIXED_ELLIPTICITY_FILE_NAME"+suffix, this, baseIndex+12);
     keywords.add(prefix+"MINIMUM_DISTANCE"+suffix, this, baseIndex+13);
     keywords.add(prefix+"MAXIMUM_DISTANCE"+suffix, this, baseIndex+14);
@@ -354,74 +358,83 @@ namespace ArrayCore {
     TRACE;
     QString log;
     log+=ArrayParameters::toString(prefix, suffix);
+    log+="#\n"
+         "#\n"
+         "#     FK method\n"
+         "#\n"
+         "#\n";
     log+="# Process types:\n"
-         "#  [All types can be used with vertical or three component datasets]\n"
-         "#  Keyword                Beamformer    Comments\n"
-         "#  DirectSteering         Capon         Cross spectrum made of raw components E, N and Z.\n"
-         "#                                       Radial and transverse projections included in steering matrix.\n"
-         "#                                       Combined optimum power.\n"
-         "#  Omni                   Capon         Same cross spectrum as DirectSteering.\n"
-         "#                                       Ouput power is the sum of power in all directions\n"
-         "#  RTBF                   Capon         According to Wathelet et al (2018).\n"
-         "#                                       Cross spectrum made of radial and transverse projections.\n"
-         "#  PoggiVertical          Capon         According Poggi et al. (2010)\n"
-         "#                                       k picked from vertical processing\n"
-         "#  PoggiRadial            Capon         According Poggi et al. (2010)\n"
-         "#                                       k picked from radial processing\n"
-         "#  Conventional           Conventional  Conventional FK processing\n"
-         "#                                       Cross spectrum made of radial and transverse projections.\n"
-         "#  ActiveRTBF             Capon         High resolution for active source\n"
-         "#                                       Cross spectrum made of radial and transverse projections.\n"
-         "#  ActiveDirectSteering   Capon         Cross spectrum made of raw components E, N and Z.\n"
-         "#                                       Radial and transverse projections included in steering matrix.\n"
-         "#  ActiveConventional     Conventional  Conventional FK processing\n"
-         "#                                       Cross spectrum made of radial and transverse projections.\n"
+         "#  [Comp is the required components]\n"
+         "#  Keyword                  Comp Comments\n"
+         "#  Conventional             Z    Conventional FK processing.\n"
+         "#  ConventionalRadial       EN   Conventional FK processing for radial projections.\n"
+         "#  ConventionalTransverse   EN   Conventional FK processing for transverse projections.\n"
+         "#  ConventionalRayleigh     ENZ  Conventional FK processing for radial projections.\n"
+         "#  Capon                    Z    High resolution FK processing (Capon, 1969).\n"
+         "#  CaponRadial              EN   High resolution FK processing (Capon, 1969) for radial projections.\n"
+         "#  CaponTransverse          EN   High resolution FK processing (Capon, 1969) for transverse projections.\n"
+         "#  RTBF                     ENZ  According to Wathelet et al (2018).\n"
+         "#                                Cross spectrum made of radial projections and vertical.\n"
+         "#                                Combined radial and vertical ellipticity steering.\n"
+         "#  RTBFRadial               ENZ  According to Wathelet et al (2018).\n"
+         "#                                Cross spectrum made of radial projections and vertical.\n"
+         "#                                Radial ellipticity steering, better for small ellipticities.\n"
+         "#  PoggiVertical            ENZ  According Poggi et al. (2010)\n"
+         "#                                k picked from vertical processing.\n"
+         "#  PoggiRadial              ENZ  According Poggi et al. (2010)\n"
+         "#                                k picked from radial processing.\n"
+         "#  RDS                      ENZ  Rayleigh Direct Steering.\n"
+         "#                                Cross spectrum made of raw components E, N and Z.\n"
+         "#                                Radial projections included in the steering matrix.\n"
+         "#                                Combined radial and vertical ellipticity steering.\n"
+         "#  LDS                      EN   Love Direct Steering.\n"
+         "#                                Cross spectrum made of raw components E and N.\n"
+         "#                                Transverse projections included in the steering matrix.\n"
+         "#  RDSRadial                ENZ  Rayleigh Direct Steering.\n"
+         "#                                Cross spectrum made of raw components E, N and Z.\n"
+         "#                                Radial projections included in the steering matrix.\n"
+         "#                                Radial ellipticity steering, better for small ellipticities.\n"
          "#  Experimental modes:\n"
-         "#  DirectSteeringVertical Capon         Cross spectrum made of raw components E, N and Z.\n"
-         "#                                       Radial and transverse projections included in steering matrix.\n"
-         "#                                       Radial ellipticity steering.\n"
-         "#  DirectSteeringRadial   Capon         Cross spectrum made of raw components E, N and Z.\n"
-         "#                                       Radial and transverse projections included in steering matrix.\n"
-         "#                                       Vertical ellipticity steering.\n"
-         "#  DirectSteeringRefined  Capon         Cross spectrum made of raw components E, N and Z.\n"
-         "#                                       Radial and transverse projections included in steering matrix.\n"
-         "#                                       Iterative ellitpticity assessment.\n";
+         "#  RTBFFixedEll             ENZ  Same as RTBF but ellipticity is fixed.\n"
+         "#                                FIXED_ELLIPTICITY_FILE_NAME must be provided.\n"
+         "#  RDSFixedEll              ENZ  Same as RDS but ellipticity is fixed.\n"
+         "#                                FIXED_ELLIPTICITY_FILE_NAME must be provided.\n"
+         "#  ActiveConventional       Z    Conventional FK processing for active sources.\n"
+         "#  ActiveRTBF               ENZ  RTBF for active sources\n"
+         "#                                Cross spectrum made of radial and transverse projections.\n"
+         "#  ActiveDS                 ENZ  Cross spectrum made of raw components E, N and Z.\n"
+         "#                                Radial and transverse projections included in steering matrix.\n"
+         "#  ActiveConventional            Conventional FK processing for active sources\n"
+         "#                                Cross spectrum made of radial and transverse projections.\n"
+         "#  Omni                     ENZ  Same cross spectrum as RDS.\n"
+         "#                                Ouput power is the sum of power in all directions\n"
+         "#  RDSVertical              ENZ  Cross spectrum made of raw components E, N and Z.\n"
+         "#                                Radial and transverse projections included in steering matrix.\n"
+         "#                                Radial ellipticity steering.\n"
+         "#  RDSRadial                ENZ  Cross spectrum made of raw components E, N and Z.\n"
+         "#                                Radial and transverse projections included in steering matrix.\n"
+         "#                                Vertical ellipticity steering.\n"
+         "#  RDSRefined               ENZ  Cross spectrum made of raw components E, N and Z.\n"
+         "#                                Radial and transverse projections included in steering matrix.\n"
+         "#                                Iterative ellitpticity assessment.\n";
     log+=prefix+"PROCESS_TYPE"+suffix+"="+processTypeString()+"\n";
-    log+="# For debug purpose, save a bit of time by skipping Love computation\n";
-    log+=prefix+"SKIP_LOVE"+suffix+"(y/n)="+(_skipLove ? "y" : "n")+"\n";
-
-    log+="# Inversion method used for getting FK peaks: Gradient or RefinedGrid\n";
-    log+=prefix+"INVERSION_METHOD"+suffix+"="+inversionMethodString()+"\n";
-
-    log+="# Wavenumber fine gridding used as a cache for the FK maps\n";
-    log+=prefix+"CACHE_GRID_STEP"+suffix+" (rad/m)="+QString::number(_cacheGridStep)+"\n";
-    log+="# If CACHE_GRID_STEP is null, GRID_STEP is computed from K_MIN*CACHE_GRID_STEP_FACTOR.\n";
-    log+=prefix+"CACHE_GRID_STEP_FACTOR"+suffix+"="+QString::number(_cacheGridStepFactor)+"\n";
-
-    log+="# Wavenumber coarse gridding used for searching maxima of the FK maps\n";
-    log+=prefix+"GRID_STEP"+suffix+" (rad/m)="+QString::number(_gridStep)+"\n";
-    log+="# If GRID_STEP is null, GRID_STEP is computed from K_MIN*GRID_STEP_FACTOR.\n";
-    log+=prefix+"GRID_STEP_FACTOR"+suffix+"="+QString::number(_gridStepFactor)+"\n";
-    log+=prefix+"GRID_SIZE"+suffix+" (rad/m)="+QString::number(_gridSize)+"\n";
-
-    log+="# Minimum velocity of the searched maxima of the FK map\n";
-    log+=prefix+"MIN_V"+suffix+" (m/s)="+QString::number(1.0/_smax)+"\n";
-    log+="# Maximum velocity of the searched maxima of the FK map\n";
-    log+=prefix+"MAX_V"+suffix+" (m/s)="+QString::number(1.0/_smin)+"\n";
-
-    log+="# Minimum azimuth of the searched maxima of the FK map (math\n";
-    log+=prefix+"MIN_AZIMUTH"+suffix+" (deg.)="+QString::number(_minAzimuth)+"\n";
-    log+="# Maximum azimith of the searched maxima of the FK map (math)\n";
-    log+=prefix+"MAX_AZIMUTH"+suffix+" (deg.)="+QString::number(_maxAzimuth)+"\n";
-
-    log+="# Theoretical Kmin and Kmax computed from array geometry\n";
-    log+="# Used only for post-processing (AVIOS project)\n";
-    log+=prefix+"K_MIN"+suffix+" (rad/m)="+QString::number(_kmin)+"\n";
-    log+=prefix+"K_MAX"+suffix+" (rad/m)="+QString::number(_kmax)+"\n";
-    log+=prefix+"N_MAXIMA"+suffix+"="+QString::number(_maxPeakCount)+"\n";
-    log+=prefix+"ABSOLUTE_THRESHOLD"+suffix+"="+QString::number(_absoluteThreshold)+"\n";
-    log+=prefix+"RELATIVE_THRESHOLD"+suffix+" (%)="+QString::number(_relativeThreshold)+"\n";
-    log+=prefix+"EXPORT_ALL_FK_GRIDS"+suffix+"="+(_exportAllFKGrids ? "y" : "n")+"\n";
+    switch(_processType) {
+    case RTBF:
+    case RTBFRadial:
+    case RTBFFixedEll:
+    case PoggiVertical:
+    case PoggiRadial:
+    case ConventionalRadial:
+    case ConventionalTransverse:
+    case ConventionalRayleigh:
+    case CaponRadial:
+    case CaponTransverse:
+      log+="# Number of steps for the computation of radial/transverse projections\n";
+      log+=prefix+"ROTATE_STEP_COUNT"+suffix+"="+QString::number(_rotateStepCount)+"\n";
+      break;
+    default:
+      break;
+    }
     log+=prefix+"DAMPING_FACTOR"+suffix+"="+QString::number(_damping)+"\n";
     log+="# If provided and PROCESS_TYPE==DirectSteering, the ellipticity is forced to the provided curve.\n"
          "# The file must contain two columns: frequency and signed ellipticity.\n"
@@ -433,34 +446,59 @@ namespace ArrayCore {
     log+=prefix+"MINIMUM_DISTANCE"+suffix+"="+QString::number(_minimumDistance)+"\n";
     log+="# Maximum distance between source and receiver (for active source only)\n";
     log+=prefix+"MAXIMUM_DISTANCE"+suffix+"="+QString::number(_maximumDistance)+"\n";
+    log+=prefix+"SOURCE_GRID_STEP"+suffix+"="+QString::number(_sourceGridStep)+"\n";
+    log+=prefix+"SOURCE_GRID_SIZE"+suffix+"="+QString::number(_sourceGridSize)+"\n";
     log+="# Experimental join processing of several arrays\n";
     log+="# Several ARRAY can be defined with a list of station names\n";
     for(QList<QStringList>::const_iterator it=_arrays.begin(); it!=_arrays.end(); it++) {
       log+=prefix+"ARRAY"+suffix+"="+it->join(',')+"\n";
     }
+    log+="#\n"
+         "#\n"
+         "#     Wavenumber grid\n"
+         "#\n"
+         "#\n";
+    log+="# Wavenumber fine gridding used as a cache for the FK maps\n";
+    log+=prefix+"CACHE_GRID_STEP"+suffix+" (rad/m)="+QString::number(_cacheGridStep)+"\n";
+    log+="# If CACHE_GRID_STEP is null, GRID_STEP is computed from K_MIN*CACHE_GRID_STEP_FACTOR.\n";
+    log+=prefix+"CACHE_GRID_STEP_FACTOR"+suffix+"="+QString::number(_cacheGridStepFactor)+"\n";
 
-    switch(_processType) {
-    case DirectSteering:
-    case DirectSteeringRadial:
-    case DirectSteeringRefined:
-    case DirectSteeringVertical:
-    case ActiveConventional:
-    case ActiveRTBF:
-    case ActiveDirectSteering:
-    case Omni:
-      break;
-    case RTBF:
-    case RTBFRadial:
-    case PoggiVertical:
-    case PoggiRadial:
-    case Conventional:
-      log+="# Number of steps for the computation of radial/transverse projections\n";
-      log+=prefix+"ROTATE_STEP_COUNT"+suffix+"="+QString::number(_rotateStepCount)+"\n";
-      break;
-    }
-    log+="# \n";
-    log+=prefix+"SOURCE_GRID_STEP"+suffix+"="+QString::number(_sourceGridStep)+"\n";
-    log+=prefix+"SOURCE_GRID_SIZE"+suffix+"="+QString::number(_sourceGridSize)+"\n";
+    log+="# Wavenumber coarse gridding used for searching maxima of the FK maps\n";
+    log+=prefix+"GRID_STEP"+suffix+" (rad/m)="+QString::number(_gridStep)+"\n";
+    log+="# If GRID_STEP is null, GRID_STEP is computed from K_MIN*GRID_STEP_FACTOR.\n";
+    log+=prefix+"GRID_STEP_FACTOR"+suffix+"="+QString::number(_gridStepFactor)+"\n";
+    log+=prefix+"GRID_SIZE"+suffix+" (rad/m)="+QString::number(_gridSize)+"\n";
+    log+="# If GRID_SIZE is null, GRID_SIZE is computed from K_MAX*GRID_SIZE_FACTOR.\n";
+    log+=prefix+"GRID_SIZE_FACTOR"+suffix+"="+QString::number(_gridSizeFactor)+"\n";
+    log+="# Effective GRID_STEP is "+QString::number(gridStep())+".\n";
+    log+="# Effective GRID_SIZE is "+QString::number(gridSize())+".\n";
+
+    log+="# Minimum velocity of the searched maxima of the FK map\n";
+    log+=prefix+"MIN_V"+suffix+" (m/s)="+QString::number(1.0/_smax)+"\n";
+    log+="# Maximum velocity of the searched maxima of the FK map\n";
+    log+=prefix+"MAX_V"+suffix+" (m/s)="+QString::number(1.0/_smin)+"\n";
+
+    log+="# Minimum azimuth of the searched maxima of the FK map (clockwise from North)\n";
+    log+=prefix+"MIN_AZIMUTH"+suffix+" (deg.)="+QString::number(_minAzimuth)+"\n";
+    log+="# Maximum azimith of the searched maxima of the FK map (clockwise from North)\n";
+    log+=prefix+"MAX_AZIMUTH"+suffix+" (deg.)="+QString::number(_maxAzimuth)+"\n";
+
+    log+="# Theoretical Kmin and Kmax computed from array geometry\n";
+    log+="# Used only for post-processing\n";
+    log+=prefix+"K_MIN"+suffix+" (rad/m)="+QString::number(_kmin)+"\n";
+    log+=prefix+"K_MAX"+suffix+" (rad/m)="+QString::number(_kmax)+"\n";
+    log+="#\n"
+         "#\n"
+         "#     Peak picking\n"
+         "#\n"
+         "#\n";
+    log+="# Inversion method used for getting FK peaks: Gradient or RefinedGrid\n";
+    log+=prefix+"INVERSION_METHOD"+suffix+"="+inversionMethodString()+"\n";
+    log+=prefix+"N_MAXIMA"+suffix+"="+QString::number(_maxPeakCount)+"\n";
+    log+=prefix+"ABSOLUTE_THRESHOLD"+suffix+"="+QString::number(_absoluteThreshold)+"\n";
+    log+=prefix+"RELATIVE_THRESHOLD"+suffix+" (%)="+QString::number(_relativeThreshold)+"\n";
+    log+=prefix+"SAVE_UNDEFINED_ELLIPTICITIES"+suffix+"="+(_saveUndefinedEllipticities ? "y" : "n")+"\n";
+    log+=prefix+"EXPORT_ALL_FK_GRIDS"+suffix+"="+(_exportAllFKGrids ? "y" : "n")+"\n";
     return log;
   }
 
@@ -481,7 +519,7 @@ namespace ArrayCore {
     case 0:
       if(version()<1) {
         _gridStep=value.toDouble(&ok); // Old release were saving grid_step instead of kmin
-        _kmin=0;                       // Force automatic computation from array geometry
+        _kmin=0.0;                     // Force automatic computation from array geometry
       } else {
         _kmin=value.toDouble(&ok);
       }
@@ -489,7 +527,7 @@ namespace ArrayCore {
     case 1:
       if(version()<1) {
         _gridSize=value.toDouble(&ok); // Old release were saving grid_size instead of kmax
-        _kmax=0;                       // Force automatic computation from array geometry
+        _kmax=0.0;                     // Force automatic computation from array geometry
       } else {
         _kmax=value.toDouble(&ok);
       }
@@ -503,6 +541,9 @@ namespace ArrayCore {
     case 18:
       _gridStepFactor=value.toDouble(&ok);
       return ok;
+    case 28:
+      _gridSizeFactor=value.toDouble(&ok);
+      return ok;
     case 24:
       _smin=1.0/value.toDouble(&ok);
       return ok;
@@ -527,6 +568,9 @@ namespace ArrayCore {
     case 6:
       _exportAllFKGrids=(value=="y");
       return true;
+    case 27:
+      _saveUndefinedEllipticities=(value=="y");
+      return ok;
     case 7:
       if(_damping>=0.0) {
         _damping=value.toDouble(&ok);
@@ -541,8 +585,8 @@ namespace ArrayCore {
     case 10:
       _arrays.append(value.split(','));
       return true;
-    case 11:
-      _skipLove=(value=="y");
+    case 11:  // For compatibility
+      obsoleteKeyword(keywords, 11);
       return true;
     case 12:
       _fixedEllipticityFileName=value;
@@ -574,34 +618,99 @@ namespace ArrayCore {
   bool FKParameters::setProcessType(QString p)
   {
     p=p.toLower();
-    if(p=="directsteering") {
-      _processType=DirectSteering;
-    } else if(p=="omni") {
-      _processType=Omni;
-    } else if(p=="conventional") {
-      _processType=Conventional;
-    } else if(p=="poggivertical") {
-      _processType=PoggiVertical;
-    } else if(p=="poggiradial") {
-      _processType=PoggiRadial;
-    } else if(p=="rtbf" ||
-              p=="projections") {
-      _processType=RTBF;
-    } else if(p=="rtbfradial") {
-      _processType=RTBFRadial;
-    } else if(p=="directsteeringrefined") {
-      _processType=DirectSteeringRefined;
-    } else if(p=="directsteeringvertical") {
-      _processType=DirectSteeringVertical;
-    } else if(p=="directsteeringradial") {
-      _processType=DirectSteeringRadial;
-    } else if(p=="activeprojections") {
-      _processType=ActiveRTBF;
-    } else if(p=="activeconventional") {
-      _processType=ActiveConventional;
-    } else if(p=="activedirectsteering") {
-      _processType=ActiveDirectSteering;
-    } else {
+    if(p.count()<3) {
+      return false;
+    }
+    switch(p[2].unicode()) {
+    case 'b':
+      if(p=="rtbf") {
+        _processType=RTBF;
+      } else if(p=="rtbfradial") {
+        _processType=RTBFRadial;
+      } else if(p=="rtbffixedell") {
+        _processType=RTBFFixedEll;
+      }
+      return false;
+    case 'g':
+      if(p=="poggivertical") {
+        _processType=PoggiVertical;
+      } else if(p=="poggiradial") {
+        _processType=PoggiRadial;
+      }
+      return false;
+    case 'n':
+      if(p=="conventional") {
+        _processType=Conventional;
+      } else if(p=="omni") {
+        _processType=Omni;
+      } else if(p=="conventionalradial") {
+        _processType=ConventionalRadial;
+      } else if(p=="conventionaltransverse") {
+        _processType=ConventionalTransverse;
+      } else if(p=="conventionalrayleigh") {
+        _processType=ConventionalRayleigh;
+      }
+      return false;
+    case 'o':
+      if(p=="projections") {                     // Kept for compatibility
+        _processType=RTBF;
+      }
+      return false;
+    case 'p':
+      if(p=="capon") {
+        _processType=Capon;
+      } else if(p=="caponradial") {
+        _processType=CaponRadial;
+      } else if(p=="capontransverse") {
+        _processType=CaponTransverse;
+      }
+      return false;
+    case 'r':
+      if(p=="directsteering") {                  // Kept for compatibility
+        _processType=RDS;
+      } else if(p=="directsteeringrefined") {    // Kept for compatibility
+        _processType=RDSRefined;
+      } else if(p=="directsteeringvertical") {   // Kept for compatibility
+        _processType=RDSVertical;
+      } else if(p=="directsteeringradial") {    // Kept for compatibility
+        _processType=RDSRadial;
+      } else if(p=="directsteeringsingle") {    // Kept for compatibility
+        _processType=RDSSingle;
+      }
+      return false;
+    case 's':
+      if(p=="rds") {
+        _processType=RDS;
+      } else if(p=="lds") {
+        _processType=LDS;
+      } else if(p=="rdsradial") {
+        _processType=RDSRadial;
+      } else if(p=="rdsvertical") {
+        _processType=RDSVertical;
+      } else if(p=="rdsrefined") {
+        _processType=RDSRefined;
+      } else if(p=="rdssingle") {
+        _processType=RDSSingle;
+      } else if(p=="rdsfixedell") {
+        _processType=RDSFixedEll;
+      }
+      return false;
+    case 't':
+      if(p=="activeconventional") {
+        _processType=ActiveConventional;
+      } else if(p=="activecapon") {
+        _processType=ActiveCapon;
+      } else if(p=="activeconventionalrayleigh") {
+        _processType=ActiveConventionalRayleigh;
+      } else if(p=="activertbf") {
+        _processType=ActiveRTBF;
+      } else if(p=="activeds") {
+        _processType=ActiveDS;
+      } else if(p=="activeprojections") {        // Kept for compatibility
+        _processType=ActiveRTBF;
+      } else if(p=="activedirectsteering") {     // Kept for compatibility
+        _processType=ActiveDS;
+      }
       return false;
     }
     return true;
@@ -610,54 +719,67 @@ namespace ArrayCore {
   QString FKParameters::processTypeString() const
   {
     switch(_processType) {
-    case DirectSteering:
+    case Conventional:
       break;
+    case Capon:
+      return "Capon";
+    case ActiveConventional:
+      return "ActiveConventional";
+    case ActiveCapon:
+      return "ActiveCapon";
     case Omni:
       return "Omni";
-    case Conventional:
-      return "Conventional";
-    case PoggiVertical:
-      return "PoggiVertical";
-    case PoggiRadial:
-      return "PoggiRadial";
+    case ConventionalRadial:
+      return "ConventionalRadial";
+    case CaponRadial:
+      return "CaponRadial";
+    case ConventionalTransverse:
+      return "ConventionalTransverse";
+    case CaponTransverse:
+      return "CaponTransverse";
+    case ConventionalRayleigh:
+      return "ConventionalRayleigh";
     case RTBF:
       return "RTBF";
     case RTBFRadial:
       return "RTBFRadial";
-    case DirectSteeringRefined:
-      return "DirectSteeringRefined";
-    case DirectSteeringVertical:
-      return "DirectSteeringVertical";
-    case DirectSteeringRadial:
-      return "DirectSteeringRadial";
+    case RTBFFixedEll:
+      return "RTBFFixedEll";
+    case RDS:
+      return "RDS";
+    case RDSRadial:
+      return "RDSRadial";
+    case RDSVertical:
+      return "RDSVertical";
+    case RDSRefined:
+      return "RDSRefined";
+    case RDSSingle:
+      return "RDSSingle";
+    case RDSFixedEll:
+      return "RDSFixedEll";
+    case PoggiVertical:
+      return "PoggiVertical";
+    case PoggiRadial:
+      return "PoggiRadial";
+    case LDS:
+      return "LDS";
+    case ActiveConventionalRayleigh:
+      return "ActiveConventionalRayleigh";
     case ActiveRTBF:
       return "ActiveRTBF";
-    case ActiveConventional:
-      return "ActiveConventional";
-    case ActiveDirectSteering:
-      return "ActiveDirectSteering";
+    case ActiveDS:
+      return "ActiveDS";
     }
-    return "DirectSteering";
+    return "Conventional";
   }
 
   bool FKParameters::isActiveProcessType() const
   {
     switch(_processType) {
-    case ActiveConventional:
-    case ActiveRTBF:
-    case ActiveDirectSteering:
-      break;
-    case Omni:
-    case Conventional:
-    case PoggiVertical:
-    case PoggiRadial:
-    case RTBF:
-    case RTBFRadial:
-    case DirectSteering:
-    case DirectSteeringRefined:
-    case DirectSteeringVertical:
-    case DirectSteeringRadial:
+    FKPARAMETERS_PASSIVE_PROCESS_TYPES
       return false;
+    FKPARAMETERS_ACTIVE_PROCESS_TYPES
+      break;
     }
     return true;
   }
@@ -687,4 +809,27 @@ namespace ArrayCore {
     return "Gradient";
   }
 
+  QString FKParameters::outputName(const QString& groupName) const
+  {
+    QString fn;
+    if(ArrayParameters::outputBaseName().endsWith(".max")) {
+      return ArrayParameters::outputBaseName();
+    } else if(ArrayParameters::outputBaseName().isEmpty()) {
+      return processTypeString().toLower()+"-"+groupName;
+    } else {
+      QString baseName;
+      if(!ArrayParameters::outputBaseName().endsWith("-") &&
+         !ArrayParameters::outputBaseName().endsWith("_")) {
+        baseName=ArrayParameters::outputBaseName()+"-";
+      } else {
+        baseName=ArrayParameters::outputBaseName();
+      }
+      if(baseName.contains(processTypeString().toLower())) {
+        return baseName+groupName;
+      } else {
+        return baseName+processTypeString().toLower()+"-"+groupName;
+      }
+    }
+  }
+
 } // namespace ArrayCore
