@@ -36,7 +36,7 @@ namespace ArrayCore {
     PoggiFKVertical(FKCache * gridCache)
       : HRFKVertical(gridCache) {}
 
-    virtual double ellipticity(const Point& k) const;
+    virtual double ellipticity(const Point& k, bool& ok);
   };
 
 } // namespace ArrayCore
