@@ -201,7 +201,7 @@ namespace GeopsyCore {
   }
 
   /*!
-    \a minimumCount is the minimum number of signals in a group to be included in the returned list.
+
   */
   QList<const AbstractSignalGroup *> AbstractSignalGroup::list() const
   {
@@ -379,7 +379,7 @@ namespace GeopsyCore {
 
   /*!
     Returns group with \a name if any in the children list. Name may contains '/' to access sub-groups.
-    \a name may start with '/' but all group names are always absolute, there is not relative paths.
+    \a name may start with '/' but all group names are always absolute, there is no relative path.
   */
   const AbstractSignalGroup * AbstractSignalGroup::find(QString name) const
   {
@@ -400,7 +400,7 @@ namespace GeopsyCore {
 
   /*!
     Returns group with \a name if any in the children list. Name may contains '/' to access sub-groups.
-    \a name may start with '/' but all group names are always absolute, there is not relative paths.
+    \a name may start with '/' but all group names are always absolute, there is no relative path.
   */
   AbstractSignalGroup * AbstractSignalGroup::find(QString name)
   {
@@ -564,6 +564,17 @@ namespace GeopsyCore {
     }
   }
 
+  /*!
+    Removes all signals
+  */
+  void AbstractSignalGroup::removeAllSignals()
+  {
+    TRACE;
+    for(const_iterator it=begin();it!=end();++it) {
+      static_cast<AbstractSignalGroup *>(*it)->removeAllSignals();
+    }
+  }
+
   /*!
     Reports a parent change to the top of the tree.
     To exploit this feature, you must have a MasterSignalGroup at the top of the tree.
@@ -700,7 +711,7 @@ namespace GeopsyCore {
   bool AbstractSignalGroup::xml_setProperty(XML_SETPROPERTY_ARGS)
   {
     TRACE;
-    switch (memberID) {
+    switch(memberID) {
     case 0:
       setName(content.toString());
       return true;
