@@ -339,7 +339,7 @@ bool DoubleSignal::filter(const FilterParameters& param)
             filrec(param.minimumFrequency(), 1, 0, _nSamples-1, 1, thisSamples, thisSamples, param.order(), _samplingPeriod);
             filrec(param.maximumFrequency(), -1, 0, _nSamples-1, 1, s, s, param.order(), _samplingPeriod);
             for(i=0; i<_nSamples; i++) {
-              thisSamples[ i ]+=s[i];
+              thisSamples[i]+=s[i];
             }
             delete [] s;
           }
@@ -355,6 +355,8 @@ bool DoubleSignal::filter(const FilterParameters& param)
       restoreType(ost);
     }
     break;
+  case FilterParameters::ChebyshevTypeI:
+    break;
   case FilterParameters::FrequencyWindow: {
       SignalType st=saveType(Spectrum);
       double fac=_samplingPeriod*static_cast<double>(_nSamples);
@@ -363,27 +365,29 @@ bool DoubleSignal::filter(const FilterParameters& param)
         TaperDelegate window(&param.frequencyWindow());
         switch (param.band()) {
         case FilterParameters::LowPass:
-          window.apply(samples,
-                       qRound(fac*param.minimumFrequency()*(1.0-param.width())),
-                       _nyquistIndex,
-                       qRound(fac*param.minimumFrequency()*(1.0-2.0*param.width())),
-                       qRound(fac*param.minimumFrequency()));
+          window.createFunction(qRound(fac*param.minimumFrequency()*(1.0-param.width())),
+                                _nyquistIndex,
+                                qRound(fac*param.minimumFrequency()*(1.0-2.0*param.width())),
+                                qRound(fac*param.minimumFrequency()));
+          window.apply(samples);
           break;
         case FilterParameters::HighPass: {
             int i=qRound(fac*param.minimumFrequency()*(1.0+1.0*param.width()));
             if(i>_nyquistIndex) {
               i=_nyquistIndex;
             }
-            window.apply(samples, 0, i,
-                         qRound(fac*param.minimumFrequency()),
-                         qRound(fac*param.minimumFrequency()*(1.0+2.0*param.width())));
+            window.createFunction(0, i,
+                                  qRound(fac*param.minimumFrequency()),
+                                  qRound(fac*param.minimumFrequency()*(1.0+2.0*param.width())));
+            window.apply(samples);
           }
           break;
         case FilterParameters::BandPass:
         case FilterParameters::BandReject:
-          window.apply(samples, 0, _nyquistIndex,
-                       qRound(fac*param.minimumFrequency()),
-                       qRound(fac*param.maximumFrequency()));
+          window.createFunction(0, _nyquistIndex,
+                                qRound(fac*param.minimumFrequency()),
+                                qRound(fac*param.maximumFrequency()));
+          window.apply(samples);
           break;
         }
       UNLOCK_SAMPLES(this) else ret=false;
@@ -648,10 +652,10 @@ bool DoubleSignal::stddevClip(double factor)
 {
   TRACE;
   bool ret=false;
-  SignalType st=saveType(Waveform); // make sure we are in time domain
-  double sum=average(0, _nSamples)*_nSamples;
-  double sum2=average2(0, _nSamples)*(_nSamples-1);
   LOCK_SAMPLES(double, thisSamples, this)
+    SignalType st=saveType(Waveform); // make sure we are in time domain
+    double sum=average(0, _nSamples)*_nSamples;
+    double sum2=average2(0, _nSamples)*(_nSamples-1);
     double invNSamples=1.0/_nSamples;
     double factor2=factor*factor;
     double varianceFactor=factor2*(sum2-sum*invNSamples*sum)/(_nSamples-1);
@@ -684,9 +688,9 @@ bool DoubleSignal::stddevClip(double factor)
     if(newStddevFactor<0.9*stddevFactor) {
       App::log(tr("Convergence criterium not reached (less than 10% variantion of stddev).\n") );
     }
+    restoreType(st);
     ret=true;
   UNLOCK_SAMPLES(this)
-  restoreType(st);
   return ret;
 }
 
@@ -704,7 +708,8 @@ bool DoubleSignal::taper(double start, double end, const WindowFunctionParameter
     TaperDelegate::RealSample samples(thisSamples);
     TaperDelegate window(&param);
     if(end-start>samplingPeriod()) {
-      window.apply(samples, start, end, samplingFrequency(), _nSamples);
+      window.createFunction(start, end, samplingFrequency(), _nSamples);
+      window.apply(samples);
     }
     ret=true;
   UNLOCK_SAMPLES(this)
@@ -725,7 +730,27 @@ bool DoubleSignal::taper(const WindowFunctionParameters& param)
   LOCK_SAMPLES(double, thisSamples, this)
     TaperDelegate::RealSample samples(thisSamples);
     TaperDelegate window(&param);
-    window.apply(samples, 0, _nSamples-1, 0, _nSamples-1);
+    window.createFunction(0, _nSamples-1, 0, _nSamples-1);
+    window.apply(samples);
+    ret=true;
+  UNLOCK_SAMPLES(this)
+  restoreType(st);
+  return ret;
+}
+
+/*!
+  Applies a taper window on the complete signal.
+
+  Returns false if not processed due to memory problem.
+*/
+bool DoubleSignal::taper(const TaperDelegate& window)
+{
+  TRACE;
+  bool ret=false;
+  SignalType st=saveType(Waveform);
+  LOCK_SAMPLES(double, thisSamples, this)
+    TaperDelegate::RealSample samples(thisSamples);
+    window.apply(samples);
     ret=true;
   UNLOCK_SAMPLES(this)
   restoreType(st);
@@ -892,12 +917,12 @@ void DoubleSignal::fastFourierTransform(SignalType st)
           cache->execute(thisSamples);
           FFTWCache::end(cache);
           // Factor according to McNamara
-          // Stable for various sampling frequencis
+          // Stable for various sampling frequencies
           // Stable for various signal durations
 #ifdef COMPATIBILITY_2_5_0
           double rn=1.0/(static_cast<double>(_nSamples)*_samplingPeriod);
 #else
-          double rn=::sqrt(2*_samplingPeriod/static_cast<double>(_nSamples));
+          double rn=::sqrt(2.0*_samplingPeriod/static_cast<double>(_nSamples));
 #endif
           for(int i=0; i<_nSamples; i++) {
             thisSamples[i]*=rn;
@@ -1554,7 +1579,7 @@ Curve<Point2D> DoubleSignal::spectrumAmplitudeCurve() const
   Curve<Point2D> c(nSamples2+1);
   CONST_LOCK_SAMPLES(double, sigSamples, sigfft)
     for(int i=0; i<=nSamples2; i++) {
-      Point2D& p=c[i];
+      Point2D& p=c.constXAt(i);  // not true but sort afterwards
       p.setX(static_cast<double>(i)*df);
       p.setY(sigfft->amplitude(sigSamples, i));
     }
@@ -2264,22 +2289,21 @@ double DoubleSignal::averageAmplitude(int itmin, int itmax) const
 }
 
 /*!
-  Returns the energy between frequency \a minF and \a maxF.
+  Returns the power between frequency \a minF and \a maxF (frequency domain)
 */
-double DoubleSignal::energy(double minF, double maxF) const
+double DoubleSignal::power(double minF, double maxF) const
 {
   TRACE;
   double en=0.0;
   const DoubleSignal * sig=saveType(Spectrum); // make sure we are in frequency domain
-  CONST_LOCK_SAMPLES(double, sigSamples, sig)
-    double fac=_samplingPeriod * (double) _nSamples;
-    int minI=(int) floor(minF * fac);
+  CONST_LOCK_SAMPLES(double, samples, sig)
+    double fac=_samplingPeriod*static_cast<double>(_nSamples);
+    int minI=qFloor(minF*fac);
     if(minI<0) minI=0;
-    int maxI=(int) ceil(maxF * fac);
-    if(maxI>=_nSamples) maxI=_nSamples-1;
-    if(minI>=maxI) return 0.0;
-    for(int i=minI; i<maxI; i++ ) {
-      en+=sig->amplitude2(sigSamples, i);
+    int maxI=qCeil(maxF*fac);
+    if(maxI>_nyquistIndex) maxI=_nyquistIndex;
+    for(int i=minI; i<=maxI; i++) {
+      en+=sig->amplitude2(samples, i);
     }
   UNLOCK_SAMPLES(sig);
   restoreType(sig);
@@ -2287,7 +2311,7 @@ double DoubleSignal::energy(double minF, double maxF) const
 }
 
 /*!
-  Time domain computation of total power. \a excludeBorderRatio can for instance 0.1 to exclude
+  Time domain computation of total power. \a excludeBorderRatio can be for instance 0.1 to exclude
   10% of signal on both ends.
 */
 double DoubleSignal::power(double excludeBorderRatio) const
