@@ -36,6 +36,7 @@
 #include "GeopsyPreferences.h"
 #include "Comments.h"
 #include "KeepSignal.h"
+#include "GeopsyCoreEngine.h"
 
 namespace GeopsyCore {
 
@@ -865,6 +866,7 @@ namespace GeopsyCore {
   {
     Q_UNUSED(context)
     writeProperty(s, "version", _version);
+    writeProperty(s, "fileCount", _files.count());
     // Save links for permanent meta data
     ASSERT(_allSharedMetaData);
     int n=_permanentSharedMetaData.count();
@@ -902,6 +904,8 @@ namespace GeopsyCore {
     switch (tag[0].unicode()) {
     case 'F':
       if(tag=="File") {
+        GeopsyCoreEngine * eng=GeopsyCoreEngine::instance();
+        eng->setProgressValue(this, _files.count());
         SignalFile * f=new SignalFile(this);
         return XMLMember(f);
       }
@@ -924,6 +928,11 @@ namespace GeopsyCore {
         return XMLMember(_seismicEvents);
       }
       break;
+    case 'f':
+      if(tag=="fileCount") {
+        return XMLMember(1);
+      }
+      break;
     case 'v':
       if(tag=="version") {
         return XMLMember(0);
@@ -956,7 +965,7 @@ namespace GeopsyCore {
     Q_UNUSED(tag)
     Q_UNUSED(attributes)
     Q_UNUSED(context)
-    switch (memberID) {
+    switch(memberID) {
     case 0:
       _version=content.toInt();
       if(_version>SIGNAL_DATABASE_CURRENT_VERSION) {
@@ -964,6 +973,11 @@ namespace GeopsyCore {
         return false;
       }
       return true;
+    case 1: {
+        GeopsyCoreEngine * eng=GeopsyCoreEngine::instance();
+        eng->setProgressMaximum(this, content.toInt()-1);
+      }
+      return true;
     default:
       return false;
     }
@@ -1031,49 +1045,49 @@ namespace GeopsyCore {
   SubSignalPool SignalDatabase::load(QStringList fileNames, SignalFileFormat format, bool viewFiles)
   {
     TRACE;
+    GeopsyPreferences * prefs=GeopsyCoreEngine::instance()->preferences();
     if(fileNames.isEmpty()) {
       QSettings& reg=CoreApplication::instance()->settings();
       reg.beginGroup("DialogOptions");
-      if(GeopsyCoreEngine::instance()->preferences()->askForFormat()) {
+      if(prefs->askForFormat()) {
         reg.setValue("LoadFormat/again", (bool) true);
       }
-      reg.setValue("TimeReference/again", (bool) true);
       fileNames=Message::getOpenFileNames(tr("Load Signals"), tr("Signal file (*)"));
     }
 
     SubSignalPool subPool;
     if(!fileNames.isEmpty()) {
       fileNames=File::expand(fileNames);
-      if(format.id()==SignalFileFormat::Unknown && !GeopsyCoreEngine::instance()->preferences()->askForFormat()) {
-        format=GeopsyCoreEngine::instance()->preferences()->fileFormat();
+      if(format.id()==SignalFileFormat::Unknown && !prefs->askForFormat()) {
+        format=prefs->fileFormat();
       }
       std::sort(fileNames.begin(), fileNames.end(), lessThanLocalAwareCompareString);
       bool retLoad=false;
       if(viewFiles) {
-        viewFiles=!GeopsyCoreEngine::instance()->preferences()->viewIfFewFiles() ||
-                  (GeopsyCoreEngine::instance()->preferences()->viewIfFewFiles() &&
-                  fileNames.count()<GeopsyCoreEngine::instance()->preferences()->maxNofFiles());
+        viewFiles=!prefs->viewIfFewFiles() ||
+                  (prefs->viewIfFewFiles() &&
+                  fileNames.count()<prefs->maxNofFiles());
       }
       GeopsyCoreEngine::instance()->setProgressMaximum(this, fileNames.count()-1);
       int iProgress=0;
       for(QStringList::Iterator it=fileNames.begin();it!=fileNames.end();++it) {
         GeopsyCoreEngine::instance()->setProgressValue(this, iProgress++);
-        if(GeopsyCoreEngine::instance()->preferences()->askForFormat()) {
+        if(prefs->askForFormat()) {
           format=GeopsyCoreEngine::instance()->askLoadFormat(*it);
         }
         App::log(tr("Loading %1...\n").arg(*it) );
         retLoad=_files.load(*it, format);
         if(retLoad) {
           subPool.addFile(_files.last());
-          if(viewFiles && GeopsyCoreEngine::instance()->preferences()->oneWindowPerFile()) {
+          if(viewFiles && prefs->oneWindowPerFile()) {
             SubSignalPool subPool;
             subPool.addFile(_files.last());
-            GeopsyCoreEngine::instance()->newSignalViewer(subPool);
+            GeopsyCoreEngine::instance()->newSignalViewer(subPool, false);
           }
         }
       }
-      if(viewFiles && !GeopsyCoreEngine::instance()->preferences()->oneWindowPerFile() && !subPool.isEmpty()) {
-        GeopsyCoreEngine::instance()->newSignalViewer(subPool);
+      if(viewFiles && !prefs->oneWindowPerFile() && !subPool.isEmpty()) {
+        GeopsyCoreEngine::instance()->newSignalViewer(subPool, false);
       }
     }
     return subPool;
@@ -1091,6 +1105,40 @@ namespace GeopsyCore {
     return load(fileList, format, viewFile);
   }
 
+  /*!
+    Convenient function to load either a database file or to import files.
+    If viewFiles is true, the imported signals or the group is showm (if GUI is available).
+  */
+  SubSignalPool SignalDatabase::load(QString dbFile, QString groupName,
+                                     const QStringList& fileNames, QString format,
+                                     bool viewFiles)
+  {
+    SubSignalPool subPool;
+    if(!dbFile.isEmpty() && open(dbFile)) {
+      if(!groupName.isEmpty()) {
+        AbstractSignalGroup * group=findGroup(groupName);
+        if(group) {
+          subPool=group->subPool();
+          if(viewFiles && !subPool.isEmpty()) {
+            GeopsyCoreEngine::instance()->newSignalViewer(subPool, true);
+          }
+        } else {
+          App::log(tr("%1: unknown group of signals (%2)\n")
+                   .arg(CoreApplication::applicationName())
+                   .arg(groupName));
+        }
+      }
+    }
+    if(!fileNames.isEmpty()) {
+      if(subPool.isEmpty()) {
+        subPool=load(fileNames, SignalFileFormat::fromName(format), viewFiles);
+      } else {
+        load(fileNames, SignalFileFormat::fromName(format), viewFiles);
+      }
+    }
+    return subPool;
+  }
+
   /*!
     Set all processed files to temporary signals and re-save the database with only the temporary signals referenced in groups
   */
