@@ -182,6 +182,7 @@ QString SignalFileFormat::name() const
   case GuralpGcf: return "GuralpGcf";
   case MiniSeed: return "MiniSeed";
   case Fourier: return "Fourier";
+  case Efispec3D: return "Efispec3D";
   case Custom:
     if(_customFormat) {
       return _customFormat->name();
@@ -232,6 +233,7 @@ QString SignalFileFormat::caption() const
   case GuralpGcf: return tr("Guralp");
   case MiniSeed: return tr("Mini seed");
   case Fourier: return tr("Fourier spectrum");
+  case Efispec3D: return "Efispec3D";
   case Custom:
     if(_customFormat) {
       return _customFormat->caption();
@@ -291,6 +293,7 @@ QString SignalFileFormat::filter() const
   case GuralpGcf: return "*.gcf *";
   case MiniSeed: return "*.mseed *.msd *";
   case Fourier: return "*.fourier";
+  case Efispec3D: return "*.gpl";
   }
   return "*";
 }
@@ -344,6 +347,7 @@ QString SignalFileFormat::suffix() const
   case GuralpGcf: return ".gcf";
   case MiniSeed: return ".mseed";
   case Fourier: return ".fourier";
+  case Efispec3D: return ".gpl";
   }
   return QString();
 }
@@ -410,12 +414,17 @@ SignalFileFormat SignalFileFormat::fromName(QString n)
       if(n=="ascii") return Ascii;
       else if(n=="asciionecolumn") return AsciiOneColumn;
       else return GeopsyCoreEngine::instance()->customFileFormats()->fromName(n);
+    } else if(n=="automatic"){
+      return SignalFileFormat::Unknown;
     }
     break;
   case 'c':
     if(n=="cityshark2") return CityShark2;
     else if(n=="city2") return CityShark2; // For compatibility
     break;
+  case 'e':
+    if(n=="efispec3d") return Efispec3D;
+    break;
   case 'g':
     if(n=="gse2") return Gse2;
     else if(n=="geopsysignal") return GeopsySignal;
@@ -520,6 +529,9 @@ SignalFileFormat SignalFileFormat::fromCaptionFilter(QString cf)
   case 'f':
     if(cf.startsWith(tr("saf sesame"))) return Saf;
     break;
+  case 'i':
+    if(cf.startsWith("efispec3d")) return Efispec3D;
+    break;
   case 'n':
     if(cf.startsWith(tr("mini seed"))) return MiniSeed;
     else if(cf.startsWith(tr("minishark"))) return MiniShark;
@@ -707,9 +719,10 @@ SignalFileFormat SignalFileFormat::fromSuffix(QString suffix)
     case 'g':
       if(suffix== "gse") {
         fmt=MultiGse2;
-      }
-      if(suffix== "gcf") {
+      } else if(suffix== "gcf") {
         fmt=GuralpGcf;
+      } else if(suffix=="gpl") {
+        fmt=Efispec3D;
       }
       break;
     case 'm':
