@@ -30,6 +30,7 @@
 #include "SignalDatabase.h"
 #include "TimePick.h"
 #include "Comments.h"
+#include "SignalGroup.h"
 
 namespace GeopsyCore {
 
@@ -82,24 +83,6 @@ namespace GeopsyCore {
   void SignalProcess::saveStep(const QString& tag)
   {
     TRACE;
-    // Save history to signal comments
-    if(_steps.isEmpty()) {
-      for(SubSignalPool::iterator it=_current->begin(); it!=_current->end(); it++) {
-        Signal * sig=*it;
-        Comments& c=sig->beginModifyMetaData<Comments>();
-        if(sig->file()) {
-          c+="'"+sig->file()->name()+"' at index "+QString::number(sig->numberInFile())+"\n";
-        }
-        sig->endModifyMetaData(c);
-      }
-    } else {
-      for(SubSignalPool::iterator it=_current->begin(); it!=_current->end(); it++) {
-        Signal * sig=*it;
-        Comments& c=sig->beginModifyMetaData<Comments>();
-        c+="  "+tag+"\n";
-        sig->endModifyMetaData(c);
-      }
-    }
     _steps.append(*_current);
     _steps.last().setName(tag);
   }
@@ -131,6 +114,13 @@ namespace GeopsyCore {
   void SignalProcess::addToHistory(const QString& s)
   {
     _history+=s;
+    // Save history to comments
+    for(SubSignalPool::iterator it=_current->begin(); it!=_current->end(); it++) {
+      Signal * sig=*it;
+      Comments& c=sig->beginModifyMetaData<Comments>();
+      c+=s;
+      sig->endModifyMetaData(c);
+    }
   }
 
   void SignalProcess::fastFourierTransform(DoubleSignal::SignalType st)
@@ -530,17 +520,38 @@ namespace GeopsyCore {
     if(endIndex<0 || endIndex>=n) {
       endIndex=n-1;
     }
-    if(startIndex>0 || endIndex<n-1) {
-      if(tag.isEmpty()) {
-        tag="selected from "+QString::number(startIndex)+" to "+QString::number(endIndex);
-      }
-      SubSignalPool newCurrent;
+    if(tag.isEmpty()) {
+      tag="selected from "+QString::number(startIndex)+" to "+QString::number(endIndex);
+    }
+    SubSignalPool newCurrent;
+    for(int i=startIndex; i<=endIndex; i++) {
+      newCurrent.addSignal(_current->at(i));
+    }
+    _steps.append(newCurrent);
+    _steps.last().setName(tag);
+    *_current=newCurrent;
+  }
+
+  void SignalProcess::newGroup(const QString& path, const QString& name, int startIndex, int endIndex)
+  {
+    int n=_current->count();
+    if(startIndex<0 || startIndex>=n) {
+      startIndex=0;
+    }
+    if(endIndex<0 || endIndex>=n) {
+      endIndex=n-1;
+    }
+    if(!name.isEmpty()) {
+      SubSignalPool sigs;
       for(int i=startIndex; i<=endIndex; i++) {
-        newCurrent.addSignal(_current->at(i));
+        sigs.addSignal(_current->at(i));
       }
-      _steps.append(newCurrent);
-      _steps.last().setName(tag);
-      *_current=newCurrent;
+      SignalGroup * g=new SignalGroup;
+      g->setSignals(sigs);
+      g->setName(name);
+      SignalDatabase * db=_current->database();
+      AbstractSignalGroup * gp=db->findGroup(path);
+      g->setParent(gp);
     }
   }
 
