@@ -566,7 +566,7 @@ int SparseTimeRange::index(const DateTime& t) const
 {
   TRACE;
   DateTime ts(t);
-  ts.addFewSeconds(1e-10); // Shift a little bit
+  ts.addFewSeconds(_epsilon); // Shift a little bit
   int n=count();
   int i=_n2-1;
   int step2=_n2 >> 1;
@@ -583,7 +583,11 @@ int SparseTimeRange::index(const DateTime& t) const
     }
     step2=step2 >> 1;
   }
-  return i;
+  if(i>=n) {
+    return n-1;
+  } else {
+    return i;
+  }
 }
 
 /*!
@@ -601,6 +605,23 @@ void SparseTimeRange::testIndex() const
   ASSERT(index(at(i).start())==i);
   ASSERT(index(at(i).center())==i);
   ASSERT(index(at(i).end())==i);
+  DateTime t=at(i).end();
+  t.addMonths(3);
+  ASSERT(index(t)==i);
+}
+
+void SparseTimeRange::testIndex(int maxCount)
+{
+  SparseTimeRange sr;
+  QDate d(2000, 1, 1);
+  for(int j=1; j<maxCount; j++) {
+    sr.clear();
+    printf("Testing %i blocks\n", j);
+    for(int i=0; i<j; i++) {
+      sr.add(TimeRange(DateTime(d, i*2, 0), DateTime(d, i*2+1, 0)));
+    }
+    sr.testIndex();
+  }
 }
 
 void SparseTimeRange::printDebug() const
