@@ -364,7 +364,7 @@ void SubSignalPool::waveletTransform(const MorletParameters& param)
       int n=sig->nSamples();
       LOCK_SAMPLES(double, sigSamples, sig)
         for(int i=0; i<n; i++) {
-          sigSamples[i]=cabs.at(i).y();
+          sigSamples[i]=cabs.y(i);
         }
       UNLOCK_SAMPLES(sig)
       sig->setType(Signal::Waveform);
@@ -984,7 +984,7 @@ int SubSignalPool::checkDuplicateRays()
 }
 
 /*!
-  Returns a vector of all receivers.
+  Returns a vector of all receiver coordinates.
 */
 Curve<Point> SubSignalPool::receivers() const
 {
@@ -998,7 +998,34 @@ Curve<Point> SubSignalPool::receivers() const
 }
 
 /*!
-  Returns a vector of all sources. One source per signal is allowed
+  Returns a vector of all receivers coordinates and names.
+*/
+QList<NamedPoint> SubSignalPool::stations() const
+{
+  TRACE;
+  QList<NamedPoint> l;
+  QMap<QString, NamedPoint> m;
+  bool ambiguities=false;
+  for(const_iterator it=begin(); it!=end(); ++it) {
+    Signal * sig=*it;
+    QMap<QString, NamedPoint>::const_iterator itm=m.find(sig->name());
+    if(itm!=m.end()) {
+      if(sig->receiver()!=itm.value()) {
+        ambiguities=true;
+      }
+    }
+    NamedPoint p(sig->receiver());
+    p.setName(sig->name());
+    m.insert(sig->name(), p);
+  }
+  if(ambiguities) {
+    App::log(tr("Ambiguities found in station list: several coordinates for a single name.\n"));
+  }
+  return m.values();
+}
+
+/*!
+  Returns a vector of all source coordinates. One source per signal is allowed
 */
 Curve<Point> SubSignalPool::sources() const
 {
@@ -1121,14 +1148,14 @@ bool SubSignalPool::associate3Components()
   // Testing if all 3 components are present (warning only)
   // Testing if nsamples and sampling rate are constant
   SubSignalPool triplet;
-  double samplingPeriod=0;
+  double samplingPeriod=0.0;
   TimeRange range;
   QString stat;
   Point rec;
   int n=count();
   for(int i=0; i<n; ) {
     Signal * sig=at(i);
-    if(samplingPeriod==0) {
+    if(samplingPeriod==0.0) {
       samplingPeriod=sig->samplingPeriod();
       range=sig->timeRange().range();
       stat=sig->name();
@@ -1138,12 +1165,12 @@ bool SubSignalPool::associate3Components()
       continue;
     }
     if(sig->name()==stat &&
-       sig->receiver()==rec &&
+       sig->receiver().isSimilar(rec, 0.001) &&
        sig->timeRange().range().intersection(range).lengthSeconds()>0.9*range.lengthSeconds()) {
       if(sig->samplingPeriod()!=samplingPeriod) {
         App::log(tr("Associating 3 components for station %1: "
                     "the sampling rate is not the same for all components.")
-                 .arg(stat).arg(sig->component()));
+                 .arg(stat));
         return false;
       } else {
         triplet.addSignal(sig);
@@ -1841,12 +1868,12 @@ bool SubSignalPool::saveArrivalTimes(QString filePath, const QString& pickName)
   s << "n  x  y  z\n";
   int n=uniqSrcList.count();
   for(int i=0; i<n; i++)
-    s << i + 1 << " " << uniqSrcList.at(i) << "\n";
+    s << i + 1 << " " << uniqSrcList.constAt(i) << "\n";
   s << "receivers positions\n";
   s << "n  x  y  z\n";
   n=recList.count();
   for(int i=0; i<n; i++) {
-    s << i + 1 << " " << recList.at(i) << "\n";
+    s << i + 1 << " " << recList.constAt(i) << "\n";
   }
   s << "arrival times\n";
   n=uniqSrcList.count();
@@ -1855,9 +1882,9 @@ bool SubSignalPool::saveArrivalTimes(QString filePath, const QString& pickName)
   for(int i=0; i<n; i++) {
     s << i+1 << " th source\n";
     int nr=0;
-    Point src=uniqSrcList.at(i);
+    const Point& src=uniqSrcList.constAt(i);
     for(int j=count()-1; j>=0; j--) {
-      if(srcList.at(j)==src) {
+      if(srcList.constAt(j)==src) {
         Signal * sig=at(j);
         ir[nr]=recList.indexOf(sig->receiver())+1;
         tr[nr]=sig->startTime().secondsTo(sig->metaData<TimePick>().value(pickName));
