@@ -49,7 +49,7 @@ namespace GeopsyCore {
     _maximumLength=0.0;
     _periodCount=0.0;
     _maximumWindowCount=0;
-    _powerOfTwo=0;
+    _maximumPrimeFactor=11; // A good compromise between speed and modification of time window length
 
     _overlap=0.0;
     _badSampleThresholdType=NoSampleThreshold;
@@ -113,7 +113,7 @@ namespace GeopsyCore {
     _maximumLength=o._maximumLength;
     _periodCount=o._periodCount;
     _maximumWindowCount=o._maximumWindowCount;
-    _powerOfTwo=o._powerOfTwo;
+    _maximumPrimeFactor=o._maximumPrimeFactor;
 
     _overlap=o._overlap;
     _badSampleThresholdType=o._badSampleThresholdType;
@@ -347,7 +347,7 @@ namespace GeopsyCore {
     printf("Maximum length=%lf\n", _maximumLength);
     printf("periodNum=%lf\n", _periodCount);
     printf("maximumWindowCount=%i\n", _maximumWindowCount);
-    printf("powerOfTwo=%s\n", _powerOfTwo ? "true" : "false");
+    printf("Maximum prime factor=%i\n", _maximumPrimeFactor);
     printf("Selected raw components: ");
     for(int i=0;i < _componentCount; i++) printf("%s, ", _rawComponent[i] ? "true" : "false");
     printf("\nSelected filter components: ");
@@ -385,8 +385,11 @@ namespace GeopsyCore {
       break;
     }
     log+=prefix+"WINDOW_MAX_COUNT"+suffix+"="+QString::number(_maximumWindowCount)+"\n";
-    log+=prefix+"WINDOW_POWER_OF_TWO"+suffix+" (y/n)=";
-    log+=_powerOfTwo ? "y\n" : "n\n";
+    log+="# Length of time windows are adjusted to avoid a prime number of samples which\n"
+         "# slows down the FFT computation. Usually a maximum prime 11 in the prime factorization\n"
+         "# of the number of samples is a good compromise between the speed and the lenght inaccuracy\n"
+         "# Leave as zero to allow any prime number factorization.\n";
+    log+=prefix+"WINDOW_MAXIMUM_PRIME_FACTOR"+suffix+"="+QString::number(_maximumPrimeFactor)+"\n";
     log+=prefix+"BAD_SAMPLE_TOLERANCE"+suffix+" (s)="+QString::number(_badSampleTolerance)+"\n";
     log+=prefix+"BAD_SAMPLE_GAP"+suffix+" (s)="+QString::number(_badSampleGap)+"\n";
     log+=prefix+"WINDOW_OVERLAP"+suffix+" (%)="+QString::number(_overlap)+"\n";
@@ -450,7 +453,7 @@ namespace GeopsyCore {
 
   int WindowingParameters::keywordCount(PARAMETERS_KEYWORDCOUNT_ARGS) const
   {
-    return 29+AbstractParameters::keywordCount();
+    return 30+AbstractParameters::keywordCount();
   }
 
   int WindowingParameters::totalKeywordCount(PARAMETERS_TOTALKEYWORDCOUNT_ARGS) const
@@ -469,6 +472,7 @@ namespace GeopsyCore {
     keywords.add(prefix+"PERIOD_COUNT"+suffix, this, baseIndex+26);
     keywords.add(prefix+"WINDOW_MAX_COUNT"+suffix, this, baseIndex+21);
     keywords.add(prefix+"WINDOW_POWER_OF_TWO"+suffix, this, baseIndex+22);
+    keywords.add(prefix+"WINDOW_MAXIMUM_PRIME_FACTOR"+suffix, this, baseIndex+29);
     keywords.add(prefix+"DO_BAD_SAMPLE_TOLERANCE"+suffix, this, baseIndex+23);        // For compatibility
     keywords.add(prefix+"BAD_SAMPLE_TOLERANCE"+suffix, this, baseIndex+3);
     keywords.add(prefix+"BAD_SAMPLE_GAP"+suffix, this, baseIndex+20);
@@ -608,8 +612,10 @@ namespace GeopsyCore {
     case 21:
       _maximumWindowCount=value.toInt(&ok);
       return ok;
-    case 22:
-      _powerOfTwo=(value=="y");
+    case 22:                     // To keep compatibility
+      if(value=="y") {
+        _maximumPrimeFactor=2;
+      }
       return true;
     case 27:
       _seismicEventTrigger=(value=="y");
@@ -617,6 +623,9 @@ namespace GeopsyCore {
     case 28:
       _seismicEventDelay=value.toDouble(&ok);
       return ok;
+    case 29:
+      _maximumPrimeFactor=value.toInt(&ok);;
+      return ok;
     default:
       break;
     }
