@@ -115,10 +115,10 @@ namespace HVCore {
     return keep;
   }
 
-  bool AbstractStation::addWindows(QTextStream& s, const QString& beginPattern, const QString& endPattern)
+  bool AbstractStation::addWindows(QTextStream& s, const QStringList& beginPatterns, const QStringList& endPatterns)
   {
     _windowsChanged=true;
-    return _timeWindows.add(s, beginPattern, endPattern);
+    return _timeWindows.add(s, beginPatterns, endPatterns, originalSignals()->startTime());
   }
 
   void AbstractStation::addWindows(const TimeRange& r, const WindowingParameters& param, const SparseKeepSignal& keep)
@@ -141,7 +141,6 @@ namespace HVCore {
     _windowsChanged=true;
   }
 
-
   void AbstractStation::inverseWindows(const TimeRange& r, const WindowingParameters& param, const SparseKeepSignal& keep)
   {
     TRACE;
@@ -280,20 +279,10 @@ namespace HVCore {
       s << "DATA_BASE=" << db->name() << "\n";
     }
     for(int i=0; i<_signals->nComponents(); i++) {
-      s << "[COMP " << QString::number(i) << "]\n";
       const SubSignalPool& subpool=_signals->originalSignals()->originals(i);
       for(int j=0; j<subpool.count(); j++) {
         Signal * sig=subpool.at(j);
-        s << " [SIG " << QString::number(j) << "] ";
-        if(sig->file()) {
-          s << sig->file()->name() << " at index " << sig->numberInFile();
-        } else { // Temporary signal with local processing listed in comments
-          QString cmt=sig->metaData<Comments>().value();
-          if(cmt.endsWith("\n")) {
-            cmt.chop(1);
-          }
-          s << cmt;
-        }
+        s << sig->file()->name() << " at index " << sig->numberInFile();
         if(db) {
           s << " (ID=" << sig->id() << ")";
         }
