@@ -66,7 +66,7 @@ namespace HVCore {
   void CurveResults::setWindows(int winCount, const HVParameters& param)
   {
     TRACE;
-    setFrequencySampling(param.sampling());
+    setFrequencySampling(param.frequencySampling());
     int ns=frequencySampling().count();
     _windows->clear();
     _windows->setPointCount(winCount, ns);
@@ -99,7 +99,7 @@ namespace HVCore {
     int di=winIndex*frequencySampling().count();
     int n=frequencySampling().count();
     for(int i=0; i<n; i++) {
-      y[i+di]=curve.at(x[i]).y();
+      y[i+di]=curve.interpole(x[i]).y();
     }
     _averageWindowLength.add(winLength);
   }
@@ -115,18 +115,18 @@ namespace HVCore {
     _stddevLow.resize(ns);
     _stddevHigh.resize(ns);
     for(int is=0; is<ns; is++) {
-      _average[is].setX(x[is]);
-      _stddevLow[is].setX(x[is]);
-      _stddevHigh[is].setX(x[is]);
+      _average.setX(is, x[is]);
+      _stddevLow.setX(is, x[is]);
+      _stddevHigh.setX(is, x[is]);
       Statistics s;
-      for(int ic=0; ic<_windows->curveCount(); ic++) {
+      for(int ic=_windows->curveCount()-1; ic>=0; ic--) {
         s.addLog(y[is+ic*ns]);
       }
       double mVal=s.meanLog();
       double sVal=s.stddevLog();
-      _average[is].setY(mVal);
-      _stddevLow[is].setY(mVal/sVal);
-      _stddevHigh[is].setY(mVal*sVal);
+      _average.setY(is, mVal);
+      _stddevLow.setY(is, mVal/sVal);
+      _stddevHigh.setY(is, mVal*sVal);
     }
     _average.sort();
     _stddevLow.sort();
@@ -142,7 +142,7 @@ namespace HVCore {
     case PeakFrequency:
       break;
     case PeakAmplitude:
-      return _average.at(f).y();
+      return _average.y(f);
     }
     return f;
   }
@@ -186,7 +186,7 @@ namespace HVCore {
       case PeakFrequency:
         break;
       case PeakAmplitude:
-        return _average.at(f0).y();
+        return _average.y(f0);
       }
       return f0;
     } else {
@@ -210,7 +210,7 @@ namespace HVCore {
     case PeakFrequency:
       break;
     case PeakAmplitude:
-      return _average.at(minF).y();
+      return _average.y(minF);
     }
     return minF;
   }
@@ -231,7 +231,7 @@ namespace HVCore {
     case PeakFrequency:
       break;
     case PeakAmplitude:
-      return _average.at(maxF).y();
+      return _average.y(maxF);
     }
     return maxF;
   }
@@ -243,11 +243,11 @@ namespace HVCore {
     ASSERT(n>0);
     double maxF, maxAmpl, f, ampl;
     f=_peaks.at(0).mean();
-    maxAmpl=_average.at(f).y();
+    maxAmpl=_average.y(f);
     maxF=f;
     for(int index=0; index<n; index++) {
       f=_peaks.at(index).mean();
-      ampl=_average.at(f).y();
+      ampl=_average.y(f);
       if(ampl>maxAmpl) {
         maxAmpl=ampl;
         maxF=f;
@@ -351,6 +351,7 @@ namespace HVCore {
 
   /*!
     Calculates peak with \a index between \a minFreq and \a maxFreq expressed in sample index.
+    Peak frequencies are assumed to follow a log-normal distribution.
   */
   void CurveResults::setPeak(int index, int minFreq, int maxFreq)
   {
@@ -359,7 +360,6 @@ namespace HVCore {
     const double * y=_windows->y();
     int imax;
     Statistics s;
-    // We must sum fmax[ic] and (fmax[ic])^2
     for(int ic=0; ic<_windows->curveCount(); ic++) {
       imax=max(y+_windows->xCount()*ic, minFreq, maxFreq);
       if(imax!=-1) {
@@ -377,14 +377,14 @@ namespace HVCore {
     Use the general max() used for each individual time windows.
     See max(double * y, int minFreq, int maxFreq) for the returned value.
   */
-  int CurveResults::averageMax(const Curve<Point2D>& meanCurve) const
+  int CurveResults::averageMax(const Curve<Point2D>& meanCurve)
   {
     TRACE;
     ASSERT(!meanCurve.isEmpty());
-    int ns=frequencySampling().count();
+    int ns=meanCurve.count();
     double * y=new double[ns];
     for(int is=0; is<ns; is++) {
-      y[is]=meanCurve.at(is).y();
+      y[is]=meanCurve.y(is);
     }
     int imax=max(y, 0, ns-1);
     delete [] y;
@@ -395,10 +395,10 @@ namespace HVCore {
     Return the index of the samples with the maximum amplitude or -1 if no maximum can be seen (right at edge of the interval
     from \a minFreq to \a maxFreq).
   */
-  int CurveResults::max(const double * y, int minFreq, int maxFreq) const
+  int CurveResults::max(const double * y, int minFreq, int maxFreq)
   {
     TRACE;
-    ASSERT(minFreq<=maxFreq && maxFreq<frequencySampling().count());
+    ASSERT(minFreq<=maxFreq);
     double valmax=y[ minFreq ], newval;
     int imax=minFreq;
     for(int is=minFreq+1; is<=maxFreq; is++) {
@@ -453,7 +453,7 @@ namespace HVCore {
     if(!log.isEmpty()) {
       QFileInfo fi(fileName);
       QDir d(fi.absolutePath());
-      QFileInfo filog(d.absoluteFilePath(fi.baseName()+".log"));
+      QFileInfo filog(d.absoluteFilePath(fi.completeBaseName()+".log"));
       if(!saveLog(filog.filePath(), log)) {
         success=false;
       }
@@ -498,7 +498,7 @@ namespace HVCore {
     // Write frequency of maxima of the average curve
     int iMax=averageMax(_average);
     if(iMax>-1) {
-      s << QString("# f0 from average\t%1\n").arg(_average[iMax].x());
+      s << QString("# f0 from average\t%1\n").arg(_average.x(iMax));
     }
     // Write f0s
     if(_peaks.isEmpty()) {
@@ -513,7 +513,7 @@ namespace HVCore {
              .arg(p.mean())
              .arg(p.mean()/p.stddev())
              .arg(p.mean()*p.stddev());
-        s << QString("# f%1 amplitude\t%2\n").arg(index).arg(_average.at(p.mean()).y());
+        s << QString("# f%1 amplitude\t%2\n").arg(index).arg(_average.interpole(p.mean()).y());
       }
     }
     s << QString("# Position\t%1\n").arg(coordinates().toString(20));
@@ -523,10 +523,10 @@ namespace HVCore {
     // Write average curves
     int ns=frequencySampling().count();
     for(int is=0; is<ns; is++) {
-      s << QString("%1\t%2\t%3\t%4\n").arg(_average[is].x())
-                                      .arg(_average[is].y())
-                                      .arg(_stddevLow[is].y())
-                                      .arg(_stddevHigh[is].y());
+      s << QString("%1\t%2\t%3\t%4\n").arg(_average.x(is))
+                                      .arg(_average.y(is))
+                                      .arg(_stddevLow.y(is))
+                                      .arg(_stddevHigh.y(is));
     }
     if(s.status()==QTextStream::WriteFailed) {
       App::log(tr("Error while writing to file '%1'\n").arg(fileName));
@@ -608,13 +608,7 @@ namespace HVCore {
     } else {
       std::sort(xValues.begin(), xValues.end());
       unique(xValues);
-      // Function available in 3.4
-      //samp.bestFit(xValues);
-      // Meanwhile use that code:
-      samp.setMinimum(xValues.first());
-      samp.setMaximum(xValues.last());
-      samp.setType(SamplingParameters::Log); // assumes log
-      samp.setCount(_average.count());
+      samp.bestFit(xValues);
     }
     samp.setCount(_average.count());
     setFrequencySampling(samp);
@@ -676,13 +670,13 @@ namespace HVCore {
       if(_stddevHigh.isEmpty()) {
         return 0.0;
       } else {
-        return _stddevHigh.at(_stddevHigh.maximumY()).y();
+        return _stddevHigh.y(_stddevHigh.maximumY());
       }
     } else {
       if(_average.isEmpty()) {
         return 0.0;
       } else {
-        return _average.at(_average.maximumY()).y();
+        return _average.y(_average.maximumY());
       }
     }
   }
@@ -734,7 +728,7 @@ namespace HVCore {
              p.mean(),
              p.mean()/p.stddev(),
              p.mean()*p.stddev(),
-             _average.at(p.mean()).y());
+             _average.interpole(p.mean()).y());
     }
   }
 
@@ -755,13 +749,62 @@ namespace HVCore {
             .arg(p.mean()*p.stddev());
         str+=QString("\nA%1=%2 [%3, %4]")
              .arg(index)
-             .arg(_average.at(p.mean()).y())
-             .arg(_stddevLow.at(p.mean()).y())
-             .arg(_stddevHigh.at(p.mean()).y());
+             .arg(_average.interpole(p.mean()).y())
+             .arg(_stddevLow.interpole(p.mean()).y())
+             .arg(_stddevHigh.interpole(p.mean()).y());
       }
     }
     str+=tr("\nCategory: %1").arg(category());
     return str;
   }
 
+  /*!
+    Returns the peak frequency statistics from a selection of windows.
+    Used for filtering windows according to Cox et al. 2020.
+  */
+  double CurveResults::peak(int index, double minFreq, double maxFreq) const
+  {
+    int iminFreq=frequencySampling().index(minFreq);
+    int imaxFreq=frequencySampling().index(maxFreq);
+    int imax=max(_windows->y()+_windows->xCount()*index, iminFreq, imaxFreq);
+    if(imax>=0) {
+      return _windows->x()[imax];
+    } else {
+      return 0.0;
+    }
+  }
+
+  /*!
+    Returns the peak frequency of the average computed on a selection of windows.
+    Used for filtering windows according to Cox et al. 2020.
+  */
+  double CurveResults::averagePeak(const QVector<bool>& selection, double minFreq, double maxFreq) const
+  {
+    int iminFreq=frequencySampling().index(minFreq);
+    int imaxFreq=frequencySampling().index(maxFreq);
+    const double * x=_windows->x();
+    const double * y=_windows->y();
+
+    int ns=frequencySampling().count();
+    Curve<Point2D> average;
+    average.resize(ns);
+    for(int is=iminFreq; is<=imaxFreq; is++) {
+      average.setX(is, x[is]);
+      Statistics s;
+      for(int ic=_windows->curveCount()-1; ic>=0; ic--) {
+        if(selection.at(ic)) {
+          s.addLog(y[is+ic*ns]);
+        }
+      }
+      average.setY(is, s.meanLog());
+    }
+    average.sort();
+    int index=averageMax(average);
+    if(index>=0) {
+      return average.x(index);
+    } else {
+      return 0.0;
+    }
+  }
+
 } // namespace HVCore
