@@ -51,8 +51,12 @@ namespace QGpCoreMath {
     _nxyz=0;
     _diag=0;
     _last=0;
-    _secondToLast=0;
-    _coordinates=nullptr;
+    _minX=0.0;
+    _minY=0.0;
+    _minZ=0.0;
+    _dX=0.0;
+    _dY=0.0;
+    _dZ=0.0;
   }
 
   /*!
@@ -61,7 +65,6 @@ namespace QGpCoreMath {
   GridSearch::~GridSearch()
   {
     TRACE;
-    delete [] _coordinates;
   }
 
   void GridSearch::setGrid(double minX, double maxX, double dX,
@@ -117,10 +120,6 @@ namespace QGpCoreMath {
       break;
     }
     _last=_nxyz-1;
-    _secondToLast=_last-_diag;
-
-    delete [] _coordinates;
-    _coordinates=new Point[_nxyz];
 
     // For 2D and 1D grids, Y and Z solution is always fixed
     if(_nz==1) {
@@ -131,35 +130,17 @@ namespace QGpCoreMath {
     }
 
     // Round min values to have a grid centered around origin
-    minX=qCeil(minX/dX)*dX;
-    minY=qCeil(minY/dY)*dY;
-    minZ=qCeil(minZ/dZ)*dZ;
-    int k=0;
-    Point p;
-    for(int iz=0; iz<_nz; iz++) {
-      p.setZ(minZ+dZ*static_cast<double>(iz));
-      for(int iy=0; iy<_ny; iy++) {
-        p.setY(minY+dY*static_cast<double>(iy));
-        for(int ix=0; ix<_nx; ix++) {
-          p.setX(minX+dX*static_cast<double>(ix));
-          _coordinates[k++]=p;
-        }
-      }
-    }
-
-    if(_function->initGrid(_nxyz)) {
-      App::log(2, tr("Initializing a grid with %1 x %2 x %3 values\n")
-                        .arg(_nx).arg(_ny).arg(_nz));
-      for(int k=0; k<_nxyz; k++) {
-        _function->initGrid(_coordinates[k], k);
-      }
-      App::log(2, tr("Grid initialized\n"));
-    }
+    _minX=qCeil(minX/dX)*dX;
+    _minY=qCeil(minY/dY)*dY;
+    _minZ=qCeil(minZ/dZ)*dZ;
+    _dX=dX;
+    _dY=dY;
+    _dZ=dZ;
   }
 
   inline void GridSearch::refineMax(int k, double maxVal)
   {
-    double val=refineMax(_coordinates[k-_diag], _coordinates[k+_diag], maxVal);
+    double val=refineMax(coordinates(k-_diag), coordinates(k+_diag), maxVal);
     FunctionSearchMaximum * max=_function->createMaximum(val, _pos);
     if(max) {
       _maxima.append(max);
@@ -198,11 +179,6 @@ namespace QGpCoreMath {
                       .arg(t).arg(nValues).arg((double)t/(double)nValues));
     chrono.start();
 #endif
-    /*if(maxk<_diag) {
-      maxk=_diag;
-    } else if(maxk>_secondToLast) {
-      maxk=_secondToLast;
-    }*/
     if(!isOnEdge(maxk)) {
       refineMax(maxk, maxVal);
       FunctionSearch::globalMax(absThres);
@@ -290,25 +266,32 @@ namespace QGpCoreMath {
     switch(_nz) {
     case 1:
       if(_ny==1) {
-        for(int k=_diag; k<=_secondToLast; k++) {
-          setNeighbors1(k);
-          if(map[k]>map[_neighbors[0]] &&
-             map[k]>map[_neighbors[1]]) {
-            refineMax(k, map[k]);
+        int kx1=_nx-1;
+        for(int kx=1; kx<kx1; kx++) {
+          setNeighbors1(kx);
+          if(map[kx]>map[_neighbors[0]] &&
+             map[kx]>map[_neighbors[1]]) {
+            refineMax(kx, map[kx]);
           }
         }
       } else {
-        for(int k=_diag; k<=_secondToLast; k++) {
-          setNeighbors2(k);
-          if(map[k]>map[_neighbors[0]] &&
-             map[k]>map[_neighbors[1]] &&
-             map[k]>map[_neighbors[2]] &&
-             map[k]>map[_neighbors[3]] &&
-             map[k]>map[_neighbors[4]] &&
-             map[k]>map[_neighbors[5]] &&
-             map[k]>map[_neighbors[6]] &&
-             map[k]>map[_neighbors[7]]) {
-            refineMax(k, map[k]);
+        // We must do nested loops to avoid testing on the outer indexes
+        int nx1=_nx-1;
+        int ky1=_nxy-_nx;
+        for(int ky=_nx; ky<ky1; ky+=_nx) {
+          int kx1=ky+nx1;
+          for(int kx=ky+1; kx<kx1; kx++) {
+            setNeighbors2(kx);
+            if(map[kx]>map[_neighbors[0]] &&
+               map[kx]>map[_neighbors[1]] &&
+               map[kx]>map[_neighbors[2]] &&
+               map[kx]>map[_neighbors[3]] &&
+               map[kx]>map[_neighbors[4]] &&
+               map[kx]>map[_neighbors[5]] &&
+               map[kx]>map[_neighbors[6]] &&
+               map[kx]>map[_neighbors[7]]) {
+              refineMax(kx, map[kx]);
+            }
           }
         }
       }
@@ -318,65 +301,82 @@ namespace QGpCoreMath {
               // Not merged with the classical loop for 2D above
               // because we do not want to find a maximum at the limit
               // between the two planes with distinct Z
-        int n=_nxy-_diag;
-        for(int k=_diag; k<=n; k++) {
-          setNeighbors2(k);
-          if(map[k]>map[_neighbors[0]] &&
-             map[k]>map[_neighbors[1]] &&
-             map[k]>map[_neighbors[2]] &&
-             map[k]>map[_neighbors[3]] &&
-             map[k]>map[_neighbors[4]] &&
-             map[k]>map[_neighbors[5]] &&
-             map[k]>map[_neighbors[6]] &&
-             map[k]>map[_neighbors[7]]) {
-            refineMax(k, map[k]);
+        int nx1=_nx-1;
+        int ky1=_nxy-_nx;
+        for(int ky=_nx; ky<ky1; ky+=_nx) {
+          int kx1=ky+nx1;
+          for(int kx=ky+1; kx<kx1; kx++) {
+            setNeighbors2(kx);
+            if(map[kx]>map[_neighbors[0]] &&
+               map[kx]>map[_neighbors[1]] &&
+               map[kx]>map[_neighbors[2]] &&
+               map[kx]>map[_neighbors[3]] &&
+               map[kx]>map[_neighbors[4]] &&
+               map[kx]>map[_neighbors[5]] &&
+               map[kx]>map[_neighbors[6]] &&
+               map[kx]>map[_neighbors[7]]) {
+              refineMax(kx, map[kx]);
+            }
           }
         }
-        for(int k=_nxy+_diag; k<=_secondToLast; k++) {
-          setNeighbors2(k);
-          if(map[k]>map[_neighbors[0]] &&
-             map[k]>map[_neighbors[1]] &&
-             map[k]>map[_neighbors[2]] &&
-             map[k]>map[_neighbors[3]] &&
-             map[k]>map[_neighbors[4]] &&
-             map[k]>map[_neighbors[5]] &&
-             map[k]>map[_neighbors[6]] &&
-             map[k]>map[_neighbors[7]]) {
-            refineMax(k, map[k]);
+        ky1=_nxyz-_nx;
+        for(int ky=_nxy+_nx; ky<ky1; ky+=_nx) {
+          int kx1=ky+nx1;
+          for(int kx=ky+1; kx<kx1; kx++) {
+            setNeighbors2(kx);
+            if(map[kx]>map[_neighbors[0]] &&
+               map[kx]>map[_neighbors[1]] &&
+               map[kx]>map[_neighbors[2]] &&
+               map[kx]>map[_neighbors[3]] &&
+               map[kx]>map[_neighbors[4]] &&
+               map[kx]>map[_neighbors[5]] &&
+               map[kx]>map[_neighbors[6]] &&
+               map[kx]>map[_neighbors[7]]) {
+              refineMax(kx, map[kx]);
+            }
           }
         }
       }
       break;
     default:
-      for(int k=_diag; k<=_secondToLast; k++) {
-        setNeighbors3(k);
-        if(map[k]>map[_neighbors[0]] &&
-           map[k]>map[_neighbors[1]] &&
-           map[k]>map[_neighbors[2]] &&
-           map[k]>map[_neighbors[3]] &&
-           map[k]>map[_neighbors[4]] &&
-           map[k]>map[_neighbors[5]] &&
-           map[k]>map[_neighbors[6]] &&
-           map[k]>map[_neighbors[7]] &&
-           map[k]>map[_neighbors[8]] &&
-           map[k]>map[_neighbors[9]] &&
-           map[k]>map[_neighbors[10]] &&
-           map[k]>map[_neighbors[11]] &&
-           map[k]>map[_neighbors[12]] &&
-           map[k]>map[_neighbors[13]] &&
-           map[k]>map[_neighbors[14]] &&
-           map[k]>map[_neighbors[15]] &&
-           map[k]>map[_neighbors[16]] &&
-           map[k]>map[_neighbors[17]] &&
-           map[k]>map[_neighbors[18]] &&
-           map[k]>map[_neighbors[19]] &&
-           map[k]>map[_neighbors[20]] &&
-           map[k]>map[_neighbors[21]] &&
-           map[k]>map[_neighbors[22]] &&
-           map[k]>map[_neighbors[23]] &&
-           map[k]>map[_neighbors[24]] &&
-           map[k]>map[_neighbors[25]]) {
-          refineMax(k, map[k]);
+      int nx1=_nx-1;
+      int nxy1=_nxy-_nx;
+      int kz1=_nxyz-_nxy;
+      for(int kz=_nxy; kz<kz1; kz+=_nxy) {
+        int ky1=kz+nxy1;
+        for(int ky=kz+_nx; ky<ky1; ky+=_nx) {
+          int kx1=ky+nx1;
+          for(int kx=ky+1; kx<kx1; kx++) {
+            setNeighbors3(kx);
+            if(map[kx]>map[_neighbors[0]] &&
+               map[kx]>map[_neighbors[1]] &&
+               map[kx]>map[_neighbors[2]] &&
+               map[kx]>map[_neighbors[3]] &&
+               map[kx]>map[_neighbors[4]] &&
+               map[kx]>map[_neighbors[5]] &&
+               map[kx]>map[_neighbors[6]] &&
+               map[kx]>map[_neighbors[7]] &&
+               map[kx]>map[_neighbors[8]] &&
+               map[kx]>map[_neighbors[9]] &&
+               map[kx]>map[_neighbors[10]] &&
+               map[kx]>map[_neighbors[11]] &&
+               map[kx]>map[_neighbors[12]] &&
+               map[kx]>map[_neighbors[13]] &&
+               map[kx]>map[_neighbors[14]] &&
+               map[kx]>map[_neighbors[15]] &&
+               map[kx]>map[_neighbors[16]] &&
+               map[kx]>map[_neighbors[17]] &&
+               map[kx]>map[_neighbors[18]] &&
+               map[kx]>map[_neighbors[19]] &&
+               map[kx]>map[_neighbors[20]] &&
+               map[kx]>map[_neighbors[21]] &&
+               map[kx]>map[_neighbors[22]] &&
+               map[kx]>map[_neighbors[23]] &&
+               map[kx]>map[_neighbors[24]] &&
+               map[kx]>map[_neighbors[25]]) {
+              refineMax(kx, map[kx]);
+            }
+          }
         }
       }
       break;
@@ -401,53 +401,27 @@ namespace QGpCoreMath {
         double val, maxVal;
         int maxIndex=0;
         // Fill in the map with the standard cached grid
-        for(int k=0; k<_nx; k++) {
+        int ky1=_nxy-_nx;
+        for(int kx=0; kx<_nx; kx++) {
           maxVal=-std::numeric_limits<double>::infinity();
-          for(int iy=0; iy<_ny; iy++) {
-            val=value(iy*_nx+k);
+          for(int ky=_nx+kx; ky<ky1; ky+=_nx) {
+            val=value(ky);
             if(val>maxVal) {
               maxVal=val;
-              maxIndex=iy;
+              maxIndex=ky;
             }
           }
-          map[k]=maxVal;
-          index3D[k]=maxIndex*_nx+k;
+          map[kx]=maxVal;
+          index3D[kx]=maxIndex;
         }
         clearMaxima();
-        int n=_nx-2;
-        switch(_ny) {
-        case 2:
-          for(int k=1; k<=n; k++) {
-            setNeighbors1(k);
-            if(map[k]>map[_neighbors[0]] &&
-               map[k]>map[_neighbors[1]]) {
-              int i=index3D[k];
-              if(i<1) {
-                i++;
-              } else if(i>_secondToLast) {
-                i--;
-              }
-              refineMax(i, map[k]);
-            }
+        int kx1=_nx-1;
+        for(int kx=1; kx<kx1; kx++) {
+          setNeighbors1(kx);
+          if(map[kx]>map[_neighbors[0]] &&
+             map[kx]>map[_neighbors[1]]) {
+            refineMax(index3D[kx], map[kx]);
           }
-          break;
-        default:
-          for(int k=1; k<=n; k++) {
-            setNeighbors1(k);
-            if(map[k]>map[_neighbors[0]] &&
-               map[k]>map[_neighbors[1]]) {
-              // Check 3D index is not on the edges, only Z axis has issues
-              // Shift only by one
-              int i=index3D[k];
-              if(i<_diag) {
-                i+=_nx;
-              } else if(i>_secondToLast) {
-                i-=_nx;
-              }
-              refineMax(i, map[k]);
-            }
-          }
-          break;
         }
       }
       break;
@@ -457,44 +431,39 @@ namespace QGpCoreMath {
         double val, maxVal;
         int maxIndex=0;
         // Fill in the map with the standard cached grid
-        for(int k=0; k<_nxy; k++) {
+        int kz1=_nxyz-_nxy;
+        for(int kxy=0; kxy<_nxy; kxy++) {
           maxVal=-std::numeric_limits<double>::infinity();
-          for(int iz=0; iz<_nz; iz++) {
-            val=value(iz*_nxy+k);
+          for(int kz=_nxy+kxy; kz<kz1; kz+=_nxy) {
+            val=value(kz);
             if(val>maxVal) {
               maxVal=val;
-              maxIndex=iz;
+              maxIndex=kz;
             }
           }
-          map[k]=maxVal;
-          index3D[k]=maxIndex*_nxy+k;
+          map[kxy]=maxVal;
+          index3D[kxy]=maxIndex;
         }
         clearMaxima();
         // Do not test if values are null to allow the detection
         // of peaks close to the limit (outside which values are null).
         // All valid values are assumed to be positive.
-        // Peaks that cut by the limits will generate false peaks right on the limit
-        // Proper filtering of theses peaks must be implemented (see FKWorker)
-        int n=_nxy-_nx-2;
-        for(int k=_nx+1; k<=n; k++) {
-          setNeighbors2(k);
-          if(map[k]>map[_neighbors[0]] &&
-             map[k]>map[_neighbors[1]] &&
-             map[k]>map[_neighbors[2]] &&
-             map[k]>map[_neighbors[3]] &&
-             map[k]>map[_neighbors[4]] &&
-             map[k]>map[_neighbors[5]] &&
-             map[k]>map[_neighbors[6]] &&
-             map[k]>map[_neighbors[7]]) {
-            // Check 3D index is not on the edges, only Z axis has issues
-            // Shift only by one
-            int i=index3D[k];
-            if(i<_diag) {
-              i+=_nxy;
-            } else if(i>_secondToLast) {
-              i-=_nxy;
+        int nx1=_nx-1;
+        int ky1=_nxy-_nx;
+        for(int ky=_nx; ky<ky1; ky+=_nx) {
+          int kx1=ky+nx1;
+          for(int kx=ky+1; kx<kx1; kx++) {
+            setNeighbors2(kx);
+            if(map[kx]>map[_neighbors[0]] &&
+               map[kx]>map[_neighbors[1]] &&
+               map[kx]>map[_neighbors[2]] &&
+               map[kx]>map[_neighbors[3]] &&
+               map[kx]>map[_neighbors[4]] &&
+               map[kx]>map[_neighbors[5]] &&
+               map[kx]>map[_neighbors[6]] &&
+               map[kx]>map[_neighbors[7]]) {
+              refineMax(index3D[kx], map[kx]);
             }
-            refineMax(i, map[k]);
           }
         }
       }
