@@ -260,8 +260,8 @@ namespace QGpCoreMath {
       setWidthType(value);
       return true;
     case 1:
-      setWidth(value.toDouble());
-      return true;
+      setWidth(value.toDouble(&ok));
+      return ok;
     case 2:                           // Kept for compatibility
       obsoleteKeyword(keywords, 2);
       setWidthTypeCompatibility(value);
@@ -277,7 +277,7 @@ namespace QGpCoreMath {
       setOrder(value.toInt(&ok));
       return ok;
     case 6:
-      setScaleType(SamplingParameters::type(value));
+      setScaleType(SamplingParameters::scaleType(value));
       return true;
     default:
       break;
@@ -292,8 +292,7 @@ namespace QGpCoreMath {
     log+=prefix+"SMOOTHING_METHOD"+suffix+"="+methodString()+"\n";
     log+=prefix+"SMOOTHING_WIDTH_TYPE"+suffix+"="+widthTypeString()+"\n";
     log+=prefix+"SMOOTHING_WIDTH"+suffix+"="+QString::number(_width)+"\n";
-    log+="Describes the way values are summed: on a linear, log or inversed scale\n";
-    log+=prefix+"SMOOTHING_SCALE_TYPE"+suffix+"="+SamplingParameters::type(scaleType())+"\n";
+    log+=prefix+"SMOOTHING_SCALE_TYPE"+suffix+"="+SamplingParameters::scaleType(scaleType())+"\n";
 #ifdef SAVITZKY_GOLAY
     log+=prefix+"SMOOTHING_ORDER"+suffix+"="+QString::number(_order)+"\n";
 #endif
