@@ -185,10 +185,11 @@ namespace QGpCoreMath {
         ASSERT(param.alpha()>0.0);
         double L=width+1.0;
         double al2=0.5*param.alpha()*L;
-        double Nal2=width-al2;
+        int al2i=qCeil(al2);
+        int Nal2=qFloor(width-al2);
         double f=M_PI/al2;
         int x;
-        for(x=0; x<al2; x++) {
+        for(x=0; x<al2i; x++) {
           _data[x]=0.5*(1.0-cos(f*x));
         }
         for(; x<=Nal2; x++) {
