@@ -54,16 +54,18 @@ namespace QGpCoreStat {
     _manual.flags=0;
     _xColumn=0;
     _yColumn=1;
-    _validColumn=-1;
     _maxVariables=SlownessFrequency;
     _noiseColumn=-1;
     _powerColumn=-1;
+    _validColumn=-1;
     _relativePowerThreshold=0.0;
     _noiseThreshold=std::numeric_limits<double>::infinity();
     _ignoreValid=false;
     _normalize=true;
     _xScaleType=Scale::Linear;
     _yScaleType=Scale::Linear;
+    _ringColumn=-1;
+    _ring=-1;
   }
 
   HistogramReader::~HistogramReader()
@@ -82,97 +84,110 @@ namespace QGpCoreStat {
       if(arg[0]=='-') {
         if(arg=="-x-column") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _xColumn=atoi(argv[i]);
+          _xColumn=CoreApplication::toInt(i, i-1, argv);
           _manual.x.column=true;
         } else if(arg=="-x-sampling") {
           CoreApplication::checkOptionArg(i, argc, argv);
           if(!_xSampling) {
             _xSampling=new SamplingParameters;
           }
-          if(strcmp(argv[i],"inversed")==0) {
-            _xSampling->setInversed(true);
-            _xLogSampling=false;
-          } else if(strcmp(argv[i],"log")==0) {
-            _xSampling->setType(SamplingParameters::Log);
-            _xLogSampling=true;
-          } else if(strcmp(argv[i],"asis")==0) {
+          if(strcmp(argv[i],"asis")==0) {
             delete _xSampling;
             _xSampling=nullptr;
             _manual.xValuesAsIs=true;
           } else {
-            _xSampling->setType(SamplingParameters::Linear);
-            _xLogSampling=false;
+            if(!_xSampling->setScaleType(QString(argv[i]))) {
+              App::log(tr("Error setting X sampling\n"));
+              return false;
+            }
+            _xLogSampling=_xSampling->scaleType()==SamplingParameters::Log;
           }
-          _manual.x.gridScaleType=true;
+           _manual.x.gridScaleType=true;
         } else if(arg=="-x-min") {
           CoreApplication::checkOptionArg(i, argc, argv);
           if(!_xSampling) {
             _xSampling=new SamplingParameters;
           }
-          _xSampling->setMinimum(atof(argv[i]));
+          _xSampling->setMinimum(CoreApplication::toDouble(i, i-1, argv));
           _manual.x.minimum=true;
         } else if(arg=="-x-max") {
           CoreApplication::checkOptionArg(i, argc, argv);
           if(!_xSampling) {
             _xSampling=new SamplingParameters;
           }
-          _xSampling->setMaximum(atof(argv[i]));
+          _xSampling->setMaximum(CoreApplication::toDouble(i, i-1, argv));
           _manual.x.maximum=true;
         } else if(arg=="-x-count") {
           CoreApplication::checkOptionArg(i, argc, argv);
           if(!_xSampling) {
             _xSampling=new SamplingParameters;
           }
-          _xSampling->setCount(atoi(argv[i]));
-          _manual.x.count=true;
+          _xSampling->setCount(CoreApplication::toInt(i, i-1, argv));
+          _manual.x.stepCount=true;
           if(_xSampling->count()<=0) {
             App::log(tr("negative or null number of X samples (option -x-count)\n") );
             return false;
           }
+        } else if(arg=="-x-step") {
+          CoreApplication::checkOptionArg(i, argc, argv);
+          if(!_xSampling) {
+            _xSampling=new SamplingParameters;
+          }
+          _xSampling->setStep(CoreApplication::toDouble(i, i-1, argv));
+          _manual.x.stepCount=true;
+          if(_xSampling->step()<=0) {
+            App::log(tr("negative or null step for X samples (option -x-step)\n") );
+            return false;
+          }
         } else if(arg=="-x-title") {
           CoreApplication::checkOptionArg(i, argc, argv);
           _xTitle=argv[i];
         } else if(arg=="-y-column") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _yColumn=atoi(argv[i]);
+          _yColumn=CoreApplication::toInt(i, i-1, argv);
           _manual.y.column=true;
         } else if(arg=="-y-sampling") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          if(strcmp(argv[i],"inversed")==0) {
-            _ySampling.setInversed(true);
-          } else if(strcmp(argv[i],"log")==0) {
-            _ySampling.setType(SamplingParameters::Log);
-          } else {
-            _ySampling.setType(SamplingParameters::Linear);
+          if(!_ySampling.setScaleType(QString(argv[i]))) {
+            App::log(tr("Error setting Y sampling\n"));
+            return false;
           }
           _manual.y.gridScaleType=true;
         } else if(arg=="-no-normalize") {
           _normalize=false;
         } else if(arg=="-y-min") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _ySampling.setMinimum(atof(argv[i]));
+          _ySampling.setMinimum(CoreApplication::toDouble(i, i-1, argv));
           _manual.y.minimum=true;
         } else if(arg=="-y-max") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _ySampling.setMaximum(atof(argv[i]));
+          _ySampling.setMaximum(CoreApplication::toDouble(i, i-1, argv));
           _manual.y.maximum=true;
         } else if(arg=="-y-count") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _ySampling.setCount(atoi(argv[i]));
-          _manual.y.count=true;
+          _ySampling.setCount(CoreApplication::toInt(i, i-1, argv));
+          _manual.y.stepCount=true;
           if(_ySampling.count()<=0) {
             App::log(tr("negative or null number of Y samples (option -y-count)\n") );
             return false;
           }
+        } else if(arg=="-y-step") {
+          CoreApplication::checkOptionArg(i, argc, argv);
+          _ySampling.setStep(CoreApplication::toDouble(i, i-1, argv));
+          _manual.y.stepCount=true;
+          if(_ySampling.step()<=0) {
+            App::log(tr("negative or null step for y samples (option -y-step)\n") );
+            return false;
+          }
         } else if(arg=="-y-factor") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _yFactor=atof(argv[i]);
+          _yFactor=CoreApplication::toDouble(i, i-1, argv);
         } else if(arg=="-y-title") {
           CoreApplication::checkOptionArg(i, argc, argv);
           _yTitle=argv[i];
         } else if(arg=="-valid-column") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _validColumn=atoi(argv[i]);
+          _validColumn=CoreApplication::toInt(i, i-1, argv);
           _manual.validColumn=true;
         } else if(arg=="-grid") {
           _action=Grid;
@@ -186,39 +201,44 @@ namespace QGpCoreStat {
           _action=GaussMix;
         } else if(arg=="-plot") {
           _action=Plot;
+        } else if(arg=="-min-velocity") {
+          _action=MinimumVelocity;
         } else if(arg=="-max") {
           CoreApplication::checkOptionArg(i, argc, argv);
           _exportMaxFileName=argv[i];
-          _action=Max;
+          _action=MaxFile;
         } else if(arg=="-pattern" || arg=="-p") {
           CoreApplication::checkOptionArg(i, argc, argv);
           _pattern=argv[i];
+        } else if(arg=="-ring") {
+          CoreApplication::checkOptionArg(i, argc, argv);
+          _ring=CoreApplication::toInt(i, i-1, argv);
         } else if(arg=="-ignore-valid") {
           _ignoreValid=true;
         } else if(arg=="-rel-power") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _relativePowerThreshold=atof(argv[i]);
+          _relativePowerThreshold=CoreApplication::toDouble(i, i-1, argv);
         } else if(arg=="-max-noise") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _noiseThreshold=atof(argv[i]);
+          _noiseThreshold=CoreApplication::toDouble(i, i-1, argv);
         } else if(arg=="-gm-type") {
           CoreApplication::checkOptionArg(i, argc, argv);
           _gaussMix.setInversionType(argv[i]);
         } else if(arg=="-gm-count") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _gaussMix.setModeCount(atoi(argv[i]));
+          _gaussMix.setModeCount(CoreApplication::toInt(i, i-1, argv));
         } else if(arg=="-gm-kmin") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _gaussMix.setKmin(atof(argv[i]));
+          _gaussMix.setKmin(CoreApplication::toDouble(i, i-1, argv));
         } else if(arg=="-gm-kmax") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _gaussMix.setKmax(atof(argv[i]));
+          _gaussMix.setKmax(CoreApplication::toDouble(i, i-1, argv));
         } else if(arg=="-gm-relerr") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _gaussMix.setRelativeError(atof(argv[i]));
+          _gaussMix.setRelativeError(CoreApplication::toDouble(i, i-1, argv));
         } else if(arg=="-gm-abserr") {
           CoreApplication::checkOptionArg(i, argc, argv);
-          _gaussMix.setAbsoluteError(atof(argv[i]));
+          _gaussMix.setAbsoluteError(CoreApplication::toDouble(i, i-1, argv));
         } else if(arg=="-gm-verbose") {
           _gaussMix.setVerbose(true);
         } else if(arg=="-curve") {
@@ -284,7 +304,8 @@ namespace QGpCoreStat {
                                           "  inversed   regular sampling on an inversed");
     h->addOption("-x-min <MIN>","Minimum of range for X axis (default=deduced from data)");
     h->addOption("-x-max <MAX>","Maximum of range for X axis (default=deduced from data)");
-    h->addOption("-x-count <COUNT>","Number of samples along X (default=100)");
+    h->addOption("-x-step <STEP>","Step between samples along X.");
+    h->addOption("-x-count <COUNT>","Number of samples along X.");
     h->addOption("-x-title <TITLE>","Title for X axis");
   }
 
@@ -298,7 +319,8 @@ namespace QGpCoreStat {
                                           "  inversed   regular sampling on an inversed");
     h->addOption("-y-min <MIN>","Minimum of range for Y axis (default=deduced from data)");
     h->addOption("-y-max <MAX>","Maximum of range for Y axis (default=deduced from data)");
-    h->addOption("-y-count <COUNT>","Number of samples along Y (default=100)");
+    h->addOption("-x-step <STEP>","Step between samples along Y.");
+    h->addOption("-y-count <COUNT>","Number of samples along Y");
     h->addOption("-y-factor <FACTOR>","Multiply all Y value by FACTOR (default=1)");
     h->addOption("-y-title <TITLE>","Title for Y axis");
   }
@@ -317,73 +339,70 @@ namespace QGpCoreStat {
     _xValues=x;
     std::sort(_xValues.begin(), _xValues.end());
     unique(_xValues);
-    double sumLin=0.0, sumLog=0.0;
-    double stepLin=std::numeric_limits<double>::infinity(), stepLog=std::numeric_limits<double>::infinity();
-    if(!_xSampling || (!_manual.x.gridScaleType && !_manual.x.count)) {
-      double dLin, dLog;
-      for(int i=2; i<_xValues.count(); i++) {
-        dLin=_xValues.at(i-1)-_xValues.at(i-2);
-        dLog=_xValues.at(i-1)/_xValues.at(i-2);
-        sumLin+=fabs(dLin-(_xValues.at(i)-_xValues.at(i-1)));
-        sumLog+=fabs(dLog-(_xValues.at(i)/_xValues.at(i-1)));
-        dLin=fabs(dLin);
-        dLog=fabs(dLog);
-        if(dLin<stepLin) {
-          stepLin=dLin;
-        }
-        if(dLog<stepLog) {
-          stepLog=dLog;
-        }
-      }
-      if(sumLin<sumLog) {
+    SamplingParameters sampling;
+    if(!sampling.bestFit(_xValues)) {
+      App::log(tr("Using X values 'as is'\n"));
+      _manual.xValuesAsIs=true;
+    }
+    if(sampling.count()>1000) { // Manually it is possible to have a higher number of samples
+      sampling.setCount(1000);  // but for automatic scaling, it is limited.
+    }
+    if(!_xSampling || (!_manual.x.gridScaleType && !_manual.x.stepCount)) {
+      switch(sampling.scaleType()) {
+      case SamplingParameters::Linear:
         if(!_xSampling) {
           App::log(tr("Identified a linear scale for X\n"));
         }
         _xLogSampling=false;
-      } else {
+        break;
+      case SamplingParameters::Log:
         if(!_xSampling) {
           App::log(tr("Identified a log scale for X\n"));
         }
         _xLogSampling=true;
+        break;
+      case SamplingParameters::Inversed:
+        if(!_xSampling) {
+          App::log(tr("Identified an inversed scale for X\n"));
+        }
+        _xLogSampling=false;
+        break;
       }
     }
-    if(!_manual.xValuesAsIs && !_xSampling) {
-      _xSampling=new SamplingParameters;
+    if(_manual.xValuesAsIs) {
+      return true;
     }
     if(_xSampling) {
       if(_xValues.count()<3) {
-        _xSampling->setType(SamplingParameters::Linear);
+        _xSampling->setScaleType(SamplingParameters::Linear);
         _xSampling->setCount(_xValues.count());
       }
-      if(!_xValues.isEmpty()) {
-        if(!_manual.x.minimum) {
-          _xSampling->setMinimum(_xValues.first());
-        }
-        if(!_manual.x.maximum) {
-          _xSampling->setMaximum(_xValues.last());
-        }
+      if(!_manual.x.minimum) {
+        _xSampling->setMinimum(sampling.minimum());
       }
-      if(!_manual.x.gridScaleType && !_manual.x.count) {
-        if(sumLin<sumLog) {
-          App::log(tr("Identified a linear scale for X with minimum step %1\n").arg(stepLin) );
-          _xSampling->setType(SamplingParameters::Linear);
-          _xSampling->setStep(stepLin);
+      if(!_manual.x.maximum) {
+        _xSampling->setMaximum(sampling.maximum());
+      }
+      if(!_manual.x.gridScaleType && !_manual.x.stepCount) {
+        if(sampling.scaleType()==SamplingParameters::Linear) {
+          App::log(tr("Identified a linear scale for X with minimum step %1\n").arg(sampling.step()));
+          _xSampling->setScaleType(SamplingParameters::Linear);
+          _xSampling->setStep(sampling.step());
         } else {
-          App::log(tr("Identified a log scale for X with minimum step %1\n").arg(stepLog) );
-          _xSampling->setType(SamplingParameters::Log);
-          if(_xSampling->minimum()<=0.0 || _xSampling->maximum()<=0.0) {
-            App::log(tr("null or negative values not allowed for log and inversed scales (X axis)\n") );
-            return false;
-          }
-          _xSampling->setStep(stepLog);
+          App::log(tr("Identified a log scale for X with minimum step %1\n").arg(sampling.step()));
+          _xSampling->setScaleType(SamplingParameters::Log);
+          _xSampling->setStep(sampling.step());
         }
       }
-      if(_xSampling->type()==SamplingParameters::Log || _xSampling->isInversed()) {
-        if(_xSampling->minimum()<=0.0 || _xSampling->maximum()<=0.0) {
-          App::log(tr("null or negative values not allowed for log and inversed scales (X axis)\n") );
-          return false;
-        }
+      if(!_manual.x.stepCount) {
+        _xSampling->setStep(sampling.step());
       }
+      if(!_xSampling->isValidRange(true)) {
+        App::log(tr("Error while checking X axis\n"));
+        return false;
+      }
+    } else {
+      _xSampling=new SamplingParameters(sampling);
     }
     return true;
   }
@@ -391,33 +410,32 @@ namespace QGpCoreStat {
   bool HistogramReader::setDefaultYLimits()
   {
     QVector<Histogram2D::Sample>::iterator it;
-    int ny=_ySampling.count();
+    double m;
     if(!_manual.y.minimum) {
       it=_samples.begin();
-      _ySampling.setMinimum(it->y());
+       m=it->y();
       for(it++; it!=_samples.end(); it++) {
-        if(it->y()<_ySampling.minimum()) {
-          _ySampling.setMinimum(it->y());
+        if(it->y()<m) {
+          m=it->y();
         }
       }
+      _ySampling.setMinimum(m);
     }
     if(!_manual.y.maximum) {
       it=_samples.begin();
-      _ySampling.setMaximum(it->y());
+      m=it->y();
       for(it++; it!=_samples.end(); it++) {
-        if(it->y()>_ySampling.maximum()) {
-          _ySampling.setMaximum(it->y());
+        if(it->y()>m) {
+          m=it->y();
         }
       }
+      _ySampling.setMaximum(m);
     }
-    _ySampling.setCount(ny);
-
-    if(_ySampling.type()==SamplingParameters::Log || _ySampling.isInversed()) {
-      if(_ySampling.minimum()<=0.0 || _ySampling.maximum()<=0.0) {
-        App::log(tr("null or negative values not allowed for log and inversed scales (Y axis)\n") );
-        return false;
-      }
+    if(!_ySampling.isValidRange(true)) {
+      App::log(tr("Error while checking Y axis\n"));
+      return false;
     }
+    _ySampling.includeLimits();
     return true;
   }
 
@@ -452,9 +470,15 @@ namespace QGpCoreStat {
         if(!maxFormatHeader && line.startsWith("# MAX FORMAT RELEASE")) {
           maxFormatHeader=true;
         }
-        if(line.startsWith("# abs_time frequency polarization slowness azimuth ") ||
-           line.startsWith("# time frequency polarization slowness azimuth ")) {
-          configureMaxFile(line);
+        if(line.startsWith("# abs_time frequency slowness azimuth ") ||
+           line.startsWith("# abs_time frequency polarization slowness azimuth ") ||  // kept for compatibility
+           line.startsWith("# time frequency polarization slowness azimuth ")) {      // kept for compatibility
+          if(!configureFKMaxFile(line)) {
+            return false;
+          }
+          maxFormatHeader=false;
+        } else if(line.startsWith("# abs_time frequency polarization ring autocorr ")) {
+          configureSPACMaxFile();
           maxFormatHeader=false;
         } else if(line=="# seconds from start | cfreq | slow | az | math-phi | semblance | beampow") {
           configureCapMaxFile();
@@ -474,6 +498,7 @@ namespace QGpCoreStat {
     QString line;
     if((_powerColumn<0 || _relativePowerThreshold==0.0) &&
        (_noiseColumn<0 || _noiseThreshold==std::numeric_limits<double>::infinity()) &&
+       (_ringColumn<0 || _ring<0) &&
        !_manual.x.minimum && !_manual.x.maximum && !_manual.y.minimum && !_manual.y.maximum) {
       Histogram2D::Sample sample;
       if(_yFactor==1.0 && _maxVariables==SlownessFrequency) {
@@ -522,10 +547,7 @@ namespace QGpCoreStat {
       SampleFilter::Sample sample;
       SampleFilter filter;
       if(File::isDataLine(lastLine, _pattern)) {
-        if(sample.read(lastLine, _xColumn, _yColumn, _noiseColumn, _powerColumn, _validColumn)) {
-          if(_yFactor!=1.0) {
-            sample.setY(sample.y()*_yFactor);
-          }
+        if(sample.read(lastLine, _xColumn, _yColumn, _noiseColumn, _powerColumn, _ringColumn, _validColumn)) {
           filter.append(sample);
         } else {
           ok=false;
@@ -533,10 +555,7 @@ namespace QGpCoreStat {
       }
       while(!s.atEnd()) {
         if(File::readLine(s, line, _pattern)) {
-          if(sample.read(line, _xColumn, _yColumn, _noiseColumn, _powerColumn, _validColumn)) {
-            if(_yFactor!=1.0) {
-              sample.setY(sample.y()*_yFactor);
-            }
+          if(sample.read(line, _xColumn, _yColumn, _noiseColumn, _powerColumn, _ringColumn, _validColumn)) {
             filter.append(sample);
           } else {
             ok=false;
@@ -548,6 +567,13 @@ namespace QGpCoreStat {
       progress.end(filter.count());
       if(ok) {
         App::log(tr("Loaded %1 samples\n").arg(filter.count()) );
+        if(_yFactor!=1.0 || _maxVariables!=SlownessFrequency) {
+          for(SampleFilter::iterator it=filter.begin(); it!=filter.end(); it++) {
+            SampleFilter::Sample& s=*it;
+            s.setY(s.y()*_yFactor);
+            s=transformSample(s);
+          }
+        }
         if(_relativePowerThreshold>0.0) {
           App::log(tr("Filtering power data < %1\n").arg(_relativePowerThreshold) );
           filter.filterPower(_relativePowerThreshold);
@@ -574,17 +600,11 @@ namespace QGpCoreStat {
           App::log(tr("Filtering Y > %1\n").arg(_ySampling.maximum()));
           filter.filterYMaximum(_ySampling.maximum());
         }
-        QVector<Histogram2D::Sample> samples=filter.samples();
-        if(_maxVariables!=SlownessFrequency) {
-          for(QVector<Histogram2D::Sample>::iterator it=samples.begin();
-              it!=samples.end(); it++) {
-            Histogram2D::Sample& s=*it;
-            s=transformSample(s);
-            _samples.append(s);
-          }
-        } else {
-          _samples.append(samples);
+        if(_ring>=0) {
+          App::log(tr("Filtering all rings other than %1\n").arg(_ring));
+          filter.filterRing(_ring);
         }
+        _samples.append(filter.samples());
       }
     }
     return ok;
@@ -641,29 +661,23 @@ namespace QGpCoreStat {
     return ns;
   }
 
-  Histogram2D * HistogramReader::histogram() const
+  Histogram2D * HistogramReader::histogram(const SamplingParameters& ySampling) const
   {
     TRACE;
     Histogram2D * h;
-    static const QString msg=tr("%5 axis from %1 to %2 with %3 samples (step=%4)\n");
     if(_xSampling) {
-       h=new Histogram2D(_xSampling->count(), _ySampling.count());
-       App::log(msg.arg(_xSampling->value(0))
-                   .arg(_xSampling->value(_xSampling->count()-1))
-                   .arg(_xSampling->count())
-                   .arg(_xSampling->step())
-                   .arg("X"));
+       h=new Histogram2D(_xSampling->count(), ySampling.count());
+       App::log(tr("X axis %1\n").arg(_xSampling->toUserString()));
        if(!h->setXSampling(*_xSampling)) {
          delete h;
          return nullptr;
        }
     } else {
-      h=new Histogram2D(_xValues.count(), _ySampling.count());
+      h=new Histogram2D(_xValues.count(), ySampling.count());
+      static const QString msg=tr("X axis from %1 to %2 with %3 sample (step=as is)\n");
       App::log(msg.arg(_xValues.first())
                   .arg(_xValues.last())
-                  .arg(_xValues.count())
-                  .arg("asis")
-                  .arg("X"));
+                  .arg(_xValues.count()));
       if(!h->setXSampling(_xValues)) {
         delete h;
         return nullptr;
@@ -680,12 +694,9 @@ namespace QGpCoreStat {
         h->setXScaleType(Scale::Linear);
       }
     }
-    App::log(msg.arg(_ySampling.value(0))
-                .arg(_ySampling.value(_ySampling.count()-1))
-                .arg(_ySampling.count())
-                .arg(_ySampling.step())
-                .arg("Y"));
-    if(!h->setYSampling(_ySampling)) {
+
+    App::log(tr("Y axis %1\n").arg(ySampling.toUserString()));
+    if(!ySampling.isValid() || !h->setYSampling(ySampling)) {
       delete h;
       return nullptr;
     }
@@ -694,50 +705,86 @@ namespace QGpCoreStat {
     if(_manual.y.viewScaleType) {
       h->setYScaleType(_yScaleType);
     } else {
-      if(_ySampling.type()==SamplingParameters::Log) {
-        h->setYScaleType(Scale::Log);
-      } else {
+      switch(ySampling.scaleType()) {
+      case SamplingParameters::Linear:
         h->setYScaleType(Scale::Linear);
+        break;
+      case SamplingParameters::Log:
+        h->setYScaleType(Scale::Log);
+        break;
+      case SamplingParameters::Inversed:
+        h->setYScaleType(Scale::Inversed);
+        break;
       }
     }
-
-    h->setSamples(_samples);
     return h;
   }
 
-  void HistogramReader::configureMaxFile(const QString& line)
+  Histogram2D * HistogramReader::histogram() const
   {
+    TRACE;
+    Histogram2D * h=histogram(_ySampling);
+    if(h) {
+      h->setSamples(_samples);
+    }
+    return h;
+  }
+
+  bool HistogramReader::configureFKMaxFile(const QString& line)
+  {
+    int slownessColumn=-1, azimuthColumn=-1, ellipticityColumn=-1;
     _validColumn=-1;
-    if(line=="# abs_time frequency polarization slowness azimuth ellipticity noise power valid") {
-      if(!_ignoreValid) {
-        _validColumn=8;
-      }
-      _noiseColumn=6;
-      _powerColumn=7;
-    } else if(line=="# time frequency polarization slowness azimuth ellipticity noise power valid") {
-      if(!_ignoreValid) {
-        _validColumn=8;
-      }
-      _noiseColumn=6;
-      _powerColumn=7;
-    } else if(line=="# time frequency polarization slowness azimuth ellipticity power valid") {
+    if(line=="# abs_time frequency slowness azimuth ellipticity noise power valid") {
+      slownessColumn=2;
+      azimuthColumn=3;
+      ellipticityColumn=4;
+      _noiseColumn=5;
+      _powerColumn=6;
       if(!_ignoreValid) {
         _validColumn=7;
       }
+    } else if(line=="# abs_time frequency polarization slowness azimuth ellipticity noise power valid" ||
+              line=="# time frequency polarization slowness azimuth ellipticity noise power valid") {
+      // Compatibility
+      slownessColumn=3;
+      azimuthColumn=4;
+      ellipticityColumn=5;
+      _noiseColumn=6;
+      _powerColumn=7;
+      if(!_ignoreValid) {
+        _validColumn=8;
+      }
+    } else if(line=="# time frequency polarization slowness azimuth ellipticity power valid") {
+      // Compatibility
+      slownessColumn=3;
+      azimuthColumn=4;
+      ellipticityColumn=5;
       _powerColumn=6;
+      if(!_ignoreValid) {
+        _validColumn=7;
+      }
     } else if(line=="# time frequency polarization slowness azimuth ellipticity valid") {
       // Compatibility for results from October 2017 to February 2018
+      slownessColumn=3;
+      azimuthColumn=4;
+      ellipticityColumn=5;
       if(!_ignoreValid) {
         _validColumn=6;
       }
     } else if(line=="# time frequency polarization slowness azimuth valid") {
-      // Compatibility for results from September to October 2017
+      // Compatibility for results from September and October 2017
       // Option -ell* will ouput junk for these results
+      slownessColumn=3;
+      azimuthColumn=4;
       if(!_ignoreValid) {
         _validColumn=5;
       }
     }
-    _headerLine="# abs_time frequency polarization slowness azimuth ellipticity noise power valid\n";
+    if(slownessColumn<0) {
+      App::log(tr("Unrecognized header line:\n  '%1'\n").arg(line));
+      return false;
+    }
+    _headerLine="# abs_time frequency slowness azimuth ellipticity noise power valid\n";
 
     // X title
     switch(_maxVariables) {
@@ -776,46 +823,46 @@ namespace QGpCoreStat {
     case SlownessWaveLength:
       _yTitle=tr("Slowness (s/m)");
       _yTitleInversedScale=tr("Velocity (m/s)");
-      _yColumn=3;
+      _yColumn=slownessColumn;
       break;
     case WaveNumberFrequency:
       _yTitle=tr("Wave number (rad/m)");
-      _yColumn=3;
+      _yColumn=slownessColumn;
       break;
     case WaveLengthFrequency:
       _yTitle=tr("Wave length (m)");
-      _yColumn=3;
+      _yColumn=slownessColumn;
       break;
     case FrequencyWaveNumber:
       _yTitle=tr("Frequency (Hz)");
       _yTitleInversedScale=tr("Period (s)");
-      _yColumn=3;
+      _yColumn=slownessColumn;
       break;
     case AzimuthFrequency:
       _yTitle=tr("Azimuth (degrees)");
-      _yColumn=4;
+      _yColumn=azimuthColumn;
       break;
     case EllipticityFrequency:
       _yTitle=tr("Ellipticity");
-      _yColumn=5;
+      _yColumn=ellipticityColumn;
       break;
     case EllipticityAbsFrequency:
       _yTitle=tr("Ellipticity");
-      _yColumn=5;
+      _yColumn=ellipticityColumn;
       break;
     case EllipticityDegFrequency:
     case EllipticityDegInversedFrequency:
       _yTitle=tr("Ellipticity angle (deg)");
-      _yColumn=5;
+      _yColumn=ellipticityColumn;
       break;
     case EllipticityRadFrequency:
     case EllipticityRadInversedFrequency:
       _yTitle=tr("Ellipticity angle (rad)");
-      _yColumn=5;
+      _yColumn=ellipticityColumn;
       break;
     case NoiseFrequency:
       _yTitle=tr("Relative incoherent noise");
-      _yColumn=6;
+      _yColumn=_noiseColumn;
       break;
     }
     // Y limits and sampling
@@ -825,8 +872,12 @@ namespace QGpCoreStat {
     case SlownessWaveLength:
     case SlownessWaveNumber:
       if(!_manual.y.gridScaleType) {
-        _ySampling.setType(SamplingParameters::Log);
+        _ySampling.setScaleType(SamplingParameters::Log);
         _manual.y.gridScaleType=true;
+        if(!_manual.y.stepCount) {
+          _ySampling.setStep(1.015);
+          _manual.y.stepCount=true;
+        }
         _yScaleType=Scale::InversedLog;
         _manual.y.viewScaleType=true;
       }
@@ -845,9 +896,9 @@ namespace QGpCoreStat {
         _ySampling.setMaximum(360.0);
         _manual.y.maximum=true;
       }
-      if(!_manual.y.count) {
-        _ySampling.setCount(72);
-        _manual.y.count=true;
+      if(!_manual.y.stepCount) {
+        _ySampling.setStep(5.0);
+        _manual.y.stepCount=true;
       }
       break;
     case EllipticityFrequency:
@@ -859,9 +910,9 @@ namespace QGpCoreStat {
         _ySampling.setMaximum(10.0);
         _manual.y.maximum=true;
       }
-      if(!_manual.y.count) {
+      if(!_manual.y.stepCount) {
         _ySampling.setCount(200);
-        _manual.y.count=true;
+        _manual.y.stepCount=true;
       }
       break;
     case EllipticityAbsFrequency:
@@ -873,11 +924,11 @@ namespace QGpCoreStat {
         _ySampling.setMaximum(10.0);
         _manual.y.maximum=true;
       }
-      if(!_manual.y.count) {
-        _ySampling.setCount(200);
-        _manual.y.count=true;
+      _ySampling.setScaleType(SamplingParameters::Log);
+      if(!_manual.y.stepCount) {
+        _ySampling.setStep(1.025);
+        _manual.y.stepCount=true;
       }
-      _ySampling.setType(SamplingParameters::Log);
       break;
     case EllipticityDegFrequency:
     case EllipticityDegInversedFrequency:
@@ -889,9 +940,9 @@ namespace QGpCoreStat {
         _ySampling.setMaximum(90.0);
         _manual.y.maximum=true;
       }
-      if(!_manual.y.count) {
-        _ySampling.setCount(200);
-        _manual.y.count=true;
+      if(!_manual.y.stepCount) {
+        _ySampling.setStep(1.0);
+        _manual.y.stepCount=true;
       }
       break;
     case EllipticityRadFrequency:
@@ -904,9 +955,9 @@ namespace QGpCoreStat {
         _ySampling.setMaximum(0.5*M_PI);
         _manual.y.maximum=true;
       }
-      if(!_manual.y.count) {
+      if(!_manual.y.stepCount) {
         _ySampling.setCount(200);
-        _manual.y.count=true;
+        _manual.y.stepCount=true;
       }
       break;
     case NoiseFrequency:
@@ -915,17 +966,17 @@ namespace QGpCoreStat {
         _manual.y.minimum=true;
       }
       if(!_manual.y.maximum) {
-        _ySampling.setMaximum(10.0);
+        _ySampling.setMaximum(100.0);
         _manual.y.maximum=true;
       }
-      if(!_manual.y.count) {
-        _ySampling.setCount(50);
-        _manual.y.count=true;
-      }
       if(!_manual.y.gridScaleType) {
-        _ySampling.setType(SamplingParameters::Log);
+        _ySampling.setScaleType(SamplingParameters::Log);
         _manual.y.gridScaleType=true;
       }
+      if(!_manual.y.stepCount) {
+        _ySampling.setStep(1.05);
+        _manual.y.stepCount=true;
+      }
       break;
     }
     switch(_maxVariables) {
@@ -942,17 +993,46 @@ namespace QGpCoreStat {
     case EllipticityRadInversedFrequency:
     case NoiseFrequency:
       break;
-    case SlownessWaveNumber:
     case SlownessWaveLength:
+      if(!_xSampling) {
+        _xSampling=new SamplingParameters;
+        _xSampling->setStep(1.025);
+      }
+      if(!_manual.x.gridScaleType) {
+        _xSampling->setScaleType(SamplingParameters::Log);
+        _manual.x.gridScaleType=true;
+      }
+      break;
+    case SlownessWaveNumber:
     case FrequencyWaveNumber:
-      //_xCount=50; // min and max should correspond to kmin and kmax, should be manually specified
-      //_xSamplingType=LinearScale;
-      //_manualXScale=true;
+      if(!_xSampling) {
+        _xSampling=new SamplingParameters;
+        _xSampling->setStep(1.025);
+      }
+      if(!_manual.x.gridScaleType) {
+        _xSampling->setScaleType(SamplingParameters::Log);
+        _manual.x.gridScaleType=true;
+      }
       break;
     }
-    if(!_pattern.isEmpty()) {
+    if(!_pattern.isEmpty()) { // Kept for compatibility
       App::log(tr("Slowness max file with pattern '%1'\n").arg(_pattern) );
     }
+    return true;
+  }
+
+
+  void HistogramReader::configureSPACMaxFile()
+  {
+    _validColumn=5;
+    _headerLine="# abs_time frequency polarization ring autocorr valid\n";
+    _xTitle=tr("Frequency (Hz)");
+    _xTitleInversedScale=tr("Period (s)");
+    _xColumn=1;
+    _yTitle=tr("Autocorr ratio");
+    _yTitleInversedScale="";
+    _yColumn=4;
+    _ringColumn=3;
   }
 
   void HistogramReader::configureCapMaxFile()
@@ -970,7 +1050,7 @@ namespace QGpCoreStat {
         _manual.y.minimum=true;
       }
       if(!_manual.y.gridScaleType) {
-        _ySampling.setType(SamplingParameters::Log);
+        _ySampling.setScaleType(SamplingParameters::Log);
         _manual.y.gridScaleType=true;
       }
       _validColumn=-1;
@@ -1007,5 +1087,4 @@ namespace QGpCoreStat {
     App::log(tr("Old slowness max file (before 2017)\n") );
   }
 
-
 } // namespace QGpCoreStat
