@@ -5,6 +5,8 @@
 
 #include "Densities.h"
 #include "GaussianMixtureParameters.h"
+#include "Histogram.h"
+#include "Histogram2D.h"
 #include "HistogramDensities.h"
 #include "HistogramReader.h"
 #include "Likelihood.h"
