@@ -41,13 +41,11 @@
 #include <cxxabi.h>
 #ifndef Q_OS_WIN
 #  include <sys/ioctl.h>
+#  include <sys/sysctl.h>
 #  include <fcntl.h>
 #  include <sys/types.h>
 #  include <sys/param.h>
 #endif
-#ifdef Q_OS_MAC
-#  include <sys/sysctl.h>
-#endif
 
 #include "CoreApplicationPrivate.h"
 #include "StandardStream.h"
@@ -154,7 +152,7 @@ namespace QGpCoreTools {
         ::exit(0);
       } else if(arg=="--verbosity" || arg=="-verbosity") {
         checkOptionArg(i, argc, argv, true);
-        AbstractStream::setApplicationVerbosity(atoi(argv[i]));
+        AbstractStream::setApplicationVerbosity(toInt(i, i-1, argv));
       } else if(arg=="-warning-critical") {
         _warningAsError=true;
       } else if(arg=="-locale") {
@@ -162,13 +160,13 @@ namespace QGpCoreTools {
         QLocale::setDefault(QLocale(argv[i]));
       } else if(arg=="-sleep") {
         checkOptionArg(i, argc, argv, true);
-        sleep(atoi(argv[i])*1000);
+        sleep(toInt(i, i-1, argv)*1000);
       } else if(arg=="-j" || arg=="-jobs") {
         checkOptionArg(i, argc, argv, true);
-        _maximumThreadCount=atoi(argv[i]);
+        _maximumThreadCount=toInt(i, i-1, argv);
         QTextStream sOut(stderr);
         if(_maximumThreadCount<1) {
-          sOut << tr("Negative or null maximum number of jobs (option -j or -jobs)") << Qt::endl;
+          sOut << tr("Negative or null maximum number of jobs: '%1'").arg(_maximumThreadCount) << ::endl;
           ::exit(2);
         }
         if(_maximumThreadCount>2*QThread::idealThreadCount()) {
@@ -964,7 +962,7 @@ namespace QGpCoreTools {
     h->addOption(tr("-h, -help [ARG]"), str);
     h->addOption(tr("-args [MAX]"), tr("List the argument history diplaying MAX entries (default=50). "
                                        "Üse '-args 0' to print all entries recorded so far."));
-    h->addOption(tr("-rargs [INDEX]"), tr("Reuse argument list with INDEX. See '-args' to get the available argument lists."
+    h->addOption(tr("-rargs [INDEX]"), tr("Reuse argument list with INDEX. See '-args' to get the available argument lists. "
                                           "Wihtout INDEX the last argument list is used."));
     h->addOption(tr("-version"), tr("Show version information"));
     h->addOption(tr("-app-version"), tr("Show short version information"));
@@ -1043,6 +1041,48 @@ namespace QGpCoreTools {
     return true;
   }
 
+  int CoreApplicationPrivate::toInt(int i, int optionIndex, char ** argv)
+  {
+    bool ok;
+    int val=QString(argv[i]).toInt(&ok);
+    if(!ok) {
+      QTextStream sOut(stderr);
+      if(optionIndex<0 && i>0) {
+        optionIndex=i-1;
+      }
+      if(optionIndex>=0) {
+        sOut << tr("Error converting numerical value: '%1' for option '%2'.")
+                .arg(argv[i])
+                .arg(argv[optionIndex]) << Qt::endl;
+      } else {
+        sOut << tr("Error converting numerical value: '%1'.").arg(argv[i]) << ::endl;
+      }
+      ::exit(2);
+    }
+    return val;
+  }
+
+  double CoreApplicationPrivate::toDouble(int i, int optionIndex, char ** argv)
+  {
+    bool ok;
+    double val=QString(argv[i]).toDouble(&ok);
+    if(!ok) {
+      QTextStream sOut(stderr);
+      if(optionIndex<0 && i>0) {
+        optionIndex=i-1;
+      }
+      if(optionIndex>=0) {
+        sOut << tr("Error converting numerical value: '%1' for option '%2'.")
+                .arg(argv[i])
+                .arg(argv[optionIndex]) << Qt::endl;
+      } else {
+        sOut << tr("Error converting numerical value: '%1'.").arg(argv[i]) << ::endl;
+      }
+      ::exit(2);
+    }
+    return val;
+  }
+
   /*!
     Read stdin until a line ending by 'END'.
     Mostly used by '-reportbug' option to read backtrace infromation.
@@ -1062,11 +1102,26 @@ namespace QGpCoreTools {
   /*!
     Convert all remaining arguments into file names (accepting wildchars)
   */
-  QStringList CoreApplicationPrivate::getFileList(int argc, char ** argv)
+  QStringList CoreApplicationPrivate::getFileList(int& argc, char ** argv)
   {
     QStringList files;
-    for(int index=1; index<argc; index++) {
-      files.append(argv[index]);
+    int j=1;
+    for(int i=1; i<argc; i++) {
+      QByteArray arg=argv[i];
+      if(arg[0]!='-') {
+        files.append(argv[i]);
+      } else if(arg=="--") {
+        for(i++;i<argc; i++) {
+          argv[j++]=argv[i];
+        }
+        break;
+      } else {
+        argv[j++]=argv[i];
+      }
+    }
+    if(j<argc) {
+      argv[j]=nullptr;
+      argc=j;
     }
     // Replace file names that include "*" with all their corresponding entries
     QDir d;
@@ -1414,8 +1469,8 @@ namespace QGpCoreTools {
       return 2;
     }
     QString appPath=QCoreApplication::instance()->applicationFilePath();
-    sOut << tr("Starting detached '%1 %2'...").arg(appPath).arg(args.join(" ")) << Qt::endl;
-    return QProcess::startDetached(appPath, args) ? 0 : 2;
+    sOut << tr("Starting '%1 %2'...").arg(appPath).arg(args.join(" ")) << Qt::endl;
+    return QProcess::execute(appPath, args) ? 0 : 2;
   }
 
   /*!
