@@ -455,7 +455,7 @@ QString File::readLineNoComments(QTextStream& s, QString * comments, int * lineN
   while(true) {
     buf=s.readLine();
     if(lineNumber) lineNumber++;
-    if(buf[0]=='#') {
+    if(!buf.isEmpty() && buf[0]=='#') {
       if(comments) {
         comments->append(buf);
         comments->append("\n");
@@ -688,6 +688,30 @@ qint16 File::fromLittleEndian(qint16 v)
 #endif
 }
 
+/*!
+  Several \a patterns are provided for compatibility. The first one is the "official"
+  one. Instead of testing all patterns for each line, the stream is read several times.
+  It may look like unefficient but it is a little bit slower only for compatibility cases.
+  For the nominal case, it is faster like this.
+
+  Returns the index of the matched pattern or -1 if non has been found.
+*/
+int File::readUntilPatterns(QTextStream& s, const QStringList& patterns)
+{
+  TRACE;
+  if(!patterns.isEmpty()) {
+    qint64 pos=s.pos();
+    int n=patterns.count();
+    for(int i=0; i<n; i++) {
+      readUntilPattern(s, patterns.at(i));
+      if(s.pos()!=pos) {
+        return i;
+      }
+    }
+  }
+  return -1;
+}
+
 void File::readUntilPattern(QTextStream& s, const QString& pattern)
 {
   TRACE;
