@@ -74,6 +74,10 @@ namespace QGpGuiMath {
     connect(a, SIGNAL(triggered()), this, SLOT(showLog()) );
     a=addCurveAction(tr("&Remove"), tr("Remove curve(s)"), true);
     connect(a, SIGNAL(triggered()), this, SLOT(remove()));
+    a=addCurveAction(tr("&Hide"), tr("Hide curve(s)"), true);
+    connect(a, SIGNAL(triggered()), this, SLOT(hideCurves()));
+    a=addCurveAction(tr("S&how"), tr("Show curve(s)"), true);
+    connect(a, SIGNAL(triggered()), this, SLOT(showCurves()));
     a=addCurveAction(tr("Resa&mple" ), tr("Resample curve(s)"), true);
     connect(a, SIGNAL(triggered()), this, SLOT(resample()));
     a=addCurveAction(tr("&Cut"), tr("Cut curve(s)"), true);
@@ -84,7 +88,7 @@ namespace QGpGuiMath {
     connect(a, SIGNAL(triggered()), this, SLOT(average()));
     a=addCurveAction(tr("&Validate samples"), tr("Change valid flag of samples for curve(s)"), true);
     connect(a, SIGNAL(triggered()), this, SLOT(validate()));
-    a=addCurveAction(tr("S&plit"), tr("Split curve into continuous segments"), true);
+    a=addCurveAction(tr("S&plit point cloud"), tr("Split point cloud into continuous segments"), true);
     connect(a, SIGNAL(triggered()), this, SLOT(split()));
 
     curveTabs->setTabsClosable(true);
@@ -821,18 +825,24 @@ namespace QGpGuiMath {
     d->setCurveLimits(min, max);
     d->setAxisNames(_proxy->xName(), _proxy->xInversedName());
     Settings::getWidget(d, "CurveBrowserSmooth");
+    d->updateAllFields();
     if(d->exec()==QDialog::Accepted) {
       Settings::setWidget(d, "CurveBrowserSmooth");
-      double minimum=d->minimumEdit->text().toDouble();
-      double maximum=d->maximumEdit->text().toDouble();
-      SamplingOptions options=LinearScale;
+      SmoothingParameters param;
+      d->smoothing->getParameters(param);
+      SamplingParameters sampling;
+      sampling.setRange(d->minimumEdit->text().toDouble(), d->maximumEdit->text().toDouble());
       if(d->logBut->isChecked()) {
-        options |= LogScale;
+        sampling.setScaleType(SamplingParameters::Log);
       } else if(d->inversedBut->isChecked()) {
-        options |= InversedScale;
+        sampling.setScaleType(SamplingParameters::Inversed);
+      } else {
+        sampling.setScaleType(SamplingParameters::Linear);
       }
       for(QList<CurveProperties *>::const_iterator it=list.begin(); it!=list.end(); it++) {
-        (*it)->smooth(minimum, maximum, options);
+        CurveProperties * curve=*it;
+        sampling.setCount(curve->proxy()->sampleCount());
+        curve->smooth(sampling, param);
       }
     }
     delete d;
@@ -858,15 +868,17 @@ namespace QGpGuiMath {
       // Create new split curves
       for(QList<CurveProperties *>::const_iterator it=list.begin(); it!=list.end(); it++) {
         CurveProxy * px=(*it)->proxy();
-        int n=px->splitCount(maxX, LogScale, maxErr, minCount);
+        px->beginSplit(maxX, LogScale, maxErr, minCount);
+        int n=px->splitCount();
         for(int i=0; i<n; i++) {
           CurveProperties * pa=addLine(_currentLayer->addLine());
           CurveProxy * pxa=pa->proxy();
           pa->beginCurveChange();
           pxa->addLog(tr("split of curve %1\n").arg((*it)->proxy()->name()));
-          pxa->split(px, i, maxX, LogScale, maxErr, minCount);
+          pxa->getSplit(px, i);
           pa->endCurveChange();
         }
+        px->endSplit();
       }
     }
     delete d;
@@ -888,6 +900,36 @@ namespace QGpGuiMath {
     emit curveModified();
   }
 
+  void CurveBrowser::showCurves()
+  {
+    TRACE;
+    QList<CurveProperties *> list=selectCurves();
+    if(list.isEmpty()) {
+      return;
+    }
+    for(QList<CurveProperties *>::const_iterator it=list.begin(); it!=list.end(); it++) {
+      int index=curveTabs->indexOf(*it);
+      CurveProperties * p=static_cast<CurveProperties *>(curveTabs->widget(index));
+      p->setCurveVisible(true);
+    }
+    emit curveModified();
+  }
+
+  void CurveBrowser::hideCurves()
+  {
+    TRACE;
+    QList<CurveProperties *> list=selectCurves();
+    if(list.isEmpty()) {
+      return;
+    }
+    for(QList<CurveProperties *>::const_iterator it=list.begin(); it!=list.end(); it++) {
+      int index=curveTabs->indexOf(*it);
+      CurveProperties * p=static_cast<CurveProperties *>(curveTabs->widget(index));
+      p->setCurveVisible(false);
+    }
+    emit curveModified();
+  }
+
   void CurveBrowser::on_curveTabs_tabCloseRequested(int index)
   {
     TRACE;
