@@ -1758,9 +1758,8 @@ namespace SciFigs {
   void Axis::mouseDoubleClickEvent(QMouseEvent * e)
   {
     TRACE;
-    if(e->buttons() & Qt::LeftButton) {
-      AxisWindow * g=_content->graph();
-      g->setActive(e->modifiers());
+    if((e->buttons() & Qt::LeftButton)) {
+      _content->graph()->setActive(e->modifiers());
       showProperties();
     } else {
       e->ignore();
@@ -1772,10 +1771,9 @@ namespace SciFigs {
     TRACE;
     if(e->buttons() & Qt::RightButton) {
       AxisWindow * w=_content->graph();
-      GraphicObject * g=w->group();
-      g->setActive(e->modifiers());
+      w->setActive(e->modifiers());
       QMenu m;
-      QList<QAction *> l=g->actions();
+      QList<QAction *> l=w->relativeGroup()->actions();
       l.removeFirst();
       l.prepend(_propertiesAction);
       m.addActions(l);
@@ -1788,7 +1786,7 @@ namespace SciFigs {
   void Axis::showProperties()
   {
     TRACE;
-    AxisWindow * g=_content->graph();
+    GraphicObject * g=_content->graph()->relativeGroup();
     switch(_orientation) {
     case North:
     case South:
