@@ -34,7 +34,6 @@
 #endif
 
 #include "GraphicSheet.h"
-#include "SelectionWidget.h"
 #include "GraphicObject.h"
 #include "GraphicObjectGroup.h"
 #include "OrderTip.h"
@@ -90,9 +89,12 @@ namespace SciFigs {
 
     Moreover, the mouse clicks are handled to move and to select the object in the sheet. Multiple selections are supported with the SHIFT key.
 
-    The selected object is marked by a SelectionWidget that is slightly larger than this object, showing a black (active selection) or grey (passive selection) border. The mouse events on the border are redirected to this object. Usually, it is not necessary to use it explicitely. See setSelected(), selected(), active(), activeObject().
+    A selected object is marked by  a black (active selection) or grey (passive selection) border.
 
-    The margins are useful when exporting to bitmaps or when the object is located close to the edge of a GraphicSheet. It extends the printing or image size by a few millimeters. Units for margins are always cm. See printLeftMargin(), printRightMargin(), printTopMargin(), printBottomMargin(), and setPrintLeftMargin(), setPrintRightMargin(), setPrintTopMargin(), setPrintBottomMargin().
+    The margins are useful when exporting to bitmaps or when the object is located close to the edge of a GraphicSheet.
+    It extends the printing or image size by a few millimeters. Units for margins are always cm.
+
+    \sa printLeftMargin(), printRightMargin(), printTopMargin(), printBottomMargin(), setPrintLeftMargin(), setPrintRightMargin(), setPrintTopMargin(), setPrintBottomMargin().
   */
 
   /*!
@@ -104,11 +106,10 @@ namespace SciFigs {
       QWidget(parent, f)
   {
     TRACE;
-    _selected=0;
-    _sheet=0;
-    _orderTip=0;
+    _selectionState=None;
+    _sheet=nullptr;
+    _orderTip=nullptr;
     _isRemovable=false;
-    _isSelectable=true;
     new QShortcut(tr("Ctrl+M"), this, SLOT(keyPopupMenu()));
 
     setAttribute(Qt::WA_OpaquePaintEvent, true);
@@ -179,7 +180,7 @@ namespace SciFigs {
   void GraphicObject::setPrintXAnchor(double val)
   {
     TRACE;
-    if(val<0.0 && _isSelectable) { // Allowed temporarily in a group
+    if(val<0.0 && !isInsideAbsoluteGroup()) { // Allowed temporarily in a group
       val=0.0;
     }
     _printAnchor.setX(val);
@@ -191,7 +192,7 @@ namespace SciFigs {
   void GraphicObject::setPrintYAnchor(double val)
   {
     TRACE;
-    if(val<0.0 && _isSelectable) { // Allowed temporarily in a group
+    if(val<0.0 && !isInsideAbsoluteGroup()) { // Allowed temporarily in a group
       val=0.0;
     }
     _printAnchor.setY(val);
@@ -204,11 +205,13 @@ namespace SciFigs {
   {
     TRACE;
     _printAnchor=o;
-    if(_printAnchor.x()<0.0 && _isSelectable) {
-      _printAnchor.setX(0.0);
-    }
-    if(_printAnchor.y()<0.0 && _isSelectable) {
-      _printAnchor.setY(0.0);
+    if(!isInsideAbsoluteGroup()) { // Allowed temporarily in a group
+      if(_printAnchor.x()<0.0) {
+        _printAnchor.setX(0.0);
+      }
+      if(_printAnchor.y()<0.0) {
+        _printAnchor.setY(0.0);
+      }
     }
   }
 
@@ -218,7 +221,7 @@ namespace SciFigs {
   void GraphicObject::setPrintLeft(double val)
   {
     TRACE;
-    if(val<0.0 && _isSelectable) { // Allowed temporarily in a group
+    if(val<0.0 && !isInsideAbsoluteGroup()) { // Allowed temporarily in a group
       val=0.0;
     }
     switch(_anchor) {
@@ -244,7 +247,7 @@ namespace SciFigs {
   void GraphicObject::setPrintTop(double val)
   {
     TRACE;
-    if(val<0.0 && _isSelectable) { // Allowed temporarily in a group
+    if(val<0.0 && !isInsideAbsoluteGroup()) { // Allowed temporarily in a group
       val=0.0;
     }
     switch(_anchor) {
@@ -269,7 +272,7 @@ namespace SciFigs {
   void GraphicObject::setPrintRight(double val)
   {
     TRACE;
-    if(val<0.0 && _isSelectable) { // Allowed temporarily in a group
+    if(val<0.0 && !isInsideAbsoluteGroup()) { // Allowed temporarily in a group
       val=0.0;
     }
     switch(_anchor) {
@@ -295,7 +298,7 @@ namespace SciFigs {
   void GraphicObject::setPrintBottom(double val)
   {
     TRACE;
-    if(val<0.0 && _isSelectable) { // Allowed temporarily in a group
+    if(val<0.0 && !isInsideAbsoluteGroup()) { // Allowed temporarily in a group
       val=0.0;
     }
     switch(_anchor) {
@@ -352,16 +355,6 @@ namespace SciFigs {
     Set the bottom margin in cm.
   */
 
-  /*!
-    \fn SelectionWidget * GraphicObject::selected() const
-    Tells whether the object is selected or not (passive or active selection is not considered)
-  */
-
-  /*!
-    \fn void GraphicObject::setSelected(SelectionWidget *s)
-    Select the object by attribuing a SelectionWidget.
-  */
-
   GraphicObject& GraphicObject::operator=(const GraphicObject& o)
   {
     TRACE;
@@ -499,7 +492,7 @@ namespace SciFigs {
     TRACE;
     bool ret=false;
     if(index >= 0) {
-      if( !_orderTip) {
+      if(!_orderTip) {
         _orderTip=new OrderTip(this);
         _orderTip->move(0, 0);
         _orderTip->show();
@@ -530,19 +523,19 @@ namespace SciFigs {
     QWidget::addAction(a);
     _propertiesAction=a;
 
-    if(sheet()) {
+    if(_sheet) {
       a=new QAction(this);
       a->setSeparator(true);
       QWidget::addAction(a);
 
       a=new QAction(tr("&Group"), this);
       a->setObjectName("Group");
-      connect(a, SIGNAL(triggered()), sheet(), SLOT(groupObjects()));
+      connect(a, SIGNAL(triggered()), _sheet, SLOT(groupObjects()));
       QWidget::addAction(a);
 
       QMenu * m=new QMenu(this);
-      m->addAction(tr("Bring to &front"), sheet(), SLOT(raiseSelection()) );
-      m->addAction(tr("Send to &back"), sheet(), SLOT(lowerSelection()) );
+      m->addAction(tr("Bring to &front"), _sheet, SLOT(raiseSelection()) );
+      m->addAction(tr("Send to &back"), _sheet, SLOT(lowerSelection()) );
 
       a=new QAction(tr("&Order"), this);
       a->setObjectName("Order");
@@ -550,15 +543,15 @@ namespace SciFigs {
       QWidget::addAction(a);
 
       m=new QMenu(this);
-      m->addAction(tr("&Left"), sheet(), SLOT(alignLeft()));
-      m->addAction(tr("&Centered"), sheet(), SLOT(alignHCenter()));
-      m->addAction(tr("&Right"), sheet(), SLOT(alignRight()));
+      m->addAction(tr("&Left"), _sheet, SLOT(alignLeft()));
+      m->addAction(tr("&Centered"), _sheet, SLOT(alignHCenter()));
+      m->addAction(tr("&Right"), _sheet, SLOT(alignRight()));
       m->addSeparator();
-      m->addAction(tr("&Top"), sheet(), SLOT(alignTop()));
-      m->addAction(tr("C&entered"), sheet(), SLOT(alignVCenter()));
-      m->addAction(tr("&Bottom"), sheet(), SLOT(alignBottom()));
+      m->addAction(tr("&Top"), _sheet, SLOT(alignTop()));
+      m->addAction(tr("C&entered"), _sheet, SLOT(alignVCenter()));
+      m->addAction(tr("&Bottom"), _sheet, SLOT(alignBottom()));
       m->addSeparator();
-      m->addAction(tr("&Grid"), sheet(), SLOT(alignGrid()));
+      m->addAction(tr("&Grid"), _sheet, SLOT(alignGrid()));
 
       a=new QAction(tr("&Alignment"), this);
       a->setObjectName("Alignment");
@@ -659,6 +652,9 @@ namespace SciFigs {
   void GraphicObject::updateMask()
   {
     TRACE;
+    if(_selectionState!=None) {
+      return;
+    }
     //qDebug() << "update mask";
     // Repaint the widget in a pixmap (like it appear on the screen)
     if(width()==0 || height()==0) return;
@@ -689,7 +685,7 @@ namespace SciFigs {
     selection widget.
 
     Call this function every time you modify the position or the size of the object,
-    e.g. with printX or printWidth properties.
+    e.g. with printAnchor or printWidth properties.
   */
   void GraphicObject::updateGeometry()
   {
@@ -1221,9 +1217,9 @@ namespace SciFigs {
       case GraphicObjectGeometryProperties::X:
         dVal=Number::toDouble(val, ok);
         if(ok) {
-          if(sheet() && isMoveSelection()) {
+          if(_sheet && isMoveSelection()) {
             if(isActive()) {
-              sheet()->moveObjects(Point2D(dVal-printXAnchor(), 0.0));
+              _sheet->moveObjects(Point2D(dVal-printXAnchor(), 0.0));
             }
           } else {
             setPrintXAnchor(dVal);
@@ -1234,9 +1230,9 @@ namespace SciFigs {
       case GraphicObjectGeometryProperties::Y:
         dVal=Number::toDouble(val, ok);
         if(ok) {
-          if(sheet() && isMoveSelection()) {
+          if(_sheet && isMoveSelection()) {
             if(isActive()) {
-              sheet()->moveObjects(Point2D(0.0, dVal-printYAnchor()));
+              _sheet->moveObjects(Point2D(0.0, dVal-printYAnchor()));
             }
           } else {
             setPrintYAnchor(dVal);
@@ -1339,7 +1335,7 @@ namespace SciFigs {
   void GraphicObject::updateGeometryProperties(PropertyProxy * pp)
   {
     TRACE;
-    if(sheet() && isActive() && pp->setCurrentCategory(_category) && pp->setCurrentTab(_tabGeomery)) {
+    if(_sheet && isActive() && pp->setCurrentCategory(_category) && pp->setCurrentTab(_tabGeomery)) {
       GraphicObjectGeometryProperties * w=static_cast<GraphicObjectGeometryProperties *>(pp->currentTabWidget());
       pp->setCurrentTabValues();
       w->setWidgets();
@@ -1386,23 +1382,23 @@ namespace SciFigs {
     TRACE;
   }
 
-  /*!
-    Select object by affecting a new selection widget \a s.
-  */
-  void GraphicObject::setSelected(SelectionWidget * s)
-  {
-    TRACE;
-    _selected=s;
-  }
-
   /*!
     Tells whether the object is selected
   */
   bool GraphicObject::isSelected() const
   {
     TRACE;
-    return !_sheet || (_selected && (_selected->state()==SelectionWidget::Selected ||
-                                     _selected->state()==SelectionWidget::Activated) );
+    if(_sheet) {
+      switch(_selectionState) {
+      case None:
+      case Hoover:
+        return false;
+      case Selected:
+      case Activated:
+        break;
+      }
+    }
+    return true;
   }
 
   /*!
@@ -1411,7 +1407,7 @@ namespace SciFigs {
   bool GraphicObject::isActive() const
   {
     TRACE;
-    return !_sheet || (_selected && _selected->state()==SelectionWidget::Activated);
+    return !_sheet || _selectionState==Activated;
   }
 
   /*!
@@ -1420,8 +1416,8 @@ namespace SciFigs {
   void GraphicObject::setActive(Qt::KeyboardModifiers s)
   {
     TRACE;
-    if(!_selected || _selected->state()!=SelectionWidget::Activated) {
-      emit wantToBeSelected(this, s);
+    if(relativeGroup()->selectionState()!=Activated) {
+      emit wantToBeSelected(relativeGroup(), s);
     }
   }
 
@@ -1434,6 +1430,19 @@ namespace SciFigs {
     return _sheet->activeObject(className);
   }
 
+  bool GraphicObject::belongsTo(QWidget * context) const
+  {
+    TRACE;
+    QWidget * p=parentWidget();
+    while(p) {
+      if(p==context) {
+        return true;
+      }
+      p=p->parentWidget();
+    }
+    return false;
+  }
+
   /*!
     Show context menu after CTRL+M request, or for other reason. The menu is displayed in the to left corner.
 
@@ -1460,31 +1469,31 @@ namespace SciFigs {
   void GraphicObject::mousePressEvent(QMouseEvent * e)
   {
     TRACE;
-    switch (e->button()) {
-    case Qt::LeftButton:
-      if(_isSelectable) {
-        setActive(e->modifiers());
-        _draggingPos=mapToParent(e->pos());
-        emit wantToStartDragging();
-        _draggingOn=true;
-        _draggingStarted=false;
-      } else {
-        e->ignore();
+    if(isInsideRelativeGroup() || (_sheet && !belongsTo(_sheet->context()))) {
+      e->ignore();
+    } else {
+      setActive(e->modifiers());
+      switch (e->button()) {
+      case Qt::LeftButton:
+        if(_sheet && belongsTo(_sheet->context())) {
+          _draggingPos=mapTo(_sheet->context(), e->pos());
+          emit wantToStartDragging();
+          _draggingOn=true;
+          _draggingStarted=false;
+        }
+        break;
+      case Qt::RightButton: {
+          QMenu m;
+          QList<QAction *> l=relativeGroup()->actions();
+          l.removeFirst();
+          l.prepend(_propertiesAction);
+          m.addActions(l);
+          m.exec(mapToGlobal(e->pos()));
+        }
+        break;
+      default:
+        break;
       }
-      break;
-    case Qt::RightButton: {
-        GraphicObject * g=group();
-        g->setActive(e->modifiers());
-        QMenu m;
-        QList<QAction *> l=g->actions();
-        l.removeFirst();
-        l.prepend(_propertiesAction);
-        m.addActions(l);
-        m.exec(mapToGlobal(e->pos()));
-      }
-      break;
-    default:
-      break;
     }
   }
 
@@ -1494,15 +1503,15 @@ namespace SciFigs {
   void GraphicObject::mouseReleaseEvent(QMouseEvent * e)
   {
     TRACE;
-    if(_isSelectable) {
+    if(isInsideRelativeGroup() || (_sheet && !belongsTo(_sheet->context()))) {
+      e->ignore();
+    } else {
       if(_draggingOn) {
         _draggingOn=false;
-        if(sheet() && sheet()->isOrderIndex()) {
-          sheet()->setOrderIndex(this);
+        if(_sheet && _sheet->isOrderIndex()) {
+          _sheet->setOrderIndex(this);
         }
       }
-    } else {
-      e->ignore();
     }
   }
 
@@ -1512,27 +1521,21 @@ namespace SciFigs {
   void GraphicObject::mouseMoveEvent(QMouseEvent * e)
   {
     TRACE;
-    if(_isSelectable) {
-      if(_draggingOn) {
-        QPoint d=mapToParent(e->pos());
-        d-=_draggingPos;
-        if(_draggingStarted || abs(d.x()) > 10 || abs(d.y()) > 10) {
-          _draggingStarted=true;
-          emit draggedTo(mapToGlobal(e->pos()));
-        }
-      }
-    } else {
+    if(isInsideRelativeGroup() || (_sheet && !belongsTo(_sheet->context()))) {
       e->ignore();
-    }
-  }
-
-  void GraphicObject::setSelectable(bool s)
-  {
-    TRACE;
-    _isSelectable=s;
-    if(!_isSelectable && _selected) {
-      _selected->deleteLater();
-      _selected=0;
+    } else {
+      if(_draggingOn && _sheet) {
+        QPoint p=mapTo(_sheet->context(), e->pos());
+        if(!_draggingStarted) {
+          QPoint d=p-_draggingPos;
+          if(abs(d.x())>10 || abs(d.y())>10) {
+            _draggingStarted=true;
+          } else {
+            return;
+          }
+        }
+        emit draggedTo(p);
+      }
     }
   }
 
@@ -1544,12 +1547,8 @@ namespace SciFigs {
     TRACE;
     if(antiReccurFlag) {
       antiReccurFlag=false;
-      if(sheet() && !selectionWidget() && _isSelectable) {
-        sheet()->addSelection(this);
-        // Here and not in addSelection(), bug observed with Qt 4.6 under Mac, show() calls GraphicObject::enterEvent()
-        ASSERT(_selected);
-        _selected->setState(SelectionWidget::Hoover);
-        _selected->show();
+      if(_sheet && !isSelected() && !isInsideRelativeGroup() && belongsTo(_sheet->context())) {
+        _sheet->addSelection(this, Hoover);
       }
       antiReccurFlag=true;
     }
@@ -1558,13 +1557,24 @@ namespace SciFigs {
   void GraphicObject::leaveEvent(QEvent *)
   {
     TRACE;
-    if(_selected && _selected->state()==SelectionWidget::Hoover) {
-      ASSERT(sheet());
-      QPoint p=sheet()->mapFromGlobal(QCursor::pos());
+    if(_selectionState==Hoover) {
+      QPoint p=_sheet->mapFromGlobal(QCursor::pos());
       QRect r=geometry();
       if(!r.contains(p)) {
-        _selected->deleteLater();
-        _selected=0;
+        _sheet->removeSelection(this);
+      }
+    }
+  }
+
+  void GraphicObject::setSelectionState(SelectionState s)
+  {
+    TRACE;
+    _selectionState=s;
+    if(_mask) {
+      if(_selectionState==None) {
+        updateMask();
+      } else {
+        clearMask();
       }
     }
   }
@@ -1573,6 +1583,7 @@ namespace SciFigs {
   {
     TRACE;
     setPrintAnchor(_oldPrintPos+delta);
+    emit positionChanged();
   }
 
   /*!
@@ -1583,9 +1594,13 @@ namespace SciFigs {
   void GraphicObject::mouseDoubleClickEvent(QMouseEvent * e)
   {
     TRACE;
-    setActive(e->modifiers());
-    if(e->buttons() & Qt::LeftButton) {
-      showProperties();
+    if(isInsideRelativeGroup() || (_sheet && !belongsTo(_sheet->context()))) {
+      e->ignore();
+    } else {
+      setActive(e->modifiers());
+      if(e->buttons() & Qt::LeftButton) {
+        showProperties();
+      }
     }
   }
 
@@ -1731,7 +1746,7 @@ namespace SciFigs {
     TRACE;
     Q_UNUSED(context)
     /* Used to re-order object according to make-up
-    if(sheet()) {
+    if(_sheet) {
       if(selectionWidget()) selectionWidget()->raise();
       raise();
     }*/
@@ -1742,17 +1757,6 @@ namespace SciFigs {
     return true;
   }
 
-  /*!
-    Share tooltip with its selection widget if any.
-  */
-  void GraphicObject::setToolTip(const QString& t)
-  {
-    if(_selected) {
-      _selected->setToolTip(t);
-    }
-    QWidget::setToolTip(t);
-  }
-
   QAction * GraphicObject::action(const QString& name)
   {
     TRACE;
@@ -1776,21 +1780,32 @@ namespace SciFigs {
   }
 
   /*!
-    If inside a group, return the base group.
+    If inside a group, return the base group (relative to sheet context)
     If not return this.
   */
-  GraphicObject * GraphicObject::group()
+  GraphicObject * GraphicObject::relativeGroup()
   {
     TRACE;
-    if(parentWidget()==_sheet || !_sheet) {
-      return this;
+    GraphicObjectGroup * g=qobject_cast<GraphicObjectGroup *>(parent());
+    if(g && (!_sheet || _sheet->context()!=g)) {
+      return g->relativeGroup();
     } else {
-      GraphicObjectGroup * g=qobject_cast<GraphicObjectGroup *>(parentWidget());
-      if(g) {
-        return g->group();
-      } else {
-        return this;
-      }
+      return this;
+    }
+  }
+
+  /*!
+    If inside a group, return the base group (relative to sheet context)
+    If not return this.
+  */
+  GraphicObject * GraphicObject::absoluteGroup()
+  {
+    TRACE;
+    GraphicObjectGroup * g=qobject_cast<GraphicObjectGroup *>(parent());
+    if(g && (!_sheet || _sheet->widget()!=g)) {
+      return g->absoluteGroup();
+    } else {
+      return this;
     }
   }
 
@@ -1816,5 +1831,50 @@ namespace SciFigs {
     updateGeometry();
   }
 
+  /*!
+    If \a nPlotsPerLine is null, it is ignored.
+  */
+  void GraphicObject::setLayout(const QList<GraphicObject *>& objects, int nPlotsPerLine, double pageHeight)
+  {
+    TRACE;
+    double x, y, dx=-1.0, dy=-1.0, minx, maxx, miny;
+    SAFE_UNINITIALIZED(x,0);
+    SAFE_UNINITIALIZED(y,0);
+    SAFE_UNINITIALIZED(minx,0);
+    SAFE_UNINITIALIZED(miny,0);
+    SAFE_UNINITIALIZED(maxx,0);
+    int pageIndex=0;
+    for(QList<GraphicObject *>::const_iterator it=objects.begin(); it!=objects.end(); it++) {
+      GraphicObject * w=*it;
+      if(dx<0.0) { // Init geometry parameters from geometry of first graph
+        minx=w->printLeft();
+        miny=w->printTop();
+        if(miny>pageHeight) miny=0.5;
+        if(nPlotsPerLine>0) {
+          dx=minx+w->printWidth();
+          maxx=minx+nPlotsPerLine*dx;
+        } else {
+          dx=0.0;
+          maxx=-1.0;
+        }
+        dy=ceil(w->printHeight()*4.0)*0.25;
+        x=minx;
+        y=miny;
+      } else {
+        x+=dx;
+        if(x+dx>maxx) {
+          x=minx;
+          y+=dy;
+        }
+        if(y+dy>(pageIndex+1)*pageHeight) {
+          pageIndex++;
+          y=miny+pageIndex*pageHeight;
+        }
+      }
+      w->setPrintLeft(x);
+      w->setPrintTop(y);
+      w->updateGeometry();
+    }
+  }
 
 } // namespace SciFigs
