@@ -36,7 +36,6 @@
 #include "GraphicSheet.h"
 #include "OrderTip.h"
 #include "PageLimit.h"
-#include "SelectionWidget.h"
 #include "SciFigsPreferences.h"
 #include "GraphicObjectFactory.h"
 #include "GraphicObjectGroup.h"
@@ -46,14 +45,14 @@
 #include "SciFigsGlobal.h"
 #include "MakeUpFilter.h"
 #include "GraphicSheetProperties.h"
-#include "SheetSelectionWidget.h"
+#include "PageSizeProperties.h"
 #include "AlignGridParameters.h"
 
 //#define SELECTION_DEBUG
 
 namespace SciFigs {
 
-  const QString GraphicSheet::xmlGraphicSheetTag="GraphicSheet";
+const QString GraphicSheet::xmlGraphicSheetTag="GraphicSheet";
 
   /*!
     \ingroup SciFigs
@@ -63,7 +62,7 @@ namespace SciFigs {
   */
 
   GraphicSheet::GraphicSheet(QWidget *parent) :
-      QScrollArea(parent)
+      QScrollArea(parent), _pageSize()
   {
     TRACE;
     setWidgetColor(this, Qt::white);
@@ -71,7 +70,7 @@ namespace SciFigs {
     setWidget(new QWidget(this));
 
     _statusBar=nullptr;
-    _currentActiveSelection=nullptr;
+    _activeObject=nullptr;
     new QShortcut(Qt::Key_Tab, this, SLOT(selectNextChild()));
     new QShortcut(Qt::Key_Tab + Qt::CTRL, this, SLOT(addSelectNextChild()));
     _orderIndex=-1;
@@ -81,9 +80,9 @@ namespace SciFigs {
     _dragDelay.setSingleShot(true);
     _dragDelay.setInterval(50);
     connect(&_dragDelay, SIGNAL(timeout()), this, SLOT(dragObjectsDelayed()));
-    addActions();
     _mouseTrackingRect=nullptr;
     _selection=new SheetSelectionWidget(this);
+    _contexts.append(widget());
   }
 
   GraphicSheet::~GraphicSheet()
@@ -98,18 +97,32 @@ namespace SciFigs {
   }
 
   /*!
-    Initialize context menu
+    Initialize context menu (called each time the context menu is displayed
   */
-  void GraphicSheet::addActions()
+  void GraphicSheet::addActions(QMenu * m)
   {
     TRACE;
     QAction * a;
 
-    a=new QAction(tr("&Properties"), this);
+    a=new QAction(tr("&Properties"), m);
     a->setObjectName("Properties");
-    a->setToolTip(tr("Set properties of object"));
+    a->setToolTip(tr("Set properties of object selection"));
     connect(a, SIGNAL(triggered(bool)) , this, SLOT(showProperties()));
-    QWidget::addAction(a);
+    m->addAction(a);
+
+    if(_contexts.count()>1) {
+      a=new QAction(tr("&Exit group"), m);
+      a->setObjectName("ExitGroup");
+      connect(a, SIGNAL(triggered(bool)) , this, SLOT(exitGroup()));
+      m->addAction(a);
+    }
+
+    if(!actions().isEmpty()) {
+      a=new QAction(m);
+      a->setSeparator(true);
+      m->addAction(a);
+      m->addActions(actions());
+    }
   }
 
   /*!
@@ -122,16 +135,8 @@ namespace SciFigs {
   void GraphicSheet::addMenu(QMenu * m)
   {
     TRACE;
-    QAction * a;
-
-    if(actions().count()==1) {
-      a=new QAction(this);
-      a->setSeparator(true);
-      QWidget::addAction(a);
-    }
-
-    a=new QAction(m->title(), this);
-    a -> setMenu(m);
+    QAction * a=new QAction(m->title(), this);
+    a->setMenu(m);
     QWidget::addAction(a);
   }
 
@@ -145,20 +150,14 @@ namespace SciFigs {
     proxy()->saveStates();
     addProperties(proxy());
     GraphicObjectList wList=selectedObjects<GraphicObject>();
-    GraphicObject * curObj;
-    if(_currentActiveSelection) {
-      curObj=_currentActiveSelection->widget();
-    } else {
-      curObj=nullptr;
-    }
     for(int i=wList.count()-1; i>=0; i--) {
       GraphicObject * obj=wList.at(i);
-      if(obj!=curObj) {
+      if(obj!=_activeObject) {
         obj->addProperties(proxy());
       }
     }
-    if(curObj) {
-      curObj->addProperties(proxy());
+    if(_activeObject) {
+      _activeObject->addProperties(proxy());
     }
     setPropertyTitle(this);
     raiseEditor(category);
@@ -168,6 +167,7 @@ namespace SciFigs {
 
   uint GraphicSheet::_category=PropertyProxy::uniqueId();
   uint GraphicSheet::_tabPrint=PropertyProxy::uniqueId();
+  uint GraphicSheet::_tabPageSize=PropertyProxy::uniqueId();
 
   /*!
 
@@ -184,6 +184,12 @@ namespace SciFigs {
       GraphicSheetProperties * w=new GraphicSheetProperties;
       pp->addTab(_tabPrint, tr("Print"), w, this);
     }
+    if(pp->setCurrentTab(_tabPageSize)) {
+      pp->addReference(this);
+    } else {
+      PageSizeProperties * w=new PageSizeProperties;
+      pp->addTab(_tabPageSize, tr("Page size"), w, this);
+    }
   }
 
   /*!
@@ -194,26 +200,35 @@ namespace SciFigs {
     TRACE;
     if(pp->setCurrentCategory(_category)) {
       pp->removeTab(_tabPrint, this);
+      pp->removeTab(_tabPageSize, this);
     }
   }
 
   void GraphicSheet::properties(PropertyWidget * w) const
   {
     TRACE;
-    w->setValue(GraphicSheetProperties::Transparency, transparency());
-    w->setValue(GraphicSheetProperties::TransparentMask, transparentMask());
+    if(w->id()==_tabPrint) {
+      w->setValue(GraphicSheetProperties::Transparency, transparency());
+      w->setValue(GraphicSheetProperties::TransparentMask, transparentMask());
+    } else if(w->id()==_tabPageSize) {
+      _pageSize.properties(w);
+    }
   }
 
-  void GraphicSheet::setProperty(uint, int pid, QVariant val)
+  void GraphicSheet::setProperty(uint wid, int pid, QVariant val)
   {
     TRACE;
-    switch(pid) {
-    case GraphicSheetProperties::Transparency:
-      setTransparency(val.toInt());
-      break;
-    case GraphicSheetProperties::TransparentMask:
-      setTransparentMask(val.toBool());
-      break;
+    if(wid==_tabPrint) {
+      switch(pid) {
+      case GraphicSheetProperties::Transparency:
+        setTransparency(val.toInt());
+        break;
+      case GraphicSheetProperties::TransparentMask:
+        setTransparentMask(val.toBool());
+        break;
+      }
+    } else if(wid==_tabPageSize) {
+      _pageSize.setProperty(pid, val);
     }
   }
 
@@ -237,7 +252,11 @@ namespace SciFigs {
         obj->setObjectName(obj->objectName());
       }
       if(show) {
-        showObject(obj);
+        if(_contexts.count()>1) {
+          static_cast<GraphicObjectGroup *>(context())->addObject(obj);
+        } else {
+          showObject(obj);
+        }
       }
     }
   }
@@ -282,8 +301,7 @@ namespace SciFigs {
   /*!
     Remove object obj from the sheet.
 
-    If object is not removable (see QObject::isRemovable()), it will be removed only if force is true
-    (default value is true).
+    If object is not removable (see QObject::isRemovable()), it will be removed only if force is true.
 
     Return true if object correctly removed.
   */
@@ -291,13 +309,14 @@ namespace SciFigs {
   {
     TRACE;
     if(force || obj->isRemovable()) {
-      SelectionWidget * tmp=obj->selectionWidget();
-      if(proxy()) obj->removeProperties(proxy());
-      if(_currentActiveSelection==tmp)
-        _currentActiveSelection=nullptr;
-      if(tmp) {
-        tmp->widget()->setSelected(nullptr);
-        tmp->deleteLater();
+      if(proxy()) {
+        obj->removeProperties(proxy());
+      }
+      if(obj->selectionState()!=GraphicObject::None) {
+        removeSelection(obj);
+      }
+      if(_activeObject==obj) {
+        _activeObject=nullptr;
       }
       obj->close();
       autoResizeContent();
@@ -317,8 +336,9 @@ namespace SciFigs {
     GraphicObjectList wList=objects();
     int nRemoved=wList.count();
     for(GraphicObjectList::iterator it=wList.begin();it!=wList.end();++it) {
-      if(removeObject(*it, force))
+      if(removeObject(*it, force)) {
         nRemoved--;
+      }
     }
     switch(nRemoved) {
     case 0:
@@ -329,9 +349,13 @@ namespace SciFigs {
     default:
       Message::warning(MSG_ID, tr("Clearing sheet"), tr("%1 objects are not removable").arg(nRemoved), Message::ok());
     }
+    // Reset properties to default
+    _transparency=255;
+    _transparentMask=false;
+    _pageSize=PageSize();
   }
 
-  void GraphicSheet::showObject(GraphicObject *obj)
+  void GraphicSheet::showObject(GraphicObject * obj)
   {
     TRACE;
     connect(obj, SIGNAL(wantToBeSelected(GraphicObject*, Qt::KeyboardModifiers)),
@@ -341,6 +365,7 @@ namespace SciFigs {
     connect(obj, SIGNAL(draggedTo(QPoint)), this, SLOT(dragObjects(QPoint)));
     connect(obj, SIGNAL(sizeChanged()), this, SLOT(resizeObject()));
     obj->setParent(widget());
+    obj->setSheet(this);
     obj->setPos();
     obj->setSize();
     obj->show();
@@ -378,10 +403,7 @@ namespace SciFigs {
       if(proxy()) {
          obj->updateGeometryProperties(proxy());
       }
-      SelectionWidget * selObj=obj->selectionWidget();
-      if(selObj) {
-        selObj->move(selObj->x(obj->x()), selObj->y(obj->y()));
-      }
+      _selection->update();
       //if(obj->_orderTip) obj->_orderTip->move(obj->x()-widget()->x(),obj->y()-widget()->y());
       autoResizeContent();
     }
@@ -396,10 +418,7 @@ namespace SciFigs {
     TRACE;
     GraphicObject * obj=static_cast<GraphicObject *>(sender());
     obj->setSize();
-    SelectionWidget * selObj=obj->selectionWidget();
-    if(selObj) {
-      selObj->resize(selObj->width(), selObj->height());
-    }
+    _selection->update();
     autoResizeContent();
   }
 
@@ -409,15 +428,17 @@ namespace SciFigs {
     occur every time which is quite slow and CPU demanding.
 
     Effective dragging of object is always delayed by 50 milliseconds. If another event occur less than 50 milliseconds
-    after a first one, \a delta modified to be as closed as possible to the current mouse position but no
+    after a first one, trackaing point is modified to be as closed as possible to the current mouse position but no
     drag action is processed. At end of the 50 milliseconds, dragObjectsDelayed() is called to effectively drag the
     selected objects.
+
+    \a p is expressed in current context.
   */
   void GraphicSheet::dragObjects(QPoint p)
   {
     TRACE;
-    _mouseTrackingPoint=mapFromGlobal(p);
-    if(!_dragDelay.isActive()) {
+    _mouseTrackingPoint=p;
+    if(!_dragDelay.isActive()) { // Maximum one effective drag per 50 ms
       _dragDelay.start();
     }
   }
@@ -425,7 +446,7 @@ namespace SciFigs {
   void GraphicSheet::dragObjectsDelayed()
   {
     TRACE;
-    Point d=widget()->mapFromParent(_mouseTrackingPoint);
+    Point d=_mouseTrackingPoint;
     d-=GraphicObject::draggingPos();
     Point2D dcm(d);
     dcm/=SciFigsGlobal::screenResolution();
@@ -446,57 +467,33 @@ namespace SciFigs {
     if(proxy()) {
       proxy()->saveStates();
     }
-    SelectionWidget * selObj;
-    if(obj) {
-      selObj=obj->selectionWidget();
-    } else {
-      selObj=nullptr;
+    if(!(m & Qt::ControlModifier)) { // Unselect all if necessary
+      _selection->clear(proxy());
+      _activeObject=nullptr;
     }
-    if(selObj) {
-      selObj->setState(SelectionWidget::Activated);
+    if(obj && obj->belongsTo(context())) {
+      if(obj->selectionState()==GraphicObject::None) {
+        addSelection(obj, GraphicObject::Activated);
+      } else {
+        obj->setSelectionState(GraphicObject::Activated);
+      }
       emit activeSelectionChanged(obj);
       if(proxy()) {
         obj->addProperties(proxy());
       }
-    } else if(obj) {
-      selObj=addSelection(obj);
-      selObj->show(); // Here and not in addSelection(), bug observed with Qt 4.6 under Mac, show() calls GraphicObject::enterEvent()
-      if(proxy()) {
-        obj->addProperties(proxy());
+      if(_activeObject && _activeObject!=obj) {
+        _activeObject->setSelectionState(GraphicObject::Selected);
+      }
+      _activeObject=obj;
+      if(_statusBar) {
+        _statusBar->showMessage(tr("Hit CTRL+M to display popup menu of the active object"));
       }
     }
-    if(!(m & Qt::ControlModifier)) {
-      // Unselect all
-      QList<SelectionWidget *> wList=findChildren<SelectionWidget>();
-      SelectionWidget * tmp;
-      foreach(tmp, wList) {
-        if(tmp!=selObj) {
-          tmp->widget()->setSelected(nullptr);
-          if(proxy()) {
-            tmp->widget()->removeProperties(proxy());
-          }
-          delete tmp;
-        }
-      }
-      // Bug fixed 28/10/04, if no control key, all selection widgets are removed
-      // Object eventually pointed by current avtive selection is deleted.
-      _currentActiveSelection=nullptr;
-      // Bug fixed for Qt 4.3 : selection mark remains visible if sheet not updated
-  #if(QT_VERSION < QT_VERSION_CHECK(4, 4, 0))
-      update();
-  #endif
-      // Bug fixed with Qt 4.4: update makes the selection quite slow with a lot of objects
-    }
-    if(_currentActiveSelection && _currentActiveSelection!=selObj) {
-      _currentActiveSelection->setState(SelectionWidget::Selected);
-    }
-    _currentActiveSelection=selObj;
-    if(_currentActiveSelection && _statusBar)
-      _statusBar->showMessage(tr("Hit CTRL+M to display popup menu of the active object"));
     if(proxy()) {
       proxy()->restoreStates();
       proxy()->setValues();
     }
+    _selection->update();
   }
 
   void GraphicSheet::selectObjects(const QRect& r, Qt::KeyboardModifiers m)
@@ -524,15 +521,16 @@ namespace SciFigs {
     }
   }
 
-  SelectionWidget * GraphicSheet::addSelection(GraphicObject * obj)
+  void GraphicSheet::addSelection(GraphicObject * obj, GraphicObject::SelectionState s)
   {
     TRACE;
-    SelectionWidget * selObj=new SelectionWidget(obj, this);
-    selObj->setParent(widget());
-    selObj->move(selObj->x(obj->x()), selObj->y(obj->y()));
-    selObj->resize(selObj->width(), selObj->height());
-    selObj->stackUnder(obj);
-    return selObj;
+    _selection->addObject(obj, s);
+  }
+
+  void GraphicSheet::removeSelection(GraphicObject * obj)
+  {
+    TRACE;
+    _selection->removeObject(obj);
   }
 
   /*!
@@ -543,10 +541,9 @@ namespace SciFigs {
   GraphicObject * GraphicSheet::activeObject(const char * className)
   {
     TRACE;
-    if(_currentActiveSelection) {
-      GraphicObject * obj=_currentActiveSelection->widget();
-      if(!className || obj->inherits(className))
-        return obj;
+    if(_activeObject) {
+      if(!className || _activeObject->inherits(className))
+        return _activeObject;
     }
     return nullptr;
   }
@@ -555,8 +552,8 @@ namespace SciFigs {
   {
     TRACE;
     // greatest right-bottom coordinates of child graphs;
-    // iterate on all graphs
-    GraphicObjectList objectList=objects();
+    // iterate on all graphs even outside context
+    GraphicObjectList objectList=widget()->findChildren<GraphicObject *>(QString(), Qt::FindDirectChildrenOnly);
     int maxRight=-INT_MAX, maxBottom=-INT_MAX;
     GraphicObject * obj;
     foreach(obj, objectList) {
@@ -673,9 +670,8 @@ namespace SciFigs {
     GraphicObject * obj;
     foreach(obj, objectList) {
       obj->deepUpdate();
-      if(obj->selectionWidget())
-        obj->selectionWidget() ->update();
     }
+    _selection->update();
   }
 
   /*!
@@ -690,9 +686,8 @@ namespace SciFigs {
     GraphicObject * obj;
     foreach(obj, objectList) {
       obj->update();
-      if(obj->selectionWidget())
-        obj->selectionWidget() ->update();
     }
+    _selection->update();
   }
 
   int GraphicSheet::maximumResolution()
@@ -756,16 +751,17 @@ namespace SciFigs {
       }
       if(imageFormat=="PDF") {
         QPrinter printer(QPrinter::HighResolution);
-        if(dpi==0)
+        if(dpi==0) {
           printer.setResolution(maximumResolution());
-        else
+        } else {
           printer.setResolution(dpi);
-        printer.setFullPage(true);
+        }
         printer.setOutputFileName(fileName);
         printer.setOutputFormat(QPrinter::PdfFormat);
+        printer.setPageLayout(_pageSize.pageLayout());
         QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
         print(printer);
-      } else if(imageFormat =="SVG") {
+      } else if(imageFormat=="SVG") {
         printSvg(fileName, dpi);
       } else { // Pixel based image format
         QPixmap pixmap=image(dpi);
@@ -796,7 +792,9 @@ namespace SciFigs {
     progress->setWindowTitle(tr("Please wait"));
     progress->setMaximum(nObjects-1);
 
-    if(dpi==0) dpi=maximumResolution();
+    if(dpi==0) {
+      dpi=maximumResolution();
+    }
     double resolution=dpi/2.54;
     double factor=resolution/SciFigsGlobal::screenResolution();
     // Find limits of actual sheet
@@ -930,11 +928,12 @@ namespace SciFigs {
   {
     TRACE;
     QPrinter printer(QPrinter::HighResolution);
-    if(dpi==0)
+    if(dpi==0) {
       printer.setResolution(maximumResolution());
-    else
+    } else {
       printer.setResolution(dpi);
-    printer.setFullPage(true);
+    }
+    printer.setPageLayout(_pageSize.pageLayout());
     if(Settings::printSetup(&printer)) {
       QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
       print(printer);
@@ -1004,7 +1003,7 @@ namespace SciFigs {
     TRACE;
     // Remove all existing limits
     PageLimit * limit;
-    QList<PageLimit *> pageList=findChildren<PageLimit>();
+    QList<PageLimit *> pageList=findChildren<PageLimit>(Qt::FindDirectChildrenOnly);
     foreach(limit, pageList) delete limit;
     // Create new limits
     int wpage=qRound(p.widthMM()*SciFigsGlobal::screenResolution()*0.1);
@@ -1020,11 +1019,10 @@ namespace SciFigs {
       limit=new PageLimit(PageLimit::Horizontal, widget());
       limit->move(0, h);
       limit->show();
-      h += hpage;
+      h+=hpage;
     }
   }
 
-
   /*!
     Copy all properties of the sheet and included objects to the clipboard
   */
@@ -1174,12 +1172,12 @@ namespace SciFigs {
     case Qt::LeftButton:
       if(!_mouseTrackingRect) {
         _mouseTrackingRect=new QRect;
-        _mouseTrackingRect->moveTopLeft(widget()->mapFromParent(e->pos()));
+        _mouseTrackingRect->moveTopLeft(context()->mapFrom(this, e->pos()));
       }
       return;
     case Qt::RightButton: {
         QMenu m;
-        m.addActions(actions());
+        addActions(&m);
         m.exec(mapToGlobal(e->pos()));
       }
       return;
@@ -1223,7 +1221,7 @@ namespace SciFigs {
   {
     TRACE;
     if(_mouseTrackingRect) {
-      _mouseTrackingPoint=e->pos();
+      _mouseTrackingPoint=context()->mapFrom(this, e->pos());
       scrollSelection();
     } else {
       QScrollArea::mouseMoveEvent(e);
@@ -1245,7 +1243,8 @@ namespace SciFigs {
   bool GraphicSheet::scrollSelection()
   {
     TRACE;
-    const QPoint& p=_mouseTrackingPoint;
+    QPoint p=context()->mapTo(this, _mouseTrackingPoint);
+    QPoint gp=mapToGlobal(p);
     bool scrolled=false;
     if(p.x()<10) {
       horizontalScrollBar()->setValue(horizontalScrollBar()->value()+p.x()-10);
@@ -1263,12 +1262,16 @@ namespace SciFigs {
     }
     // If tracking selection rectangle, repeat scroll if the mouse remains on the edge
     if(_mouseTrackingRect) {
-      _mouseTrackingRect->setBottomRight(widget()->mapFromParent(_mouseTrackingPoint));
+      _mouseTrackingRect->setBottomRight(_mouseTrackingPoint);
       _selection->update();
       if(scrolled) {
         QTimer::singleShot(500, this, SLOT(scrollSelection()));
       }
     }
+    // Ensure scrolling without moving the mouse
+    if(scrolled) {
+      _mouseTrackingPoint=context()->mapFromGlobal(gp);
+    }
     return scrolled;
   }
 
@@ -1281,14 +1284,12 @@ namespace SciFigs {
     TRACE;
     GraphicObjectList objectList=objects();
     if(objectList.isEmpty()) return;
-    GraphicObject * cur;
-    if(_currentActiveSelection)
-      cur=_currentActiveSelection->widget();
-    else
-      cur=nullptr;
     GraphicObjectList::iterator it;
     for(it=objectList.begin(); it!=objectList.end(); ++it) {
-      if(*it==cur) {++it;break;}
+      if(*it==_activeObject) {
+        ++it;
+        break;
+      }
     }
     if(it!=objectList.end()) {
       selectObjects(*it, Qt::NoModifier);
@@ -1307,14 +1308,12 @@ namespace SciFigs {
     TRACE;
     GraphicObjectList objectList=objects();
     if(objectList.isEmpty()) return;
-    GraphicObject * cur;
-    if(_currentActiveSelection)
-      cur=_currentActiveSelection->widget();
-    else
-      cur=nullptr;
     GraphicObjectList::iterator it;
     for(it=objectList.begin(); it!=objectList.end(); ++it) {
-      if(*it==cur) {++it;break;}
+      if(*it==_activeObject) {
+        ++it;
+        break;
+      }
     }
     if(it!=objectList.end()) {
       selectObjects(*it, Qt::ControlModifier);
@@ -1377,10 +1376,10 @@ namespace SciFigs {
   void GraphicSheet::alignX(double localPosition)
   {
     TRACE;
-    if(_currentActiveSelection) {
-      GraphicObject * obj=_currentActiveSelection->widget();
-      double refPos=obj->printLeft()+localPosition*obj->printWidth();
+    if(_activeObject) {
+      double refPos=_activeObject->printLeft()+localPosition*_activeObject->printWidth();
       GraphicObjectList objectList=selectedObjects<GraphicObject>();
+      GraphicObject * obj;
       foreach(obj, objectList) {
         obj->setPrintXAnchor(refPos-localPosition*obj->printWidth());
         moveObject(obj);
@@ -1396,10 +1395,10 @@ namespace SciFigs {
   void GraphicSheet::alignY(double localPosition)
   {
     TRACE;
-    if(_currentActiveSelection) {
-      GraphicObject * obj=_currentActiveSelection->widget();
-      double refPos=obj->printTop()+localPosition*obj->printHeight();
+    if(_activeObject) {
+      double refPos=_activeObject->printTop()+localPosition*_activeObject->printHeight();
       GraphicObjectList objectList=selectedObjects<GraphicObject>();
+      GraphicObject * obj;
       foreach(obj, objectList) {
         obj->setPrintYAnchor(refPos-localPosition*obj->printHeight());
         moveObject(obj);
@@ -1413,16 +1412,16 @@ namespace SciFigs {
   void GraphicSheet::alignGrid()
   {
     TRACE;
-    if(_currentActiveSelection) {
+    if(_activeObject) {
       AlignGridParameters * d=new AlignGridParameters(this);
       Settings::getWidget(d);
       if(d->exec()==QDialog::Accepted) {
         Settings::setWidget(d);
 
         Point2D step=d->step();
-        GraphicObject * obj=_currentActiveSelection->widget();
-        Point2D origin=Point2D(obj->printLeft(), obj->printTop());
+        Point2D origin=Point2D(_activeObject->printLeft(), _activeObject->printTop());
         GraphicObjectList objectList=selectedObjects<GraphicObject>();
+        GraphicObject * obj;
         foreach(obj, objectList) {
           Point2D pos=Point2D(obj->printLeft(), obj->printTop());
           pos-=origin;
@@ -1451,11 +1450,10 @@ namespace SciFigs {
     GraphicObject * obj;
     foreach(obj, objectList) {
       obj->raise();
-      if(obj->selectionWidget())
-        obj->selectionWidget() ->stackUnder(obj);
     }
-    if(_orderIndex>=0)
+    if(_orderIndex>=0) {
       showOrderIndex(true);
+    }
   }
 
   /*!
@@ -1468,10 +1466,10 @@ namespace SciFigs {
     GraphicObject * obj;
     foreach(obj, objectList) {
       obj->lower();
-      obj->selectionWidget()->lower();
     }
-    if(_orderIndex>=0)
+    if(_orderIndex>=0) {
       showOrderIndex(true);
+    }
   }
 
   void GraphicSheet::setOrderIndex(GraphicObject * obj)
@@ -1492,8 +1490,6 @@ namespace SciFigs {
       GraphicObject * objAbove=*it;
       if(objAbove) {
         obj->stackUnder(objAbove);
-        if(obj->selectionWidget())
-          obj->selectionWidget() ->stackUnder(obj);
         _orderIndex++;
       }
       showOrderIndex(true);
@@ -1646,7 +1642,6 @@ namespace SciFigs {
     if(m) m->addAction(a);
 
     a=new QAction(QIcon(":ordertool.png"), tr("&Order"), this);
-    if(shortcuts) a->setShortcut(tr("Ctrl+O"));
     a->setStatusTip(tr("Order objects of the sheet"));
     a->setCheckable(true);
     connect(a, SIGNAL(toggled(bool)), this, SLOT(showOrderIndex(bool)));
@@ -1744,7 +1739,7 @@ namespace SciFigs {
     emit currentFileChanged(_currentFile);
   }
 
-  void GraphicSheet::fileOpen()
+  bool GraphicSheet::fileOpen()
   {
     TRACE;
     QString fileName=Message::getOpenFileName(tr("Open an existing document"), tr("Page file (*.page)"));
@@ -1752,6 +1747,9 @@ namespace SciFigs {
     QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
     if(fileOpen(fileName)) {
       addRecentFile(fileName);
+      return true;
+    } else {
+      return false;
     }
   }
 
@@ -1762,7 +1760,7 @@ namespace SciFigs {
   {
     TRACE;
     bool ret=false;
-    if(fileName.length()>0) {
+    if(!fileName.isEmpty()) {
       QFileInfo fi(fileName);
       fileName=fi.absoluteFilePath();
       if(_statusBar) {
@@ -1844,8 +1842,8 @@ namespace SciFigs {
   {
     TRACE;
     copy();
-    if(_currentActiveSelection) {
-      _currentActiveSelection->widget()->deleteObject();
+    if(_activeObject) {
+      _activeObject->deleteObject();
     }
   }
 
@@ -1923,6 +1921,7 @@ namespace SciFigs {
   void GraphicSheet::xml_writeChildren(XML_WRITECHILDREN_ARGS) const
   {
     TRACE;
+    _pageSize.xml_save(s, context);
     GraphicObjectList objectList;
     XMLSciFigs * scifigsContext=static_cast<XMLSciFigs *>(context);
     if(scifigsContext->selection()) {
@@ -1943,6 +1942,9 @@ namespace SciFigs {
   XMLMember GraphicSheet::xml_member(XML_MEMBER_ARGS)
   {
     TRACE;
+    if(tag==PageSize::xmlPageSizeTag) {
+      return XMLMember(&_pageSize);
+    }
     static const QString tmp("objectName");
     XMLSciFigs * scifigsContext=static_cast<XMLSciFigs *>(context);
     XMLRestoreAttributeIterator it=attributes.find(tmp);
@@ -1951,7 +1953,7 @@ namespace SciFigs {
         GraphicObject * obj=GraphicObjectFactory::instance()->create(tag.toString());
         if(obj) {
           obj->setRemovable(true);
-          addObject(obj);
+          addObject(obj, false);
           return XMLMember(obj);
         } else {
           App::log(tr("Unknown object type %1\n").arg(tag.toString()) );
@@ -1990,8 +1992,10 @@ namespace SciFigs {
   {
     TRACE;
     Q_UNUSED(context)
-    showObject(static_cast<GraphicObject *>(child));
-    _loadedObjects.append(static_cast<GraphicObject *>(child));
+    if(child->xml_tagName()!=PageSize::xmlPageSizeTag) {
+      showObject(static_cast<GraphicObject *>(child));
+      _loadedObjects.append(static_cast<GraphicObject *>(child));
+    }
   }
 
   bool GraphicSheet::xml_polish(XML_POLISH_ARGS)
@@ -2007,11 +2011,17 @@ namespace SciFigs {
   {
     TRACE;
     QPrinter printer(QPrinter::HighResolution);
-    if(dpi==0) printer.setResolution(maximumResolution());
-    else printer.setResolution(dpi);
+    if(dpi==0) {
+      printer.setResolution(maximumResolution());
+    } else {
+      printer.setResolution(dpi);
+    }
     printer.setFullPage(true);
-    if(outputToFile) printer.setOutputFileName(name);
-    else printer.setPrinterName(name);
+    if(outputToFile) {
+      printer.setOutputFileName(name);
+    } else {
+      printer.setPrinterName(name);
+    }
     print(printer);
   }
 
@@ -2095,6 +2105,7 @@ namespace SciFigs {
     TRACE;
     GraphicObjectList selWList=selectedObjects<GraphicObject>();
     if(selWList.count()>1) {
+      selectObjects(nullptr);
       GraphicObjectGroup * g=new GraphicObjectGroup(nullptr);
       g->setObjectName("Group");
       addObject(g);
@@ -2127,7 +2138,6 @@ namespace SciFigs {
         GraphicObjectList wList=g->objects();
         GraphicObject * w;
         foreach(w, wList) {
-          w->setSelectable(true);
           w->setPrintLeft(g->printLeft()+w->printLeft());
           w->setPrintTop(g->printTop()+w->printTop());
           showObject(w);
@@ -2147,6 +2157,29 @@ namespace SciFigs {
     }
   }
 
+  void GraphicSheet::enterGroup()
+  {
+    TRACE;
+    if(!_activeObject) {
+      return;
+    }
+    GraphicObjectGroup * cur=qobject_cast<GraphicObjectGroup *>(_activeObject);
+    if(!cur) {
+      return;
+    }
+    selectObjects(nullptr);
+    _contexts.append(cur);
+  }
+
+  void GraphicSheet::exitGroup()
+  {
+    TRACE;
+    if(_contexts.count()>1) {
+      selectObjects(nullptr);
+      _contexts.removeLast();
+    }
+  }
+
   AxisWindow * GraphicSheet::addGraph()
   {
     TRACE;
