@@ -307,23 +307,7 @@ void LineLayer::drawCurve(const GraphContentOptions& gc, QPainter& p, double dot
                           Rect& r, const AbstractLine& line, int startAt, int endAt) const
 {
   TRACE;
-  if(!line.isVisible() || line.count()==0) {
-    return;
-  }
-  if(line.isSorted()) {
-    line.indexRange(r.x1(), r.x2(), startAt, endAt);
-  }
-  Point p1;
-  while(startAt<endAt) {
-    if(line.isValid(startAt)) {
-      p1=line.point(startAt, _pointOptions);
-      break;
-    }
-    startAt++;
-  }
-  if(endAt<=startAt) {
-    return;
-  }
+  Point p1=line.point(startAt, _pointOptions);
   QPoint pscreen1, pscreen2;
   QPolygon ap(endAt-startAt);
   Pen pen=line.pen();
@@ -432,7 +416,30 @@ void LineLayer::paintData(const LayerPainterRequest& lp, QPainter & p, double do
   for(it=begin() ;it!=end(); ++it) {
     if(lp.terminated()) return;
     AbstractLine& line=**it;
-    drawCurve(gc, p, dotpercm, h, r, line, 0, line.count());
+    if(line.isVisible() && line.count()>0) {
+      int startAt=0;
+      int endAt=line.count();
+      if(line.isSorted()) {
+        line.indexRange(r.x1(), r.x2(), startAt, endAt);
+      }
+      // First and last segments may have one point outside of visible area
+      while(startAt>0 && line.isValid(startAt)) {
+        startAt--;
+      }
+      while(endAt<line.count() && line.isValid(endAt)) {
+        endAt++;
+      }
+      // Go to first valid point
+      while(startAt<endAt) {
+        if(line.isValid(startAt)) {
+          break;
+        }
+        startAt++;
+      }
+      if(startAt<endAt) {
+        drawCurve(gc, p, dotpercm, h, r, line, startAt, endAt);
+      }
+    }
   }
 }
 
@@ -721,7 +728,7 @@ bool LineLayer::xml_setProperty(XML_SETPROPERTY_ARGS)
 {
   TRACE;
   Q_UNUSED(attributes)
-  switch (memberID) {
+  switch(memberID) {
   case 0: {
       AbstractLine * line=LineFactory::instance()->create(content.toString());
       if(line) {
@@ -750,16 +757,16 @@ void LineLayer::setPenColor(int curveIndex, int red, int green, int blue)
 }
 
 void LineLayer::setPenStyle(int curveIndex, double width, QString lineStyle,
-                                QString capStyle, QString joinStyle)
+                            QString capStyle, QString joinStyle)
 {
   TRACE;
   if(curveIndex>=count()) return;
   AbstractLine * l=_lines.at(curveIndex);
   Pen p=l->pen();
   p.setWidth(width);
-  p.setStyle(Pen::styleValue(lineStyle) );
-  p.setCapStyle(Pen::capStyleValue(capStyle) );
-  p.setJoinStyle(Pen::joinStyleValue(joinStyle) );
+  p.setStyle(Pen::styleValue(lineStyle));
+  p.setCapStyle(Pen::capStyleValue(capStyle));
+  p.setJoinStyle(Pen::joinStyleValue(joinStyle));
   LayerLocker ll(this);
   l->setPen(p);
 }
@@ -800,7 +807,7 @@ void LineLayer::setSymPenColor(int curveIndex, int red, int green, int blue)
 }
 
 void LineLayer::setSymPenStyle(int curveIndex, double width,
-                                             QString lineStyle, QString capStyle, QString joinStyle)
+                               QString lineStyle, QString capStyle, QString joinStyle)
 {
   TRACE;
   if(curveIndex>=count()) return;
@@ -808,8 +815,8 @@ void LineLayer::setSymPenStyle(int curveIndex, double width,
   Symbol s=l->symbol();
   Pen p=s.pen();
   p.setWidth(width);
-  p.setStyle(Pen::styleValue(lineStyle) );
-  p.setCapStyle(Pen::capStyleValue(capStyle) );
+  p.setStyle(Pen::styleValue(lineStyle));
+  p.setCapStyle(Pen::capStyleValue(capStyle));
   p.setJoinStyle(Pen::joinStyleValue(joinStyle));
   s.setPen(p);
   LayerLocker ll(this);
