@@ -414,6 +414,7 @@ TargetList * TargetListWidget::targetList()
   }
   if(_ellPeakWidget) {
     tl->ellipticityPeakTarget().setValue(_ellPeakWidget->value());
+    tl->ellipticityPeakTarget().setMinimumAmplitude(_ellPeakWidget->minimumAmplitude());
     tl->ellipticityPeakTarget().setMisfitWeight(ellPeakWeight->text().toDouble());
     tl->ellipticityPeakTarget().setMinimumMisfit(ellPeakMinMisfit->text().toDouble());
   }
@@ -495,6 +496,7 @@ void TargetListWidget::setFrom(TargetList * tl)
     on_ellPeakSet_clicked();
     ASSERT(_ellPeakWidget);
     _ellPeakWidget->setValue(tl->ellipticityPeakTarget().value());
+    _ellPeakWidget->setMinimumAmplitude(tl->ellipticityPeakTarget().minimumAmplitude());
     ellPeakWeight->setText(QString::number(tl->ellipticityPeakTarget().misfitWeight()));
     ellPeakMinMisfit->setText(QString::number(tl->ellipticityPeakTarget().minimumMisfit()));
   } else {
