@@ -56,14 +56,9 @@ int modeWaveform(int argc, char ** argv, QString exportFile);
 
 int showGroups();
 SignalFileFormat getExportFormat(QString fStr);
-SignalFileFormat getImportFormat(QString fStr);
 bool listExportFormats();
 bool listImportFormats();
 
-void loadSignals(int& argc, char ** argv);
-SubSignalPool loadSignals(SignalDatabase * db, bool viewFiles);
-bool checkRemainingArguments(int argc, char ** argv);
-
 static bool debugStream=false;
 
 int main(int argc, char ** argv)
@@ -91,6 +86,10 @@ int main(int argc, char ** argv)
       } else if(arg=="-import-format") {
         CoreApplication::checkOptionArg(i, argc, argv);
         format=argv[i];
+        if(SignalFileFormat::fromName(format)==SignalFileFormat::Unknown) {
+          App::log(tr("geopsy: unknown file format %1, see -h for help\n").arg(format) );
+          return 2;
+        }
       } else if(arg=="-group") {
         CoreApplication::checkOptionArg(i, argc, argv);
         groupName=argv[i];
@@ -114,7 +113,7 @@ int main(int argc, char ** argv)
       } else if(arg=="-port") {
         CoreApplication::checkOptionArg(i, argc, argv);
         mode=SeedlinkView;
-        serverPort=atoi(argv[i]);
+        serverPort=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-streams") {
         CoreApplication::checkOptionArg(i, argc, argv);
         mode=SeedlinkView;
@@ -153,7 +152,7 @@ int main(int argc, char ** argv)
       argv[j++]=argv[i];
     }
   }
-  if(j < argc) {
+  if(j<argc) {
     argv[j]=nullptr;
     argc=j;
   }
@@ -176,8 +175,8 @@ int modeGui(int argc, char ** argv, bool seedlinkView)
 {
   Application a(argc, argv, help);
 
-  loadSignals(argc, argv);
-  if(!checkRemainingArguments(argc,argv)) {
+  signalFiles+=CoreApplication::getFileList(argc, argv);
+  if(!CoreApplication::checkRemainingArgs(argc,argv)) {
     return 2;
   }
 
@@ -206,7 +205,7 @@ int modeGui(int argc, char ** argv, bool seedlinkView)
   if(seedlinkView) {
     w->seedlink(serverAddress, serverPort, streamSelection);
   } else {
-    loadSignals(w->database(), true);
+    w->database()->load(database, groupName, signalFiles, format, true);
   }
 
  int appReturn=qApp->exec();
@@ -234,7 +233,7 @@ int modeExport(int argc, char ** argv, QString exportFile)
         useOriginalBaseName=true;
       } else if(arg=="-export-max-signal") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        maximumSignalsPerFile=atoi(argv[i]);
+        maximumSignalsPerFile=CoreApplication::toInt(i, i-1, argv);
         if(maximumSignalsPerFile<1) {
           App::log(tr("geopsy: maximum number of signals per file must be greater than 0\n") );
           return 2;
@@ -264,9 +263,9 @@ int modeExport(int argc, char ** argv, QString exportFile)
   Engine gp;
   gp.setConsoleProgress(showProgress);
   SignalDatabase db;
-  loadSignals(argc, argv);
-  SubSignalPool subPool=loadSignals(&db, false);
-  if(!checkRemainingArguments(argc,argv)) {
+  signalFiles+=CoreApplication::getFileList(argc, argv);
+  SubSignalPool subPool=db.load(database, groupName, signalFiles, format, false);
+  if(!CoreApplication::checkRemainingArgs(argc,argv)) {
     return 2;
   }
   // Check if there are signals
@@ -297,7 +296,7 @@ int modeWaveform(int argc, char ** argv, QString waveformFile)
       argv[j++]=argv[i];
     }
   }
-  if(j < argc) {
+  if(j<argc) {
     argv[j]=nullptr;
     argc=j;
   }
@@ -309,11 +308,11 @@ int modeWaveform(int argc, char ** argv, QString waveformFile)
   GeopsyCoreEngine gp;
   gp.setConsoleProgress(showProgress);
   SignalDatabase db;
-  loadSignals(argc, argv);
-  SubSignalPool subPool=loadSignals(&db, false);
-  if(!checkRemainingArguments(argc,argv)) {
+  signalFiles+=CoreApplication::getFileList(argc, argv);
+  if(!CoreApplication::checkRemainingArgs(argc,argv)) {
     return 2;
   }
+  SubSignalPool subPool=db.load(database, groupName, signalFiles, format, false);
   // Check if there are signals
   if(subPool.isEmpty()) {
     App::log(tr("geopsy: no signal available, check options -file, -db or -group, see -h for help\n") );
@@ -338,7 +337,7 @@ int modeGroupList(int argc, char ** argv)
 {
   CoreApplication a(argc, argv, help);
 
-  if(!checkRemainingArguments(argc,argv)) {
+  if(!CoreApplication::checkRemainingArgs(argc,argv)) {
     return 2;
   }
   GeopsyCoreEngine gp;
@@ -361,70 +360,6 @@ int modeGroupList(int argc, char ** argv)
   }
 }
 
-/*!
-  Must be called after all other option parsing. Only non-option arguments are extracted.
-*/
-void loadSignals(int& argc, char ** argv)
-{
-  // Get file list from non-option arguments
-  int j=1;
-  for(int i=1;i<argc; i++) {
-    QByteArray arg=argv[i];
-    if(arg[0]!='-') {
-      signalFiles.append(argv[i] );
-    } else if(arg=="--") {
-      for(i++;i<argc; i++) {
-        argv[j++]=argv[i];
-      }
-      break;
-    } else {
-      argv[j++]=argv[i];
-    }
-  }
-  if(j < argc) {
-    argv[j]=nullptr;
-    argc=j;
-  }
-}
-
-SubSignalPool loadSignals(SignalDatabase * db, bool viewFiles)
-{
-  SubSignalPool subPool;
-  if(!database.isEmpty() && db->open(database)) {
-    if(!groupName.isEmpty()) {
-      AbstractSignalGroup * group=db->findGroup(groupName);
-      if(group) {
-        subPool=group->subPool();
-        if(viewFiles && !subPool.isEmpty()) {
-          GeopsyCoreEngine::instance()->newSignalViewer(subPool);
-        }
-      } else {
-        App::log(tr("geopsy: unknown group of signals (%1), see -groups\n").arg(groupName) );
-      }
-    }
-  }
-  if(!signalFiles.isEmpty()) {
-    if(subPool.isEmpty()) {
-      subPool=db->load(signalFiles, getImportFormat(format), viewFiles);
-    } else {
-      db->load(signalFiles, getImportFormat(format), viewFiles);
-    }
-  }
-  return subPool;
-}
-
-bool checkRemainingArguments(int argc, char ** argv)
-{
-  if(argc>1) {
-    for(int i=1; i<argc; i++) {
-      App::log(tr("geopsy: bad option '%1', see -h\n").arg(argv[i]) );
-    }
-    return false;
-  } else {
-    return true;
-  }
-}
-
 SignalFileFormat getExportFormat(QString fStr)
 {
   SignalFileFormat format=SignalFileFormat::fromName(fStr);
@@ -438,17 +373,6 @@ SignalFileFormat getExportFormat(QString fStr)
   return format;
 }
 
-SignalFileFormat getImportFormat(QString fStr)
-{
-  if(fStr.isEmpty() || fStr.toLower()=="automatic") return SignalFileFormat::Unknown;
-  SignalFileFormat format=SignalFileFormat::fromName(fStr);
-  if(format.id()==SignalFileFormat::Unknown) {
-    App::log(tr("geopsy: unknown file format %1\n").arg(fStr) );
-    exit(2);
-  }
-  return format;
-}
-
 bool listExportFormats()
 {
   TRACE;
