@@ -177,7 +177,7 @@ void FKTimeWindows::setTimeWindowLayer(TimeWindowLayer * twLayer)
 /*!
   
 */
-bool FKTimeWindows::setParameters(const ArrayStations * array, double frequency, FKParameters * param)
+bool FKTimeWindows::setParameters(const ArraySelection * array, double frequency, FKParameters * param)
 {
   TRACE;
   // Before changing anything to parameters, disable the grid plot (setComponent will reset it)
@@ -423,7 +423,7 @@ void FKTimeWindows::setCurrentBlockSet()
     App::log(tr("Processing block set starting at %1\n")
              .arg(winList.at(blocks.first()).start()
                   .toString(DateTime::defaultUserFormat)));
-    if(_crossSpectrum->calculate(blocks, func, nullptr)) {
+    if(_crossSpectrum->calculate(blocks, func)) {
       if(_gridLayer->opacity()>0.0) {
         _gridLayer->setFunction(func);
       } else if(_curveLayer->opacity()>0.0) {
@@ -438,7 +438,9 @@ void FKTimeWindows::setCurrentBlockSet()
 
   waveNumGrid->deepUpdate();
   if(_timeWindowLayer) _timeWindowLayer->deepUpdate();
-  //showFKPeaks();
+  if(showPeaksEdit->isChecked()) {
+    showFKPeaks();
+  }
 }
 
 void FKTimeWindows::highLightWindows(TimeWindowList& winList)
@@ -469,7 +471,7 @@ void FKTimeWindows::setPlotOptions()
     break;
   case ArrayStations::ThreeComponents:
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
       plotOption->clear();
       plotOption->addItem(tr("Wavenumber power"));
@@ -530,9 +532,9 @@ void FKTimeWindows::setPlotOption()
   switch(mode) {
   case ArrayStations::Vertical:
     ellipticity->setEnabled(false);
-    fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+    fk=AbstractFKFunction::create(mode, param, _gridCache);
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
     case FKParameters::ActiveConventional:
       viewKxCurve();
@@ -568,11 +570,11 @@ void FKTimeWindows::setPlotOption()
     break;
   case ArrayStations::ThreeComponents:
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
       ellipticity->setEnabled(false);
       _gridLayer->setVariables(LiveGridLayer::CrossXY);
-      fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+      fk=AbstractFKFunction::create(mode, param, _gridCache);
       viewKEllGrid();
       setKEllGridLimits();
       break;
@@ -587,7 +589,7 @@ void FKTimeWindows::setPlotOption()
         _gridLayer->setVariables(LiveGridLayer::InvCrossXY);
         break;
       }
-      fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+      fk=AbstractFKFunction::create(mode, param, _gridCache);
       viewKEllGrid();
       setKEllGridLimits();
       break;
@@ -603,7 +605,7 @@ void FKTimeWindows::setPlotOption()
       case 7:
         ellipticity->setEnabled(true);
         _gridLayer->setFixedVariable(Angle::degreesToRadians(ellipticity->value()));
-        fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+        fk=AbstractFKFunction::create(mode, param, _gridCache);
         viewKxKyGrid();
         setKxKyGridLimits();
         switch(plotOption->currentIndex()) {
@@ -645,7 +647,7 @@ void FKTimeWindows::setPlotOption()
       case 8:
         ellipticity->setEnabled(false);
         _gridLayer->setVariables(LiveGridLayer::Value_XYMaxZ);
-        fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+        fk=AbstractFKFunction::create(mode, param, _gridCache);
         viewKxKyGrid();
         setKxKyGridLimits();
         setPowerLimits();
@@ -653,7 +655,7 @@ void FKTimeWindows::setPlotOption()
       case 9:
         ellipticity->setEnabled(false);
         _gridLayer->setVariables(LiveGridLayer::Z_XYMaxZ);
-        fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+        fk=AbstractFKFunction::create(mode, param, _gridCache);
         viewKxKyGrid();
         setKxKyGridLimits();
         setEllipticityLimits();
@@ -661,7 +663,7 @@ void FKTimeWindows::setPlotOption()
       case 10:
         ellipticity->setEnabled(false);
         _gridLayer->setVariables(LiveGridLayer::CrossXY);
-        fk=AbstractFKFunction::create(AbstractFKFunction::Love, mode, param, _gridCache);
+        fk=AbstractFKFunction::create(mode, param, _gridCache);
         viewKxKyGrid();
         setKxKyGridLimits();
         setPowerLimits();
@@ -741,6 +743,18 @@ void FKTimeWindows::on_plotOption_currentIndexChanged(int)
   setPlotOption();
 }
 
+void FKTimeWindows::on_showPeaksEdit_toggled(bool checked)
+{
+  if(checked) {
+    showFKPeaks();
+  } else {
+    LayerLocker ll(_fkPeaks);
+    Curve<Point2D>& plot=static_cast<PlotLine2D *>(_fkPeaks->line(0))->curve();
+    plot.clear();
+    _fkPeaks->deepUpdate();
+  }
+}
+
 void FKTimeWindows::on_ellipticity_valueChanged(double)
 {
   setPlotOption();
@@ -768,7 +782,7 @@ void FKTimeWindows::currentVelocitySlowness(QPoint mousePos)
   switch(mode) {
   case ArrayStations::Vertical:
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
     case FKParameters::ActiveConventional:
       prop=Propagation1D;
@@ -781,7 +795,7 @@ void FKTimeWindows::currentVelocitySlowness(QPoint mousePos)
     break;
   case ArrayStations::ThreeComponents:
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
       prop=Propagation1D;
       break;
@@ -851,9 +865,9 @@ void FKTimeWindows::showFKPeaks()
 
   switch(mode) {
   case ArrayStations::Vertical:
-    fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+    fk=AbstractFKFunction::create(mode, param, _gridCache);
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
     case FKParameters::ActiveConventional:
       return;
@@ -865,7 +879,7 @@ void FKTimeWindows::showFKPeaks()
     return;
   case ArrayStations::ThreeComponents:
     switch(param->processType()) {
-    case FKParameters::ActiveDirectSteering:
+    case FKParameters::ActiveDS:
     case FKParameters::ActiveRTBF:
     case FKParameters::ActiveConventional:
       return;
@@ -880,10 +894,10 @@ void FKTimeWindows::showFKPeaks()
       case 6:
       case 7:
       case 8:
-        fk=AbstractFKFunction::create(AbstractFKFunction::Rayleigh, mode, param, _gridCache);
+        fk=AbstractFKFunction::create(mode, param, _gridCache);
         break;
       case 9:
-        fk=AbstractFKFunction::create(AbstractFKFunction::Love, mode, param, _gridCache);
+        fk=AbstractFKFunction::create(mode, param, _gridCache);
         break;
       default:
         return;
@@ -892,10 +906,13 @@ void FKTimeWindows::showFKPeaks()
     }
     break;
   }
+  fk->setCrossSpectrum(_crossSpectrum);
   fk->setMaximumSlowness(param->maximumSlowness());
   fk->setMaximumWavenumber(param->effectiveGridSize(_timeWindows->frequency().center()));
   fk->setTotalLimits(_timeWindows->frequency().squaredOmega());
 
+
+  LayerLocker ll(_fkPeaks);
   Curve<Point2D>& plot=static_cast<PlotLine2D *>(_fkPeaks->line(0))->curve();
   plot.clear();
   double gridSize=param->effectiveGridSize(_timeWindows->frequency().center());
@@ -923,10 +940,13 @@ void FKTimeWindows::showFKPeaks()
   for(it=gs->begin(); it!=gs->end(); ++it) {
     Point kell=(*it)->pos();
     plot.append(kell);
-    App::log(tr("Peak at k=(%1, %2), e=%3 deg. v=%4\n")
+    double v=2.0*M_PI*_timeWindows->frequency().center()/Point2D(kell.x(), kell.y()).length();
+    App::log(tr("Peak at k=(%1, %2), e=%3 deg. v=%4, s=%5, az=%6 deg.\n")
              .arg(kell.x()).arg(kell.y())
              .arg(Angle::radiansToDegrees(kell.z()))
-             .arg(2.0*M_PI*_timeWindows->frequency().center()/Point2D(kell.x(), kell.y()).length()));
+             .arg(v)
+             .arg(1.0/v)
+             .arg(Angle::mathToGeographic(kell.azimuth())));
   }
   delete gs;
 
