@@ -37,6 +37,8 @@ public:
   HVToolWidget(QWidget * parent);
 
   bool setSubPool(SubSignalPool * subPool);
+private slots:
+  void frequencyWindowRejection();
 protected:
   virtual AbstractResultSheet * createResultSheet();
   virtual AbstractSummary * createSummary();
