@@ -71,20 +71,28 @@ bool CurveReader::setOptions(int& argc, char ** argv)
       if(arg=="-resample") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=Resample;
-        _nResample=atoi(argv[i]);
+        _nResample=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-resample-ext") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=ResampleExtrapole;
-        _nResample=atoi(argv[i]);
+        _nResample=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-cut") {
         _mode=Cut;
       } else if(arg=="-cut-int") {
         _mode=CutInterpole;
       } else if(arg=="-column") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _columnIndex=atoi(argv[i]);
+        _columnIndex=CoreApplication::toInt(i, i-1, argv);
         if(_columnIndex<1) {
-          App::log(tr("gpcurve: column index must be greater or equal to 1 (%1)\n").arg(argv[i]) );
+          App::log(tr("gpcurve: column index must be greater or equal to 1 (%1)\n").arg(argv[i]));
+          return false;
+        }
+      } else if(arg=="-smooth") {
+        _mode=Smooth;
+        CoreApplication::checkOptionArg(i, argc, argv);
+        _dx=CoreApplication::toDouble(i, i-1, argv);
+        if(_dx<0.0) {
+          App::log(tr("gpcurve: window size cannot be negative\n"));
           return false;
         }
       } else if(arg=="-swap") {
@@ -112,7 +120,7 @@ bool CurveReader::setOptions(int& argc, char ** argv)
       } else if(arg=="-value") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=FunctionValue;
-        _xValue=atof(argv[i]);
+        _xValue=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-merge-replace") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=MergeReplace;
@@ -136,15 +144,15 @@ bool CurveReader::setOptions(int& argc, char ** argv)
       } else if(arg=="-pow") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=Power;
-        _logBase=atof(argv[i]);
+        _logBase=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-log") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=Log;
-        _logBase=atof(argv[i]);
+        _logBase=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-multiply") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=Multiply;
-        _multiplyFactor=atof(argv[i]);
+        _multiplyFactor=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-function") {
         CoreApplication::checkOptionArg(i, argc, argv);
         _mode=ParseFunction;
@@ -160,29 +168,29 @@ bool CurveReader::setOptions(int& argc, char ** argv)
         _misfitDof=atoi(argv[i] );
       } else if(arg=="-min") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _minX=atof(argv[i]);
+        _minX=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-max") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _maxX=atof(argv[i]);
+        _maxX=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-dx") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _dx=atof(argv[i]);
+        _dx=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-angles") {
         _mode=Angles;
       } else if(arg=="-split") {
         _mode=Split;
       } else if(arg=="-max-x") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _splitMaxX=atof(argv[i]);
+        _splitMaxX=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-max-err") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _splitMaxErr=atof(argv[i]);
+        _splitMaxErr=CoreApplication::toDouble(i, i-1, argv);
       } else if(arg=="-min-count") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _splitMinCount=atoi(argv[i]);
+        _splitMinCount=CoreApplication::toInt(i, i-1, argv);
       } else if(arg=="-extrapolate") {
         CoreApplication::checkOptionArg(i, argc, argv);
-        _nResample=atoi(argv[i]);
+        _nResample=CoreApplication::toInt(i, i-1, argv);
         _mode=Extrapolate;
       } else if(arg=="-sampling") {
         CoreApplication::checkOptionArg(i, argc, argv);
@@ -241,13 +249,14 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case Resample:
   case Cut:
   case CutInterpole:
+  case Smooth:
     curve.sort();
     // const must be passed to Curve to avoid reset of sorted flag
     if(_minX==-std::numeric_limits<double>::infinity()) {
-      _minX=const_cast<const Curve<PointND>&>(curve).first().x();
+      _minX=const_cast<const Curve<PointND>&>(curve).firstX();
     }
     if(_maxX==std::numeric_limits<double>::infinity()) {
-      _maxX=const_cast<const Curve<PointND>&>(curve).last().x();
+      _maxX=const_cast<const Curve<PointND>&>(curve).lastX();
     }
     break;
   default:
@@ -265,8 +274,8 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case Resample: {
       // _minX and _maxX can be set by the user to any value, not linked to actual range of curve
       // To avoid extrapolation we want to keep the original range of the curve
-      double minOriginalX=const_cast<const Curve<PointND>&>(curve).first().x();
-      double maxOriginalX=const_cast<const Curve<PointND>&>(curve).last().x();
+      double minOriginalX=const_cast<const Curve<PointND>&>(curve).firstX();
+      double maxOriginalX=const_cast<const Curve<PointND>&>(curve).lastX();
       curve.resample(_nResample, _minX, _maxX, _samplingType);
       curve.cut(minOriginalX, maxOriginalX, LinearScale);
       sOut << curve.toString();
@@ -276,6 +285,31 @@ bool CurveReader::exec(Curve<PointND>& curve)
     curve.cut(_minX, _maxX, LinearScale);
     sOut << curve.toString();
     break;
+  case Smooth: {
+      SamplingParameters s;
+      s.setRange(_minX, _maxX);
+      switch(_samplingType) {
+      case LogScale:
+        s.setScaleType(SamplingParameters::Log);
+        break;
+      case InversedScale:
+        s.setScaleType(SamplingParameters::Inversed);
+        break;
+      default:
+        s.setScaleType(SamplingParameters::Linear);
+        break;
+      }
+      s.setCount(_nResample);
+      SmoothingParameters p;
+      p.setMethod(SmoothingParameters::Function);     // TODO make it customizable
+      p.setWidthType(SmoothingParameters::Constant);  // TODO make it customizable
+      p.setWidth(_dx);
+      p.setScaleType(SamplingParameters::Log);        // TODO make it customizable
+      p.windowFunction().setShape(WindowFunctionParameters::Cosine);  // TODO make it customizable
+      curve.smooth(s, p, &pointOptions);              // Only required for Savitzky
+      sOut << curve.toString();
+    }
+    break;
   case CutInterpole:
     curve.cut(_minX, _maxX, Interpole);
     sOut << curve.toString();
@@ -290,7 +324,7 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case MinValue: {
       int i=curve.minimumY(0, &pointOptions);
       ASSERT(i>-1);
-      sOut << curve.at(i).toString(20) << "\n";
+      sOut << curve.constAt(i).toString(20) << "\n";
     }
     break;
   case MaxIndex:
@@ -299,15 +333,15 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case MaxValue: {
       int i=curve.maximumY(0, &pointOptions);
       ASSERT(i>-1);
-      sOut << curve.at(i).toString(20) << "\n";
+      sOut << curve.constAt(i).toString(20) << "\n";
     }
     break;
   case LocalMin: {
       int i=curve.localMinimumY(0, &pointOptions);
       while(i>=0) {
         sOut << i << " "
-             << curve.at(i).y(&pointOptions) << " "
-             << 2.0*curve.at(i).y(&pointOptions)/(curve.at(i-1).y(&pointOptions)+curve.at(i+1).y(&pointOptions))<< ::endl;
+             << curve.y(i, &pointOptions) << " "
+             << 2.0*curve.y(i, &pointOptions)/(curve.y(i-1, &pointOptions)+curve.y(i+1, &pointOptions)) << ::endl;
         i=curve.localMinimumY(i, &pointOptions);
       }
     }
@@ -323,7 +357,7 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case LocalMinValue: {
       int i=curve.localMinimumY(0, &pointOptions);
       while(i>=0) {
-        sOut << curve.at(i).y(&pointOptions) << ::endl;
+        sOut << curve.y(i, &pointOptions) << ::endl;
         i=curve.localMinimumY(i, &pointOptions);
       }
     }
@@ -332,8 +366,8 @@ bool CurveReader::exec(Curve<PointND>& curve)
       int i=curve.localMaximumY(0, &pointOptions);
       while(i>=0) {
         sOut << i << " "
-             << curve.at(i).y(&pointOptions) << " "
-             << 2.0*curve.at(i).y(&pointOptions)/(curve.at(i-1).y(&pointOptions)+curve.at(i+1).y(&pointOptions))<< ::endl;
+             << curve.y(i, &pointOptions) << " "
+             << 2.0*curve.y(i, &pointOptions)/(curve.y(i-1, &pointOptions)+curve.y(i+1, &pointOptions)) << ::endl;
         i=curve.localMaximumY(i, &pointOptions);
       }
     }
@@ -349,22 +383,22 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case LocalMaxValue: {
       int i=curve.localMaximumY(0, &pointOptions);
       while(i>=0) {
-        sOut << curve.at(i).y(&pointOptions) << ::endl;
+        sOut << curve.y(i, &pointOptions) << ::endl;
         i=curve.localMaximumY(i, &pointOptions);
       }
     }
     break;
   case FunctionValue:
     curve.sort();
-    sOut << curve.at(_xValue).toString(20) << "\n";
+    sOut << curve.interpole(_xValue).toString(20) << "\n";
     break;
   case MergeReplace: {
       loadReference();
       for(int i=0;i<curve.count();i++) {
-        const PointND& p=curve.at(i);
+        const PointND& p=curve.constAt(i);
         int index=_referenceCurve.indexAfter(p.x());
-        if(index<_referenceCurve.count() && _referenceCurve.at(index).x()==p.x()) {
-          sOut << _referenceCurve.at(index).toString(20) << "\n";
+        if(index<_referenceCurve.count() && _referenceCurve.x(index)==p.x()) {
+          sOut << _referenceCurve.constAt(index).toString(20) << "\n";
         } else {
           sOut << p.toString(20) << "\n";
         }
@@ -374,8 +408,8 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case MergeInterpolate: {
       loadReference();
       for(int i=0;i<curve.count();i++) {
-        const PointND& p=curve.at(i);
-        sOut << p.toString(20) << " " << _referenceCurve.at(p.x()).toString(20) << "\n";
+        const PointND& p=curve.constAt(i);
+        sOut << p.toString(20) << " " << _referenceCurve.interpole(p.x()).toString(20) << "\n";
       }
     }
     break;
@@ -384,14 +418,14 @@ bool CurveReader::exec(Curve<PointND>& curve)
       for(int i=0;i<curves.count();i++) {
         Curve<PointND>& c=curves[i];
         // Filling holes in the curve sampling by re-interpolation
-        double min=c.at(c.minimumX()).x();
-        double max=c.at(c.maximumX()).x();
+        double min=c.x(c.minimumX());
+        double max=c.x(c.maximumX());
         double step=c.minimumStep(_samplingType);
         int n;
         if(_samplingType & LogScale) {
-          n=1+qRound(log(c.at(max).x()/c.at(min).x())/log(step));
+          n=1+qRound(log(c.interpole(max).x()/c.interpole(min).x())/log(step));
         } else {
-          n=1+qRound(c.at(max).x()-c.at(min).x()/step);
+          n=1+qRound(c.interpole(max).x()-c.interpole(min).x()/step);
         }
         c.resample(n, min, max, _samplingType);
         sOut << "# split " << i << ": " << c.count() << " samples\n";
@@ -402,13 +436,13 @@ bool CurveReader::exec(Curve<PointND>& curve)
   case MinMax: {
       if(_minCurve.count()==0) {
         int ns=curve.count();
-        int nd=curve.first().count();
+        int nd=curve.constAt(0).count();
         _minCurve.resize(ns);
         _maxCurve.resize(ns);
         for(int is=0; is<ns; is++) {
-          const PointND& p=curve.at(is);
-          PointND& pmin=_minCurve[is];
-          PointND& pmax=_maxCurve[is];
+          const PointND& p=curve.constAt(is);
+          PointND& pmin=_minCurve.at(is);
+          PointND& pmax=_maxCurve.at(is);
           pmin.setX(p.x());
           pmax.setX(p.x());
           for(int id=0; id<nd; id++) {
@@ -418,15 +452,15 @@ bool CurveReader::exec(Curve<PointND>& curve)
         }
       } else {
         int ns=_minCurve.count();
-        int nd=_minCurve.first().count();
-        if(ns!=curve.count() || nd!=curve.first().count()) {
+        int nd=_minCurve.constAt(0).count();
+        if(ns!=curve.count() || nd!=curve.constAt(0).count()) {
           App::log(tr("Curves must have exactly the same sampling.\n") );
           return false;
         }
         for(int is=0; is<ns; is++) {
-          const PointND& p=curve.at(is);
-          PointND& pmin=_minCurve[is];
-          PointND& pmax=_maxCurve[is];
+          const PointND& p=curve.constAt(is);
+          PointND& pmin=_minCurve.at(is);
+          PointND& pmax=_maxCurve.at(is);
           for(int id=0; id<nd; id++) {
             double v=p.y(id);
             if(v<pmin.y(id)) pmin.setY(id, v);
@@ -481,8 +515,8 @@ bool CurveReader::exec(Curve<PointND>& curve)
       double sum=0.0;
       int nData=0, nValues =0;
       for(int i=0;i<curve.count();i++) {
-        const PointND& p=curve.at(i);
-        PointND r=_referenceCurve.at(p.x());
+        const PointND& p=curve.constAt(i);
+        PointND r=_referenceCurve.interpole(p.x());
         RealStatisticalValue sv(r.y(0), r.y(1));
         RealValue v(p.y(0));
         sum += sv.misfit(nValues, nData, v, _misfitType, _misfitMin);
@@ -520,17 +554,17 @@ bool CurveReader::exec(Curve<PointND>& curve)
     sOut << curve.toString();
     break;
   case Log:
-    curve.yLog10(&pointOptions);
-    if(_logBase!=10.0) {
-      curve.yMultiply(1.0/log10(_logBase), &pointOptions);
+    curve.yLog(&pointOptions);
+    if(_logBase!=::exp(1.0)) {
+      curve.yMultiply(1.0/::log(_logBase), &pointOptions);
     }
     sOut << curve.toString();
     break;
   case Power:
-    if(_logBase!=10.0) {
-      curve.yMultiply(log10(_logBase), &pointOptions);
+    if(_logBase!=::exp(1.0)) {
+      curve.yMultiply(::log(_logBase), &pointOptions);
     }
-    curve.yExp10(&pointOptions);
+    curve.yExp(&pointOptions);
     sOut << curve.toString();
     break;
   case Extrapolate:
