@@ -145,12 +145,7 @@ void WaranWidget::initResultGraph(AxisWindow * g, IrregularGrid2DPlot *& gridLay
 {
   TRACE;
   g->xAxis()->setRange(0.1,100.0);
-  g->xAxis()->setTitle(tr("Frequency (Hz)"));
-  g->xAxis()->setTitleInversedScale(tr("Period (s)"));
-  g->xAxis()->setScaleType(Scale::Log);
-  g->xAxis()->setAutoTicks(false); 
-  g->xAxis()->setMajorTicks(2.0);
-  g->xAxis()->setMinorTicks(0.5);
+  g->xAxis()->setFrequency();
   g->yAxis()->setRange(0.0,5.0);
   g->yAxis()->setTitle(tr("Slowness (s/m)"));
   g->yAxis()->setTitleInversedScale(tr("Velocity (m/s)"));
@@ -450,7 +445,7 @@ void WaranWidget::updateArrayMap()
   Curve<NamedPoint>& c=l->curve();
   c.resize(n);
   for(int i=0; i<_array.count(); i++) {
-    NamedPoint& p=c[ i ];
+    NamedPoint& p=c.constXAt(i);
     const StationSignals * stat=_array.at(i);
     p=stat->coordinates();
     p.setZ(0.0); // Avoid error bars plotted from Z components
@@ -483,8 +478,8 @@ void WaranWidget::updateCoArrayMap()
   Curve<NamedPoint>& c=l->curve();
   c.resize(nCouples);
   for(int i=0;i < nCouples;i++ ) {
-    NamedPoint& p=c[ i ];
-    const StationPair& couple=stationCouples[ i ];
+    NamedPoint& p=c.at(i);
+    const StationPair& couple=stationCouples[i];
     /*const Point& p1=couple.station1()->coordinates();
     const Point& p2=couple.station2()->coordinates();
     double dx=p1.x() - p2.x();
